!------------------------------------------------------------
      MODULE MappaSA
        USE connect_mod, only: bond_type
        USE atom_basic, only: atom_type
             integer              :: nats_mappa
             integer              :: MaxAtoms_Mappa=2500
             integer              :: MaxBonds_Mappa=9800
             integer, allocatable :: iconn_mappa(:)
             real, allocatable    :: xyzs_mappa(:,:)
             integer, allocatable :: nz_mappa(:) 
             real, allocatable    :: bfac_mappa(:)
             character(len=16),allocatable :: aLab_mappa(:)
             type(atom_type), dimension(:), allocatable :: atm_dm
             type(bond_type), dimension(:), allocatable :: legm_dm
      END MODULE MappaSA
!------------------------------------------------------------
      subroutine AllocaMappaSA(iAction)
      USE MappaSa

      integer ierv(100), iAction, nerr, ierr
!
!-- Allocate / Deallocate arrays  ( iAction = 1 / 2 )
!
      ierv=0
      if (iAction == 1) then
          if (.not. allocated(iconn_mappa)) then
             nerr=1
             allocate (iconn_mappa(1:MaxBonds_Mappa), stat = ierv(nerr))
             nerr = nerr + 1
             allocate (xyzs_mappa(1:5,1:MaxAtoms_Mappa),stat=ierv(nerr))
             nerr = nerr + 1
             allocate (nz_mappa(1:MaxAtoms_Mappa), stat = ierv(nerr))
             nerr = nerr + 1
             allocate (bfac_mappa(1:MaxAtoms_Mappa), stat = ierv(nerr))
             nerr = nerr + 1
             allocate (aLab_mappa(1:MaxAtoms_Mappa), stat = ierv(nerr))
          endif
          ierr = sum(ierv(1:nerr))
          if (ierr.ne.0) then
              write(0,*) 'MappaSA Allocation Error'
              stop
          endif
      else
          if (allocated(iconn_mappa)) then
              deallocate (iconn_mappa,xyzs_mappa,nz_mappa,bfac_mappa,aLab_mappa,  stat = ierv(1))
              ierr = sum(ierv(:))
              if (ierr.ne.0) then
                  write(0,*) 'MappaSA DeAllocation Error'
                  stop
              endif
          endif
         endif

      return
      end
!------------------------------------------------------------
MODULE directmod

implicit none

type tors_type
   integer               :: rcod
   integer, dimension(4) :: ators
   real                  :: torsval
   real, dimension(2)    :: trbound
end type

CONTAINS


   subroutine check_tors(torsion,ierr)
   USE Catena2
   USE Molcom, only:jscreen
   type(tors_type), dimension(:) :: torsion
   integer, intent(out)          :: ierr
!
   if(jscreen > 0) call AllocaCatena2(1)
   call torsion_to_varz(torsion)
   call check_tors_grafica(ierr)
!
   end subroutine check_tors
   
!---------------------------------------------------------------------------

   subroutine torsion_to_varz(torsion)
   USE var_samod
   implicit none
   type(tors_type), dimension(:), intent(in) :: torsion
   integer                                   :: i,numt
!  integer                                   :: numtorsion
!  numtorsion= numero di torsioni
!  numt= numero di torsioni da affinare
!
   numt=0
   nvarz(1) = 0
   nvarz(2) = 0
   nvarz(3) = 0
   numtorsion = size(torsion)
   numdeg=0
   nmult3(:)=0
   do i=1,numtorsion
      if(torsion(i)%rcod.eq.1) then
        numt=numt+1
        nvarz3(1,numt,1) = torsion(i)%ators(1)
        nvarz3(2,numt,1) = torsion(i)%ators(2)
        nvarz3(3,numt,1) = torsion(i)%ators(3)
        nvarz3(4,numt,1) = torsion(i)%ators(4)
        torstol(numt)=torsion(i)%trbound(2)
      endif
   enddo
   nvarz(3) = numt
   nmult3(1:numt)=1
   numdeg=nvarz(3)
!
   end subroutine torsion_to_varz

!---------------------------------------------------------------------------

END module directmod
   subroutine get_tors_info(filenam, ifExtMap, ier, natsgrc)
!
!  Importa struttura, disegna struttura sull'area grafica, 
!  cerca DOF interni nella struttura
   USE Riflessi
   USE Conteggi, only:npunti
   USE MappaSA
   USE General,only : cell,Msg,ERR_WINDOW,neutro,celem
   USE unit_cell
   USE directmod
   USE rotationmod
   USE progtype
   USE Atomi
   USE drawatoms, only:importa0, draw_xyzs
   USE connect_mod
   USE atom_type_util
   USE variables
   USE fragmentmod
   USE var_samod, only:numdeg,i_get_tors_info
   USE molcom, only:jscreen
   USE arrayutil
   USE sinterface, only: fillatom

   character(len=*), intent(in)                     :: filenam
   character(len=1)                                 :: dire
   integer                                          :: ier
   logical                                          :: kpr = .true.
   integer, allocatable, dimension(:)                  :: vremh
   type(container_type), allocatable, dimension(:) :: connect
   type(bond_type), allocatable, dimension(:)       :: leg
   type(atom_type), allocatable, dimension(:)       :: atom
   type(rotation_type), dimension(:), allocatable   :: rotat
   type(tors_type), dimension(:), allocatable       :: torsion
   type(fragment_type), dimension(:), allocatable   :: fragm
   integer                                          :: numt,numf
   integer                                          :: i,iz,j,l
   integer                                          :: istop,nExitCode
   integer                                          :: natsgroup,labmapgroup,natsmapgroup,labgroup
   integer                                          :: ifExtMap, natsgrc
   integer                                          :: npermtot_sav,npermtot
   real                                             :: res_min,res_used,tetmax
   real,dimension(3)                                :: x1,x11,x2
   type(cell_type)                                  :: cellt
   integer notra,ntrasl,nauto,mauto
   common/solo/notra,ntrasl,nauto,mauto(2)
!  real rmz, rmiz, thmax_mod
   real rmz, rmiz
   common /matr/rmz(3,3),rmiz(3,3)
   common/groupm/natsgroup,labgroup(50),natsmapgroup,labmapgroup(50)
   integer imod_ext,i_four_map,natr
   common/mode/imod_ext,i_four_map,natr,fnamee,fnamep
   common/per_grafica/res_used,tetmax,npermtot_sav,npermtot,res_min
   character fnamee(10)*80,fnamep*80
!  integer :: nleg, inv, iln, lsn, lix37, ljxx, lkxx, llxx, nrefc
   integer :: nleg, inv, iln, lsn, lix37, ljxx, lkxx, llxx
!
    write(6,1)
 1    format(///,1h ,78('+'),//,    &
      47h expo2005  : Simulated Annealing routine       ,17x,    &
      14hRelease  02.01,//,1h ,78('+'),/)
!
   nauto=1
   istop=0
   i_get_tors_info=1

!  Importa il file in xyzs e disegna
   ier = 0
   aLab(:)=' '

   call importa0(filenam,atm,lconn,ier,.false.)    ! ricorda di gestire errore su import file

!CNEW
!---porto in cella
   inv=1           ! calcolo di rmiz
   call Orto_new(cell(1),cell(2),cell(3),cell(4),cell(5),cell(6),inv)
!!!corr - mi pare non serve   code = code_fragfile(filenam)
   inv=0           ! calcolo di rmz
   call Orto_new(cell(1),cell(2),cell(3),cell(4),cell(5),cell(6),inv)

   do i=1,nats
      do j=1,3
         x1(j) = xyzs(j,i)
      end do
      call molt1_rot(rmz, x1, x11)
!
!----porto la struttura importata nella nuova cella---------
!
      call molt1_rot(rmiz, x11, x2)
      do j=1,3
         xyzs(j,i)=x2(j)
      end do
   end do

!--elimina gli idrogeni se presenti e la struttura non da neutroni
   call xyz_to_atom(atom,xyzs(:3,:nats),bfac,occr=xyzs(5,:),occh=occpch,nz=nz,lab=aLab,inte=xyzs(4,:),elem=celem)
   if (neutro==0) then
    allocate(vremh(nats))
    vremh(:) = 0
    l=0
    do i=1,nats
     dire=aLab(i)
     call lcase(dire)
     if(dire=='h') then
       l=l+1
       vremh(i) = 1
     endif
    end do
    if(l>0) then
!corr      call remove_atoms_from_list(atom,vremh,1,icon=iconn) 
      call remove_atoms_from_list(atom,vremh,1,legm=lconn)
      nats = size(atom)
      call fillatom(atom)
!corr      call Molplo()
      if (jscreen > 0) call draw_xyzs()
    endif
   end if                        ! fine if su neutro
   
   if (ier /= 0) return
   iz = 1
   call EnableSingleIteration(iz)
!
!  Carica il modello in atom
   if (kpr) write(6,*) ' '

!  Rielabora la connettività
   cellt = set_cell_type(cell)
   call iconn_to_connect(nats,iconn,connect)
   call connect_to_leg(connect,atom,cellt%get_g(),leg,nleg)

!  calcola il num. di frammenti 
   call get_fragments_from_leg(atom(:nats),cellt,leg,connect,numf,fragm)
   if(numf>=3) istop=1
   if(numf==2) then
      if(fragm(1)%nat>1.and.fragm(2)%nat>1) istop=1

!--controlla se il modello importato contiene un atomo isolato

      if(fragm(1)%nat==1.or.fragm(2)%nat==1) then
         natsgroup=1
         natsmapgroup=1
         labmapgroup(1)=nats+1
         if(fragm(1)%nat==1) labgroup(1)=fragm(1)%pos(1) 
         if(fragm(2)%nat==1) labgroup(1)=fragm(2)%pos(1) 
      endif
   endif
   if(istop.eq.1) then
     Msg='More than 2 fragments are present.'//char(10)//  &
         ' The procedure cannot be applied'
     call MsgWinErr('ERROR',Msg,ERR_WINDOW,nExitCode)
     ier=-1
     return     
   endif
   deallocate(fragm)
!
!  Estrai tutte le rotazioni dal modello strutturale
   call get_rotation_from_leg(atom,connect,leg,cellt,numt,rotat,.true.)
!
   allocate(torsion(numt))
!  Assegna una torsione alla rotazione
   if (numt > 0) then
!     if (kpr) call print_rotation(rotat,1)
!corr      ortom = orthomatrix(cell)
!corr      ortoi = orthomatrixi(cell)
      do i=1,numt
         call set_torsion_for_rotat(connect,rotat(i))
         rotat(i)%torsval = torsion_value_for_rotat(rotat(i),atom,cellt)
      enddo
!
!     Passa le informazioni nella variabile torsion
!     allocate(torsion(numt))
      do i=1,numt
         torsion(i)%rcod = rotat(i)%rcod
         torsion(i)%ators = rotat(i)%ators
         torsion(i)%torsval = rotat(i)%torsval
         torsion(i)%trbound = rotat(i)%trbound
      enddo
   endif
!
! gestisce tutto sulla Mappa Fourier
!
   numdeg=numt
 
!deve stare qui perche' devo prima cambiare nats se tolgo gli idrogeni
   call Mappa_Four(ifExtMap,ier)  
   npermtot=nats_mappa*nats             ! numero di pernutazioni totali
   npermtot_sav=npermtot
   if(natsgroup>0) then
      npermtot=(nats_mappa-1)*(nats-1)
      npermtot_sav=npermtot
   endif

   call AllocaConteggi(1)
   call LeggiRiflessi

! per riempire vat()
   iln=37
   call srlin (iln,lsn,lix37,ljxx,lkxx,llxx)
   if (lix37.gt.0) then
       call ysfl37(npunti)
   else
       write(Msg,8)
       call MsgWinErr('ERROR',Msg,ERR_WINDOW,nExitCode)
       ier=-1
       return
   endif
 8 format('No counts in direct acces file')

   call calcola_range(1)

!! call evaluate_res_sadm(nrefc,thmax_mod,ier) 
!  nrefl=nrefc
!
!  Trasferisce informazioni al C
   natsgrc = natsgroup
   call fill_info(numt,torsion,atom,nats)   ! mi pare che atom non serva
!
   end subroutine get_tors_info

!---------------------------------------------------------------------------
   subroutine Mappa_Four(ifExtMap,ier)
   USE atom_type_util
   USE connect_mod
   USE MappaSA
   USE importexport
   USE molcom, only:kscreen
   USE General, only: cell,celem,spacegr
   USE unit_cell
   USE gen_frm
   USE errormod

   common /external_file/mapfile,fragfile
   character*256 mapfile, fragfile
   type(bond_type), dimension(:), allocatable :: legm
   type(atom_type), dimension(:), allocatable :: atom
   integer                                    :: ifExtMap,ier
   integer                                    :: ktype
   integer                                    :: i,j   !!!,lsn,lixx,ljxx,lkxx,llxx,n15v
   type(cell_type)                            :: cellt
   type(error_type)                           :: err

! se ifExtMap = 0 non ho fornito il nome del file
 
   if (.not. allocated(iconn_mappa)) call AllocaMappaSA(1)

   ier = 0
   if(ifExtMap.ne.0) then   ! ho dato una lista esterna di atomi
      ktype = code_fragfilen(mapfile)
      call import_structure(mapfile,ktype,atom,legm,celem,cell,spacegr,.false.,.true.,.true.,err)
      if (err%signal) ier = 1
      if(ier==0) then
         nats_mappa=size(atom)
         do i=1,nats_mappa
           do j=1,3
              xyzs_mappa(j,i) = atom(i)%xc(j)
           end do
           xyzs_mappa(4,i) = atom(i)%inte
           xyzs_mappa(5,i) = atom(i)%och
           bfac_mappa(i)=atom(i)%biso
           nz_mappa(i)=atom(i)%get_nz()
           aLab_mappa(i)=atom(i)%lab
         end do
!
         call bond_to_iconn(legm,iconn_mappa)
!
         call copy_atoms(atm_dm,atom)
         cellt = set_cell_type(cell)
         call iconn_to_legm(iconn_mappa,legm_dm,atm_dm,cellt%get_g())
      endif
   endif

   if(kscreen>0) then
   call modificomappa(ifExtMap) ! fa il taglio sui picchi della mappa. Nuovo nats_mappa

!riporta in cella il baricentro dei picchi Fourier
   if(ifExtMap==0) call xyz_to_atom(atom,xyzs_mappa(:3,:nats_mappa),bf=bfac_mappa, &
                                    nz=nz_mappa,lab=aLab_mappa,elem=celem)

    do i=1,nats_mappa
      call translate_in_cell(atom(i:i))
       do j=1,3
          xyzs_mappa(j,i) = atom(i)%xc(j)
       end do
    end do
   endif

   end subroutine Mappa_four
!---------------------------------------------------------------------------
   subroutine get_tors_info_nograf(ier)
!
!  Cerca DOF interni nella struttura quando non c'e' la grafica.
   USE General,only : cell,celem
   USE unit_cell
   USE directmod
   USE rotationmod
   USE progtype
   USE Atomi
   USE connect_mod
   USE atom_type_util
   USE variables
   USE fragmentmod
   USE arrayutil

   integer                                          :: ier
   type(container_type), dimension(:), allocatable :: connect
   type(bond_type), dimension(:), allocatable       :: leg
   logical                                          :: kpr = .true.
   type(atom_type), allocatable, dimension(:)       :: atom
   type(rotation_type), dimension(:), allocatable   :: rotat
   type(tors_type), dimension(:), allocatable       :: torsion
   integer                                          :: numt
   integer                                          :: i
   integer                                          :: nleg
   type(cell_type)                                  :: cellt
!
   ier = 0
!  Carica il modello in atom
   allocate(atom(nats))
   call xyz_to_atom(atom,xyzs(:3,:nats),bfac,occr=xyzs(5,:),occh=occpch,nz=nz,lab=aLab,elem=celem)
   cellt = set_cell_type(cell)
   if (kpr) write(6,*) ' '

!  Rielabora la connettività
   call iconn_to_connect(nats,iconn,connect)
   call connect_to_leg(connect,atom,cellt%get_g(),leg,nleg)
!
!  Estrai tutte le rotazioni dal modello strutturale
!corr   call get_rotation_from_leg(connect,leg,atom,numt,rotat)
   call get_rotation_from_leg(atom,connect,leg,cellt,numt,rotat,.true.)
!
   allocate(torsion(numt))
!  Assegna una torsione alla rotazione
   if (numt > 0) then
!corr      ortom = orthomatrix(cell)
!corr      ortoi = orthomatrixi(cell)
      do i=1,numt
         call set_torsion_for_rotat(connect,rotat(i))
         rotat(i)%torsval = torsion_value_for_rotat(rotat(i),atom,cellt)
      enddo
!
!     Passa le informazioni nella variabile torsion
!     allocate(torsion(numt))
      do i=1,numt
         torsion(i)%rcod = rotat(i)%rcod
         torsion(i)%ators = rotat(i)%ators
         torsion(i)%torsval = rotat(i)%torsval
         torsion(i)%trbound = rotat(i)%trbound
      enddo
   endif
   call torsion_to_varz(torsion)
!
   end subroutine get_tors_info_nograf
!---------------------------------------------------------------------------

   integer function passa_tors(numt)
!
   USE directmod
!
   type(tors_type), dimension(:), allocatable :: torsion
   integer                                    :: numt
   integer                                    :: ierr
!
   passa_tors = 0
   allocate(torsion(numt))
   call get_tors(torsion, numt)
   call check_tors(torsion, ierr)
   passa_tors = ierr
!
   end function passa_tors
   
!---------------------------------------------------------------------------
   subroutine check_file_type(filename,ier)
   USE errormod
   USE gen_frm
   !USE importexport, only:code_fragfilen
   character(len=*), intent(in) :: filename
   integer, intent(out)         :: ier
   integer                      :: code
   type(error_type)             :: err
!
   ier = 0
   code = code_fragfilen(filename)
   if (code == 0) then
       call err%set('Unrecognized file type')
       call err%print()
       ier = -1                 !!mr 051007
   endif
!
   end subroutine check_file_type
!---------------------------------------------------------------------------
      subroutine prelim_sadm(resmin, resmax)

      USE Atomi
      USE General, only:nrefl,dtor,rtod,alambda
      USE var_samod, only:i_SA,i_get_tors_info
      USE Riflessi, only:TThet
!corr USE var_samod, only:i_SA,ifLabelsSav,i_get_tors_info

      common /external_file/mapfile,fragfile
      character*256 mapfile, fragfile

!corr      integer irho
      real thmin,thmax,thstep
!corr      common/genob/irho,thetamin
      common/genob2/thmin,thmax,thstep
      common /lista15v/ store15v(30000),n15v
      real                                       :: resmin, resmax, store15v
      integer                                    :: lsn,lixx,ljxx,lkxx,llxx,n15v      
!
      i_SA=1
      i_get_tors_info=0

      resmin=1./(2.0*(sin(TThet(nrefl)*0.5*dtor)/alambda))
      resmax=1./(2.0*(sin(TThet(1)*0.5*dtor)/alambda))
!
!se non ho fatto la Fourier o ho usato la direttiva extmap in input non chiama la save_map_from_DM
      call srlin (15,lsn,lixx,ljxx,lkxx,llxx)
      if (lixx.lt.0.or.n15v.lt.0) return
      write(0,*)'MAPFILE='//trim(mapfile)//'FINE'
      if(mapfile(1:1)/=char(0)) return
!     if(nome2(1:1)/=char(0)) return
!  salva la mappa Fourier in xyzs_mappa
      call save_map_from_DM()
!
      return
      end subroutine prelim_sadm
! ----------------------------------------------------------------------
   subroutine finish_sadm(ifExtMap, mapfile, fragfile,ier)
!
   USE General, only:nrefl,Msg,ERR_WINDOW
   USE MappaSA, only:nats_mappa,atm_dm,legm_dm
   USE Atomi, only:nats
   USE directmod
   USE var_samod, only: i_SA,init_time_sa
   USE GREXPOMOD
   USE arrayutil
   USE connect_mod
   USE exportfiles
   USE atom_type_util

   real res_min,res_used,tetmax,thmin, thmax, thstep
   integer npermtot_sav,npermtot
   character(len=*) mapfile, fragfile
   common /conteggisav/npuntisav,nrefsav
!corr   common/genob/irho,thetamin
   common/genob2/thmin,thmax,thstep
   common/atomod/natsmod,natstot,natsold_fig,natstott
   common/per_grafica/res_used,tetmax,npermtot_sav,npermtot,res_min
   common/groupm/natsgroup,labgroup(50),natsmapgroup,labmapgroup(50)
   common /lista15v/ store15v(30000),n15v
   real                                       :: end_time, store15v
   character(len = 80)                        :: ext
   integer                                    :: natsgroup,labmapgroup,natsmapgroup,labgroup
   integer                                    :: natsmod,natstot,natsold_fig,natstott
   integer                                    :: pos,pos1 
   integer                                    :: npuntisav,nrefsav
   integer                                    :: lsn,lixx,ljxx,lkxx,llxx,n15v
   integer                                    :: ifExtMap,nExitCode,ier

!
!Se non ho nessuma mappa Fourier

   if ((mapfile(1:1)==char(0)).and.ifExtMap==0) then
      call srlin (15,lsn,lixx,ljxx,lkxx,llxx)
      if (lixx.lt.0.or.n15v.lt.0) then
          write(Msg,1)
          call MsgWinErr('ERROR',Msg,ERR_WINDOW,nExitCode)
          ier=-1
          return
1         format('No peaks Fourier supplied')
      endif
   endif
!
!  se ho modificato npermtot devo modificare nats_mappa
   if(npermtot/=npermtot_sav) then
     if(natsgroup>0) then     ! ho un atomo isolato nel modello importato
!warn gfor       nats_mappa=nint(float(npermtot)/float(nats))+1.
       nats_mappa=nint(float(npermtot)/float(nats)+1.)
     else
       nats_mappa=nint(float(npermtot)/float(nats))
     endif
!ricalcolo il num. totale di atomi da passare alla sdirect
     natstot=nats+nats_mappa
   endif
!
!ricalcolo dopo le modifiche di grafica
!! call evaluate_res_sadm(nrefc,thmax_mod,ier) 
!! nrefl=nrefc
!! npunti = clocate(theta_int(:,7),thmax_mod)

!  estrae nome della struttura importata
   pos = index(fragfile,'.',back=.true.)
   pos1 = index(fragfile,'/',back=.true.)
   ext = fragfile(pos1+1:pos+3)
!
!corr   timeactive = .true.
!corr   call init_timestep(6,(/'SA step di temperatura     ', &
!corr                          'Cost function calculation  ', &
!corr                          'Moves                      ', &
!corr                          'Graphic                    ', &
!corr                          'Tempo/run                  ', &
!corr                          'Total time                 '/))
!
   call cpu_time(init_time_sa)

   call sdirect(1,ifExtMap,ext,ier)
   call export_files() ! esporta modello conn '%export'

   call cpu_time(end_time)
   write(6,*)'INIT_TIME SA,END_TIME SA,TOTAL_TIME SA',init_time_sa,end_time,end_time-init_time_sa

!corr   call reallocate(atm_dm,0)
   call clear_atoms(atm_dm)
   call clear_bonds(legm_dm)

!chiude plot di CF
   if (allocated(grplot)) deallocate(grplot)
   call gr_close_viewer()
   fortran_work = .false.

!corr   call end_timestep()
   i_SA=0
   call AllocaMappaSA(2)
!
!  se leng = 0 non ho fornito il nome del file
!  se ifExtMap = 0 non ho fornito il nome del file
!
   end subroutine finish_sadm
   
!---------------------------------------------------------------------------
   subroutine evaluate_res_sadm(nrefc,thmax_mod,ier) 
!
!  Data la risoluzione calcola il numero di riflessi e il 2theta massimo
!
!  res_used  = risoluzione in ingresso
!  thmax_mod = 2thmax modificato
!  nrefl     = numero di riflessi
!  nrefc     = numero di riflessi da passare al C

   USE Conteggi, only:theta_int,npunti
   USE General, only:nrefl,dtor,rtod,alambda
   USE Riflessi, only:TThet
   USE arrayutil
   implicit none
   real res_min,res_used,tetmax,thmin, thmax, thstep, thmax_mod
!  real resmin
   integer ier, i, nrefc,npermtot_sav,npermtot
!corr   common/genob/irho,thetamin
   common/genob2/thmin,thmax,thstep
   common/per_grafica/res_used,tetmax,npermtot_sav,npermtot,res_min
   common /conteggisav/npuntisav,nrefsav

   integer                                    :: npuntisav,nrefsav
   
!
!  Calcola il valore minimo consentito per res
!  resmin=1./(2.0*(sin(TThet(nrefl)*0.5*dtor)/alambda))
   ier=0
   nrefc = nrefl
!  if (res_used < resmin) then
   if (res_used <= res_min) then
      thmax_mod = tetmax
      nrefc=nrefsav
!     ier = 1   ! segnala che res e' fuori range
   else
       thmax_mod = 2*rtod*asin(alambda/(2*res_used))
!      do_1: do i=1,nrefl
       do_1: do i=1,nrefsav
          if(TThet(i).gt.thmax_mod) then
!mr          nrefl=i-1
             nrefc=i-1
             exit do_1
          endif
       end do  do_1
       ier = 0
   endif
      nrefl = nrefc
      tetmax=thmax_mod
      npunti = clocate(theta_int(:,7),thmax_mod)
!     write(6,*)'res=',res_used,nrefl,nrefc,thmax_mod,npunti

   end subroutine evaluate_res_sadm
!---------------------------------------------------------------------------
   subroutine Mappa_Ext(ier)
   USE MappaSA, only:nats_mappa
   USE Atomi, only:nats

   integer    :: ier
   real       :: res_min,res_used,tetmax
   integer    :: npermtot_sav,npermtot
   integer    :: natsgroup,labmapgroup,natsmapgroup,labgroup
   common/groupm/natsgroup,labgroup(50),natsmapgroup,labmapgroup(50)
   common/per_grafica/res_used,tetmax,npermtot_sav,npermtot,res_min
!
   call Mappa_Four(1,ier)  
   npermtot=nats_mappa*nats             ! numero di pernutazioni totali
   npermtot_sav=npermtot
   if(natsgroup>0) then
      npermtot=(nats_mappa-1)*(nats-1)
      npermtot_sav=npermtot
   endif

   end subroutine Mappa_Ext
!---------------------------------------------------------------------------
   subroutine sannelexeDM(ier)
   USE General, only:ifAutomatic    !, card
   USE var_samod, only:i_SA
   USE exportfiles
   USE molcom, only:kscreen

   common /external_file/mapfile,fragfile
   character*256 mapfile, fragfile
   integer        :: ifExtMap
   integer        :: ier

!  write(6,1)
!1 format(///,1h ,78('+'),//,47h expo2005  : Simulated Annealing routine       ,17x, &
!             14hRelease  02.01,//,1h ,78('+'),/)

   if (kscreen == 0.or.ifAutomatic==1) then
       i_SA=1
       ifExtMap=0
       mapfile=' '
       fragfile=' '
       call sdirect(0,ifExtMap,ier)
       i_SA=0
       if(ier<0) return
       call export_files()
   else
       mapfile= char(0)
       fragfile= char(0)
!  nome1(1:1) = char(0)
!  nome2(1:1) = char(0)
!  do i=1, 2
!     ipdir=ipdir+1
!     line=card(ipdir)
!     diret=line
!     call cutst(line,lenp,dire,lend)
!     call lcase(dire)
!
!     if (dire(1:6).eq.'extmod') then      ! legge modello esterno tipo .fra
!         if (lenp.gt.0) nome1 = line(1:Lung(line)) // char(0)
!         write(*, *) nome1
!     else if (dire(1:6).eq.'extmap') then   ! legge mappa Fourier esterna 
!         if (lenp.gt.0) nome2 = line(1:Lung(line)) // char(0) 
!         write(*, *) nome2
!     endif
!  end do
!      call aprisannelwinDM(nome1, nome2)
       call aprisannelwinDM(fragfile)
   endif

   end subroutine sannelexeDM
!---------------------------------------------------------------------------
   subroutine cancel_sadm()
!subrutine che gestisce le cose in SA+DM quando clicco il tasto QUIT
   USE MappaSA
   USE variables
   USE General, only:nrefl
   USE Conteggi, only:npunti
   USE drawatoms
   USE sinterface
   USE connect_mod
   USE var_samod, only:i_get_tors_info
   USE atom_type_util

   implicit none
   integer :: npuntisav, nrefsav
   common /conteggisav/npuntisav,nrefsav
!
   if(i_get_tors_info==1) then
      npunti = npuntisav
      nrefl = nrefsav
   endif
   if(allocated(atm_dm)) then
      call update_model(atm_dm,legm_dm,1)
      call clear_atoms(atm_dm)
      call clear_bonds(legm_dm)
!
      call draw_atoms(atm,lconn)
   endif
!
   end subroutine cancel_sadm
