!#define FFTW
! ----------------------------------------------------------------------
   !MODULE ParametriMappa  in moduli.f per dipendence invfour.f-fourier.f90
   !  integer              :: np(3)
   !  integer              :: nr3dSize
   !  real                 :: factr
   !END MODULE ParametriMappa
! ----------------------------------------------------------------------
module Mappa
     USE ParametriMappa
     real, allocatable :: xmap(:,:,:)
     real, allocatable :: enve(:,:,:)
     integer, allocatable :: nr3d(:)
     complex, allocatable   :: xmapC(:,:,:)
#ifdef FFTW
     integer*8            :: FORTH = 0,    BACK = 0
     include 'fftw3.f'
#endif
end module Mappa

module fourier_mod

implicit none

#ifdef FFTW

private :: MakeMap_old, MakeMap_fftw
interface MakeMap
    module procedure MakeMap_old, MakeMap_fftw
end interface

#else

private :: MakeMap_old
interface MakeMap
    module procedure MakeMap_old
end interface

#endif

private :: Search99_xmap, Search99_noxmap
interface Search99
    module procedure Search99_xmap, Search99_noxmap
end interface

private scalemap_xmap, scalemap_vect
interface scalemap
    module procedure scalemap_xmap, scalemap_vect
end interface 

contains

   subroutine MakeMap_old(khl,fop,iphi2,wtt,numbMax,jpatt,x,ee,eep)
!  jpatt = 0   -> Fourier
!  jpatt = 1   -> Patterson
!  X(nx,ny,nz)
   USE ParametriMappa
   USE General
   USE Symmetry
   USE Phasemodule, only:nex
   !use variables, only: cryst,dataset
   integer, dimension(3,nrefl), intent(in) :: khl
   real, dimension(nrefl), intent(in)      :: fop,wtt
   integer, dimension(nrefl), intent(in)   :: iphi2
   integer, intent(in)                     :: jpatt
   integer, intent(in)                     :: numbMax
   real, dimension(:), intent(out), optional :: ee,eep
   real x(0:np(1)-1,0:np(2)-1,0:np(3)-1)
   complex fh,fm,phasec
   integer lmod
   external lmod
   integer :: i,j,l,jn
   integer :: ih,ik,il
   integer :: nu, nv, nww
   real, dimension(numbMax) :: e,ep
   integer, dimension(3) :: i2
   real :: f000, fourcoef, phase
!
   nu=np(1)
   nv=np(2)
   nww=np(3)
   f000 = 0.0
   !f000 = get_f000(cryst(1)%at,cryst(1)%elem,cryst(1)%spg,dataset(1)%radtype)
!
   x(:,:,:) = 0.0
   if (jpatt == 0) then
       do i=1,numbMax
          fourcoef = fop(i)*wtt(i)
          phase=dtor*iphi2(i)
          ep(i)=fourcoef*sin(phase)
          e(i)=fourcoef*cos(phase)
       enddo
   else
       do i=1,numbMax
          ep(i)=0.0
          e(i) = fop(i)*wtt(i)
       enddo
   endif
!corr  100 continue
!**   write(lo,*)' MakeMap: Number of reflection used',irefactiv,
!**  *' su ',numbMax
!
!     generate one complete hemisphere of reflexions
!
   nex = 0
   do jn=1,numbMax
      fh=cmplx(e(jn),ep(jn))

      do j=1,nsym
         do l=1,3
            i2(l) = khl(1,jn) * kmat(j,1,l) + khl(2,jn) * kmat(j,2,l) + khl(3,jn) * kmat(j,3,l)
         enddo
         phase =-twopi * (khl(1,jn) * tmat(j,1) + khl(2,jn) * tmat(j,2) + khl(3,jn) * tmat(j,3))
         fm = fh * phasec(phase)

         if (i2(3) >= 0) then
             ih=  lmod(i2(1),nu)
             ik=  lmod(i2(2),nv)
             il=2*lmod(i2(3),nww)
             x(ih,ik,il)  =real(fm)
             x(ih,ik,il+1)= aimag(fm)
         endif

         if (-i2(3) >= 0) then
             ih=  lmod(-i2(1),nu)
             ik=  lmod(-i2(2),nv)
             il=2*lmod(-i2(3),nww)
             x(ih,ik,il)  =real(fm)
             x(ih,ik,il+1)= -aimag(fm)
         endif
      enddo
   enddo
!
   x(0,0,0)=f000
   !x = x / cryst(1)%cell%volume()
   call fft3hr(x,nu,nv,nww)
!
   if (present(ee)) then
       ee(:numbmax) = e(:)
       eep(:numbmax) = ep(:)
   endif
!
   end subroutine makemap_old

!----------------------------------------------------------------------------

#ifdef FFTW
   subroutine MakeMap_fftw(khl,fop,iphi2,numbrefl,jpatt,x,FORTHi,BACKi)
   USE General
   USE Symmetry
   USE Mappa, only : np, xmapC, FFTW_ESTIMATE
   USE Riflessi, only : work1, work2
!
   implicit none
   integer, dimension(3,numbrefl), intent(in) :: khl 
   integer*8    :: FORTHi, BACKi
   integer      :: numbrefl, jpatt, j, jn, i2(3), iphi2(numbrefl)
   integer      :: n1, n2, n3, n0, i, l, ih, ik, il, lmod
   real         :: x(0:np(1)-1,0:np(2)-1,0:np(3)-1), fop(numbrefl)
   real         :: fourcoef, phase, phase1, twopim, ee, eep
   complex      :: fh, phasec, fm
   real, parameter :: f000 = 0.0
!
   n1 = np(1)
   n2 = np(2)
   n3 = np(3)
   n0 = n1 / 2 + 1
!
!-- make plans
!
   if (FORTHi == 0) then
       call sfftw_plan_dft_c2r_3d(FORTHi, n1, n2, n3, xmapC,x, FFTW_ESTIMATE)
       call sfftw_plan_dft_r2c_3d(BACKi,  n1, n2, n3, x,xmapC, FFTW_ESTIMATE)
   endif

   xmapC(:,:,:) = cmplx(0.0,0.0)

   if (jpatt == 0) then
       do i=1,numbrefl
          fourcoef = fop(i)
          phase=dtor*float(iphi2(i))
          work1(i)=fourcoef*sin(phase)
          work2(i)=fourcoef*cos(phase)
!         write(6,*) i, fop(i),iphi2(i)
       enddo
   else
       do i=1,numbrefl
          work1(i) = 0.0                            ! gianluca 13/01/2010
          work2(i) = fop(i)
       enddo
   endif

   twopim = -twopi
   do jn=1, numbrefl
      eep     = work1(jn)
      ee      = work2(jn)
      fh      = cmplx(ee, eep)
      do j=1, nsym
         do l=1,3
            i2(l) = khl(1,jn) * kmat(j,1,l) + khl(2,jn) * kmat(j,2,l) + khl(3,jn) * kmat(j,3,l)
         enddo
         phase1 = khl(1,jn) * tmat(j,1) + khl(2,jn) * tmat(j,2) + khl(3,jn) * tmat(j,3)
         phase = twopim * phase1
         fm = fh * phasec(phase)
         if (i2(1) >= 0) then
             ih = lmod(i2(1),n0)
             ik = lmod(i2(2),n2)
             il = lmod(i2(3),n3)
             xmapC(ih,ik,il) = conjg(fm)
         endif
         if (-i2(1) >= 0) then
             ih = lmod(-i2(1),n0)
             ik = lmod(-i2(2),n2)
             il = lmod(-i2(3),n3)
             xmapC(ih,ik,il) = fm
         endif
      enddo
   enddo

   xmapC(0,0,0) = cmplx(f000, 0.0)

   call sfftw_execute(FORTHi)

   return
   end subroutine makemap_fftw

!----------------------------------------------------------------------

   subroutine invmap_fftw(khl, iphi2, fop2, numbMax, n1, n2, n3, BACKi)
 
   USE General
   USE Symmetry
   USE Riflessi, only : mkang, wt
   USE Mappa
   !USE PhaseModule

   implicit none
   integer, dimension(:,:), intent(in) :: khl   ! (3,nrefl)
   integer, dimension(:),  intent(out) :: iphi2
   real, dimension(:), intent(out)     :: fop2
   integer, intent(in)                 :: numbMax
   integer, intent(in)                 :: n1, n2, n3
   integer*8, intent(inout)            :: BACKi
   integer                             :: n0, i2(3), lmod
   integer                             :: i, j, l, iu, iv, iw
   real                                :: phaseof, scfft1, phim, faze1, faze, favi
   complex                             :: sf, phasec
   
   n0 = n1 / 2 + 1
!---- Scale ----!
   scfft1 = 1.0/ real(n1 * n2 * n3 * nsym)

   call sfftw_execute(BACKi)
!---- Fill the reflection list ----!
   do i=1,numbMax
      wt(i) = 0.0
!---- Average over symmetry equivalents for this reflection ----!
      sf=cmplx(0.0,0.0)

      do j=1,nsym
         do l=1,3
            i2(l) = khl(1,i) * kmat(j,1,l) + khl(2,i) * kmat(j,2,l) + khl(3,i) * kmat(j,3,l)
         enddo

         phim = twopi * ( khl(1,i) * tmat(j,1) + khl(2,i) * tmat(j,2) + khl(3,i) * tmat(j,3) ) 
         if (hemiFFTW(i2(1),i2(2),i2(3))) then
            iu = lmod(i2(1),n0)
            iv = lmod(i2(2),n2)
            iw = lmod(i2(3),n3)
            sf = sf + conjg(xmapC(iu,iv,iw)) * phasec(phim)
         else
            iu = lmod(-i2(1),n0)
            iv = lmod(-i2(2),n2)
            iw = lmod(-i2(3),n3)
            sf = sf + xmapC(iu,iv,iw) * phasec(phim)
         endif
      enddo
                                                    
      sf=sf*scfft1
      fop2(i)=abs(sf)   ! F.d.S calc.
      faze=phaseof(sf)*rtod
      if (mkang(i).ne.1.and.nsym.ne.1) then
          faze1= favi(mkang(i),faze)
          faze=faze1
      endif
      faze=mod(faze+3600.,360.)
      iphi2(i)=faze   ! la fase calc.
   enddo

   end subroutine invmap_fftw

!-----------------------------------------------------------------------

   logical function hemiFFTW(ih,ik,il)
   integer, intent(in) :: ih, ik, il

   hemiFFTW = (ih > 0 .or. (ih == 0 .and. (il > 0 .or. (il == 0 .and. ik > 0) ) ) )

   end function hemiFFTW

!-----------------------------------------------------------------------
   subroutine DistruggiPianiAll
   USE Mappa
!
   if (FORTH /= 0) then
       call sfftw_destroy_plan(FORTH)
       call sfftw_destroy_plan(BACK)
   endif
   FORTH  = 0
   BACK   = 0
!
   end subroutine DistruggiPianiAll

#endif

! ----------------------------------------------------------------------

   subroutine Search99_xmap(xmap,mode,ier,numb)
   USE Atomi
   USE ParametriMappa
   USE General
   USE PhaseModule
   USE elements, only: numelem
   implicit none
   real xmap(0:np(1)-1,0:np(2)-1,0:np(3)-1)
   integer :: negmp, mode, ier, numb, i
!
   negmp = 0
   call pksrch_sir(xmap,mode,ier,numb,negmp)
   if (neutro.ne.0) then
       do i=1,numelem(celem)
          if (celem(i)%cl < 0.0) negmp=1
       enddo
   endif
   if (negmp.eq.1) then
       call negmap_sir(xmap)
       call pksrch_sir(xmap,mode,ier,numb,negmp)
   endif
!
   end subroutine Search99_xmap

! ----------------------------------------------------------------------

   subroutine Search99_noxmap(mode,ier,numb)
   USE mappa, only: xmap
   implicit none
   integer :: mode, ier, numb
!
   call Search99_xmap(xmap,mode,ier,numb)
!
   end subroutine Search99_noxmap

! ----------------------------------------------------------------------

   subroutine AllocaMappa(iAction)
   USE General
   USE Mappa
   USE PhaseModule 
!
!-- Allocate / Deallocate arrays  ( iAction = 1 / 2 )
!
   integer, intent(in) :: iAction
   integer :: i
   integer, dimension(3) :: ierv
   integer :: ierr
!
   ifPatt = 1

   do i=1,3
      ierv(i) = 0
   enddo

   nr3dSize = 3 * ( np(1) + 2 ) * ( np(3) + 2 )

   ierr = 0
   if (iAction == 1) then
       IF (.NOT. ALLOCATED(xmap)) ALLOCATE (xmap(0:np(1)-1,0:np(2)-1,0:np(3)+1), stat = ierv(1))
!  *    ALLOCATE (xmap(0:np(1)-1,0:np(2)-1,0:np(3)-1), stat = ierv(1))
       IF (.NOT. ALLOCATED(nr3d)) ALLOCATE (nr3d (nr3dSize), stat = ierv(3))
#ifdef FFTW
       IF (.NOT. ALLOCATED(xmapC)) ALLOCATE (xmapC(0:np(1)/2,0:np(2)-1,0:np(3)-1), stat=ierv(3))
#endif
       if (ifPatt > 0) then 
           IF (.NOT. ALLOCATED(enve)) ALLOCATE (enve(0:np(1)-1,0:np(2)-1,0:np(3)+1), stat = ierv(2))
       endif
!
       if (ierv(1).ne.0.or.ierv(2).ne.0.or.ierv(3).ne.0) then
           write(0,*) 'Map Allocation Error'
           stop
       endif
       xmap(:,:,:) = 0.0
       nr3d(:) = 0
       if (ifPatt > 0) then 
           enve(:,:,:) = 0.0
       end if
#ifdef FFTW
       xmapC(:,:,:) = cmplx(0.0, 0.0)
       call DistruggiPianiAll
#endif
   else
       IF (ALLOCATED(xmap)) DEALLOCATE (xmap,nr3d,  stat = ierr)
       IF (ALLOCATED(enve)) DEALLOCATE (enve,  stat = ierr)
#ifdef FFTW
       IF (ALLOCATED(xmapC)) DEALLOCATE (xmapC, stat = ierr)
#endif
       if (ierr.ne.0) then
           write(0,*) 'Mappa DeAllocation Error'
           stop
       endif
   endif
!
   return
   end subroutine AllocaMappa

!-----------------------------------------------------------------------

   subroutine maxmin3d(denmin,denmax,ave,std,npoints,id)
   USE Mappa
   real, intent(out), optional    :: denmin,denmax
   real, intent(out), optional    :: ave,std
   integer, intent(out), optional :: npoints
   integer, intent(in)            :: id
   integer                        :: ix,iy,iz,npunt
   real                           :: xx

   ave  =  0.0
   std  =  0.0
   npunt=0
!
   if(id==0) then
      if (present(std) .and. present(ave)) then
          do iz = 0, np(3)-1
             do iy = 0, np(2)-1
                do ix = 0, np(1)-1
                   xx=xmap(ix,iy,iz)
                   ave=ave + xx
                   std=std + xx * xx
                   npunt=npunt+1
                enddo
             enddo
          enddo
      endif
      if (present(denmax)) denmax = maxval(xmap)
      if (present(denmin)) denmin = minval(xmap)
   else
      if (present(std) .and. present(ave)) then
          do iz = 0, np(3)-1
             do iy = 0, np(2)-1
                do ix = 0, np(1)-1
                   xx=xmap(ix,iy,iz)
                   if(xx>0) then   
                      ave=ave + xx
                      std=std + xx * xx
                      npunt=npunt+1
                   endif
                enddo
             enddo
          enddo
      endif
      if (present(denmax)) denmax = maxval(xmap,mask=xmap>0)
      if (present(denmin)) denmin = minval(xmap,mask=xmap>0)
   endif
!
   if (present(std) .and. present(ave)) then
       ave=ave/npunt
       std=std/npunt - ave*ave
       if (std > 0.0) then
           std=sqrt(std)
       else
           std=0.0
       endif
   endif
   if (present(npoints)) npoints = npunt
!
   end subroutine maxmin3d

!-----------------------------------------------------------------------

   subroutine four_point(xyzs,ref,fcoef,cellt,spg,radtype)
!--calcola valore di rho per punti definiti in dummy(i,j), j=1, nats
!-- in input: 
!       nreflDim      num riflessi da usare via USE General
!       khl(3,kj), fcoef(kj), iphaz(kj)  (indici, coef, fase) via USE Riflessi
!       dummy(1,j) = x
!       dummy(2,j) = y
!       dummy(3,j) = z
!       dummy(5,j) = occ
!       ifSort = 1/0  sort/no sort of dummy with respect to rho
!-- in output:
!       dummy(4,j) = rho
!       rho(xyz) = 1/Vc [ F(000) 
!                + 2 SUM(h+)SUM(k)SUM(l) |F|cos[2pi(hx+ky+lz)-alpha] ]
!
      USE General, only: ihmx,celem
      use spginfom
      use elements
      use unit_cell
      use trig_constants, only: twopi
      use reflection_type_util
!
      real, dimension(:,:), intent(inout) :: xyzs
      type(reflection_type), dimension(:), allocatable, intent(in) :: ref
      real, dimension(:), intent(in) :: fcoef
      integer :: nats
      real, dimension(size(xyzs,1),size(xyzs,2)) :: dummy
      type(cell_type), intent(in) :: cellt
      type(spaceg_type), intent(in) :: spg
      integer, intent(in) :: radtype

!corr      common/ylimi/nfl,lfl,nlx,nflmx,lflmx,nlmx,idummy(1)
!corr      integer :: nfl,lfl,nlx,nflmx,lflmx,nlmx,idummy
!corr      common/xunit/ncru,ncwu,ncpu,ncpt,itty,istat2,iabort,intty
!corr      integer :: ncru,ncwu,ncpu,ncpt,itty,istat2,iabort,intty
      integer :: m,jy,ih,ik,il,j,i1,kj,imen1,indr,NRFourTot
      real :: fht,a,ff0,f000,xalph,ArgTot,rho0,cm
      real :: fmilles,xphase
      real, dimension(6) :: pp
      real, dimension(3,3) :: matr
!
!corr      real, dimension(3) :: fxx,fxx2p
      integer :: ikln(60),ill(3)
!corr      real, dimension(:), allocatable :: temp
      integer :: ifat1,ifat2
!      
!corr      allocate(temp(numelem(celem)))
!      allocate(dummy(5,numatoms(atm)))
      dummy = xyzs
      dummy(4,:) = 0.0
                  dummy(5,:) = abs(dummy(5,:))
      nats = size(dummy,2)
!      dummy(5,:) = atm%ocry
      ifat1=(2*ihmx(3)+1)*(2*ihmx(2)+1)
      ifat2=(2*ihmx(3)+1)
      NRFourTot=0
      fmilles=1.0/360.0
   matr = cellt%get_r()
   pp(:) = (/0.25*matr(1,1),0.25*matr(2,2),0.25*matr(3,3),0.5*matr(1,2),0.5*matr(1,3),0.5*matr(2,3)/)

!=====================================================
!--loop over the reflections
!=====================================================
      do kj=1,numrefl(ref)
!corr         xphase=iphaz(kj)*fmilles
         xphase=ref(kj)%ph*fmilles
!====================================================
!--loop based on  the symmetry positions
!====================================================
         do j=1,spg%nsym
!corr             do m=1,3
!corr                ill(m)=0.
!corr                do n=1,3
!corr                   ill(m)=ill(m)+ref(kj)%hkl(n)*kmat(j,n,m)
!corr                enddo
!corr             enddo
             ill = matmul(ref(kj)%hkl,spg%symop(j)%rot(:,:))
             indr=ill(1)*ifat1+ill(2)*ifat2+ill(3)
             if (j.ne.1) then
!--elimina i doppioni
               imen1=j-1
               do i1=1,imen1
                  if (iabs(indr).eq.iabs(ikln(i1))) indr=0
               enddo
             endif
             ikln(j)=indr
             if(indr.eq.0) cycle
             NRFourTot=NRFourTot+1
!--calculate the h.t terms
!corr             fht=0.
!corr             do m=1,3
!corr                fht=fht+ref(kj)%hkl(m)*tmat(j,m)
!corr             enddo  
             fht = dot_product(ref(kj)%hkl,spg%symop(j)%trn(:))
             xalph=xphase-fht         !  alpha-(h.t)
!==========================================                             
!--loop based on the peaks                                 
!==========================================                             
            do jy = 1 , nats
               if (dummy(5,jy).ge.0.0) then
                   a=0.0
                   do m=1,3
                      a=a+dummy(m,jy)*ill(m) 
                   enddo
                   ArgTot=(a-xalph)*twopi         !  2pi[(hx+ky+lz)-alpha]
                   dummy(4,jy)=dummy(4,jy)+fcoef(kj)*cos(ArgTot)
               endif
!--end of peaks              
            enddo   
!--end of symmetry               
         enddo 
!--end of reflections  
      enddo  
!==========================================                             
!--final calculation       
!==========================================  
      f000 = 0.0                           
      rho0 = 0.0
      if (radtype == ELECTRON_SOURCE) then
          ih = 1
          ik = 0
          il = 0
!corr          cm = cell(1)
          cm = cellt%get_par(1)
!corr          if (cell(2) < cm) then
          if (cellt%get_par(2) < cm) then
              ih = 0
              ik = 1
              il = 0
!corr              cm = cell(2)
              cm = cellt%get_par(2)
          endif
!corr          if (cell(3) < cm) then
          if (cellt%get_par(3) < cm) then
              ih = 0
              ik = 0
              il = 1
          endif
          rho0 = rhof(pp,ih,ik,il)
          rho0  = rho0 * rho0
!         rho0  = 0
      endif
      do j = 1,numelem(celem)
!        if (neutro ==2) then
!            ff0 = zeff(j)
!        else
!corr             ff0 = ScattFact(j, rho0)
             ff0 = at_scatt(celem(j), rho0, radtype)
!corr             temp(j) = ff0
!        endif
!corr         f000 = f000 + float(celem(j)%nw) * ff0  !no. atomi in cella*numero atomico
         f000 = f000 + celem(j)%nw * ff0  !no. atomi in cella*numero atomico
      enddo
!      write(ncwu,1020) f000,volume
!1020  format('F(000) =',f10.2,/,'Volume =',f10.2)      
!      write(lo,1010)nreflDim,NRFourTot
!1010  format('Indipendent reflections:'i8,/,'Total reflections:'i8)      
      do jy = 1 , nats
         if (dummy(5,jy).ge.0.0) then
             dummy(4,jy) = 2.0 * dummy(4,jy) + f000
             dummy(4,jy) = dummy(4,jy) / cellt%volume()
             if (dummy(4,jy) < 0.0) then
                 dummy(4,jy) = 0.001
             endif
         endif
         write(0,*)'PEAK ',jy,dummy(4,jy)
      enddo
!
!corr      if (ifSort == 1) then
!corr          call sortf(dummy, righe, nats, 4)
!corr      endif
!
      return
      end

!-----------------------------------------------------------------------

   real function get_f000(atom,elem,spg,radtype)  result(f000)
   use elements
   use atom_basic
   use spginfom
   type(atom_type), dimension(:), allocatable, intent(in)    :: atom
   type(element_type), dimension(:), allocatable, intent(in) :: elem
   type(spaceg_type), intent(in)                             :: spg
   integer, intent(in)                                       :: radtype
   f000 = 0.0
   if (numelem(elem) == 0) return
   if (numatoms(atom) == 0) then
       if (radtype == RX_SOURCE) then
           f000 = sum(elem%nw*elem%z)
       else
           f000 = sum(at_scatt(elem,0.0,radtype))
       endif
   else
       if (radtype == RX_SOURCE) then
           f000 = sum(atom%z()*atom%och*atom%ocry*spg%nsymop)
       else
           f000 = sum(at_scatt(elem(atom%kscatt()),0.0,radtype)*atom%och*atom%ocry*spg%nsymop)
       endif
   endif
   end function get_f000

!-----------------------------------------------------------------------

   subroutine scalemap_xmap(f000,cell)
   use Mappa
   use unit_cell
   real, intent(in)            :: f000
   type(cell_type), intent(in) :: cell
!
   xmap = (xmap + f000) / cell%volume()
!
   end subroutine scalemap_xmap

!-----------------------------------------------------------------------

   subroutine scalemap_vect(inte,f000,cell)
   use unit_cell
   real, dimension(:), intent(inout) :: inte
   real, intent(in)                  :: f000
   type(cell_type), intent(in)       :: cell
!
   inte = (inte + f000) / cell%volume()
!
   end subroutine scalemap_vect

   end module fourier_mod

#ifdef FFTW
      subroutine testit()
      implicit none
      INCLUDE 'fftw3.f'
      !INCLUDE 'fftw_f77.i'
      double complex in, out
      integer, parameter :: N=16
      dimension in(N), out(N)
      integer*8 p,p2
      integer i,j
      real fact
      write(*,*)"stuff in data"
      do i=1,N
        j=i-1
        in(i)=cmplx(j*j,1)
      enddo
      write(*,*)"create plans:",FFTW_FORWARD,FFTW_ESTIMATE
      call dfftw_plan_dft_1d(p ,N,in,out,FFTW_FORWARD,FFTW_ESTIMATE)
      call dfftw_plan_dft_1d(p2,N,in,out,FFTW_BACKWARD,FFTW_ESTIMATE)
      write(*,*)"do it"
      call dfftw_execute_dft(p, in, out)
      do i=1,N
        write(*,"(f12.4,1x,f12.4)")out(i)
      enddo
      write(*,*)
      write(*,*)"undo it"
      call dfftw_execute_dft(p2, out,in)
      fact=1.0/N
      do i=1,N
        write(*,"(f10.2,1x,f10.2)")in(i)*fact
      enddo
      write(*,*)"clean up"
      call dfftw_destroy_plan(p)
      call dfftw_destroy_plan(p2)
      end subroutine testit

!-----------------------------------------------------------------------

      subroutine testfftw()
      implicit none
      INCLUDE 'fftw3.f'
      !INCLUDE 'fftw_f77.i'
      double complex in, out
      integer, parameter :: N=16
      dimension in(N), out(N)
      integer*8 p,p2
      integer i,j
      real fact
      write(*,*)"stuff in data"
      do i=1,N
        j=i-1
        in(i)=cmplx(j*j,1)
      enddo
      write(*,*)"create plans:",FFTW_FORWARD,FFTW_ESTIMATE
      call dfftw_plan_dft_1d(p ,N,in,out,FFTW_FORWARD,FFTW_ESTIMATE)
      call dfftw_plan_dft_1d(p2,N,in,out,FFTW_BACKWARD,FFTW_ESTIMATE)
      write(*,*)"do it"
      call dfftw_execute_dft(p, in, out)
      do i=1,N
        write(*,"(f12.4,1x,f12.4)")out(i)
      enddo
      write(*,*)
      write(*,*)"undo it"
      call dfftw_execute_dft(p2, out,in)
      fact=1.0/N
      do i=1,N
        write(*,"(f10.2,1x,f10.2)")in(i)*fact
      enddo
      write(*,*)"clean up"
      call dfftw_destroy_plan(p)
      call dfftw_destroy_plan(p2)
      end subroutine testfftw

#endif

!----------------------------------------------------------------------

    real function GetXMap(i, j, k)
    USE Mappa
!   
    implicit none
    integer       :: i, j, k
!   
    GetXMap = xmap(i,j,k)
!   
    return
    end

!----------------------------------------------------------------------

      function kspecb(newvet,xo,xn,key,itype,iser,khead,iactn)
!
!-- function to handle atoms in special position
!
!   return value
!          kspecb = +1 atom in general position
!                 = -1 atom in special position
!-- meaning of parameters
!     xo    = array containing original values for x,y,z,occ,u's,c-occ
!     xn    = array containing modified values for x,y,z,occ,u's,c-occ
!    key    = array containing conditions for      x,y,z,    u's,c-occ
!  itype    = type of atom
!   iser    = serial number
!  khead    =  0  heading for special position to be print
!           =  1  heading not to be printed
!     iactn =     action to be taken :
!           =  1  compute crystallographic occupation
!           =  2  generate key array and update x's and u's
!           =  3  print information
!    newvet = matrix containing information about
!             the symmetry operators involved
      USE kspec_mod
      USE general, only: spacegr,cell
      USE symmetry, only: jsys
      USE unit_cell
      implicit none
      integer                   :: kspecb
      real, dimension(11)       :: xo,xn
      integer, dimension(10)    :: key
      integer                   :: itype,iser,khead,iactn
      integer, dimension(192,3) :: newvet
      real, dimension(3,3)      :: mt
!
      mt = matrice_metrica(cell)
      kspecb =  kspecb_new(xo,xn,key,spacegr,jsys,mt,iactn)
!
      return
      end
!
