#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <cstring>
#include <string>
#include <cstdio>
#include <sstream>
#include <fstream>
#include <iomanip>
#include <cstdlib>
#include <cmath>
#include <cctype>

#include <utils.h>

// gdk/Cairo support

void disegna_pixnew(gpointer winout, gpointer context, gpointer myp,
                    int xb, int yb, int *xd, int *yd,
                    bool force_pos)
{
      int wi, he, xdest, ydest;

      drawable_get_size(myp, &wi, &he);

      xdest = xb - wi / 2;
      ydest = yb - he / 2;
      if(force_pos && (xdest < 0))
         xdest = 0;
      render_pixbuf_to_drawable (myp, winout, context, xdest, ydest);
      if(xdest >= 0)
         *xd = 2*xb - xdest;
      else
         *xd = 2*xb;
      if(ydest >= 0)
         *yd = 2*yb - ydest;
      else
         *yd = 2*yb;
}

void disegna_pix(gpointer winout, gpointer context,
                    gpointer myp, int xb, int yb)
{
      render_pixbuf_to_drawable (myp, winout, context, xb, yb);
}

void save_pixbuf_from_mem(os_and_gc *rt, const char *filename)
{
     GError *error = NULL;
     GdkPixbuf *dest = (GdkPixbuf *) pixbuf_get_from_struct (rt);
     if(dest != NULL)
     {
        gdk_pixbuf_save (dest, toUTF8(filename), "png", &error, NULL);
        g_object_unref(dest);
     }
}

void save_pixbuf_from_window(GtkWidget *vw, const char *filename)
{
     os_and_gc *rt = g_new0(os_and_gc, 1);

     rt->os = get_object(vw, "offscreen");
     rt->gc = gtk_widget_get_window(vw);
     
     save_pixbuf_from_mem(rt, filename);
     g_free(rt);
}

#ifndef USE_CAIRO

gpointer draw_rgb_image (gpointer outPix, gpointer context, 
                     int x, int y, int w, int h, 
                     unsigned char *fbuffer, int stride)
{
    gdk_draw_rgb_image ((GdkDrawable *)outPix, (GdkGC *)context,
                         x, y, w, h,
                         GDK_RGB_DITHER_MAX, fbuffer, w*stride);
    return fbuffer;
}

void render_pixbuf_to_drawable(gpointer mypixmap, gpointer winout,
                               gpointer context, int xdest, int ydest)
{
     gdk_pixbuf_render_to_drawable ((GdkPixbuf *)mypixmap,
                                    (GdkDrawable *)winout,
                                    (GdkGC *) context, 0, 0,
                                     xdest, ydest, -1, -1,
                                     GDK_RGB_DITHER_NONE, 0, 0);
}

void flush_os(gpointer outPix, gpointer outWindow, gpointer context)
{
    if(outPix && outWindow && context)
       draw_drawable(outWindow, context, outPix, 0, 0);
}

void draw_string(gpointer outPix,PangoFontDescription *fontdesc,
                 gpointer context, int xb, int yb, const char *string,
                 GdkColor *fore,GdkColor *back, PangoAlignment align)
{
     PangoContext *pc = gdk_pango_context_get();
#ifdef MUST_SET_CM
     gdk_pango_context_set_colormap(pc, gdk_colormap_get_system());
#endif
     PangoLayout *layout = pango_layout_new(pc);
     GdkGC *gc = (GdkGC *) context;

     pango_layout_set_font_description (layout, fontdesc);
     if(parse_text(string))
        pango_layout_set_markup (layout, string, -1);
     else
        pango_layout_set_text (layout, string, -1);
     pango_layout_set_alignment (layout, align);

     gdk_draw_layout_with_colors((GdkDrawable *)outPix, gc, xb, yb, layout,
              fore, back);
     g_object_unref (layout);
     g_object_unref (pc);
}

void draw_Vstring(gpointer outPix, PangoFontDescription *fontdesc,
                gpointer context, int xb, int yb, const char *string,
                GdkColor *fore, GdkColor *back, int yalign)
{
     int xi, yi, i,lar, alt;
     char string1[40];

     if(context == NULL)
        return;
     get_metrics(fontdesc, &lar, &alt);
     PangoContext *pc = gdk_pango_context_get();
#ifdef MUST_SET_CM
     gdk_pango_context_set_colormap(pc, gdk_colormap_get_system());
#endif
     PangoLayout *layout = pango_layout_new(pc);
     GdkGC *gc = (GdkGC *) context;

     pango_layout_set_font_description (layout, fontdesc);
     xi = xb;
     yi = yb;
     switch(yalign)
     {
     case 0:
          yi = yb - (int)strlen(string)*alt/2;
          break;
     case -1:
          yi = yb - (int)strlen(string)*alt;
          break;
     default:
          yi = yb;
          break;
     }
     char *newstring = g_markup_escape_text (string, -1);
     for(i=0; i<(int)strlen(newstring);i++)
     {
         ScriviStringa(string1, sizeof(string1), "<tt> %c </tt>", newstring[i]);
         pango_layout_set_markup (layout, string1, -1);
         gdk_draw_layout_with_colors((GdkDrawable *)outPix, gc, xi, yi,
             layout, fore, back);
         yi += alt;
     }
     g_object_unref (layout);
     g_object_unref (pc);
}

#ifndef PANGO_TYPE_MATRIX
void draw_Vstring(gpointer outPix, PangoFontDescription *fontdesc,
                 gpointer context, int xb, int yb, const char *string,
                 GdkColor *fore, int align)
{
    draw_Vstring(outPix, fontdesc, context, xb, yb, string,
                 fore, NULL, align);
}
#else
void draw_Vstring(gpointer outPix1, PangoFontDescription *fontdesc,
                 gpointer gc1, int xb, int yb, const char *string,
                 GdkColor *fore, int align)
{
   double angle = 90;
   GdkDrawable  *outPix = (GdkDrawable  *)outPix1;
   GdkGC *gc = (GdkGC *)gc1;
   GdkScreen *screen = gdk_gc_get_screen (gc);
   PangoMatrix matrix = PANGO_MATRIX_INIT;

   PangoRenderer *renderer = gdk_pango_renderer_get_default (screen);
   gdk_pango_renderer_set_drawable (GDK_PANGO_RENDERER (renderer), outPix);
   gdk_pango_renderer_set_gc (GDK_PANGO_RENDERER (renderer), gc);
   pango_matrix_translate (&matrix, xb, yb);

   PangoContext *pc = gdk_pango_context_get_for_screen (screen);
   PangoLayout *layout = pango_layout_new (pc);
   pango_layout_set_font_description (layout, fontdesc);
   if(parse_text(string))
      pango_layout_set_markup (layout, string, -1);
   else
      pango_layout_set_text (layout, string, -1);
//   pango_layout_set_markup (layout, string, -1);
   gdk_pango_renderer_set_override_color (GDK_PANGO_RENDERER (renderer),
                                          PANGO_RENDER_PART_FOREGROUND, fore);
   pango_matrix_rotate (&matrix, angle);
   pango_context_set_matrix (pc, &matrix);
   pango_layout_context_changed (layout);
   pango_renderer_draw_layout (renderer, layout, yb, xb);
   gdk_pango_renderer_set_override_color (GDK_PANGO_RENDERER (renderer),
                                       PANGO_RENDER_PART_FOREGROUND, NULL);
   gdk_pango_renderer_set_drawable (GDK_PANGO_RENDERER (renderer), NULL);
   gdk_pango_renderer_set_gc (GDK_PANGO_RENDERER (renderer), NULL);

   g_object_unref (layout);
   g_object_unref (pc);
}
#endif

void destroy_os_and_gc(gpointer os, gpointer gc)
{
    if(os)
       g_object_unref(os);
    if(gc)
       g_object_unref(gc);
}

os_and_gc *get_os_and_gc(GtkWidget* widget)
{
    os_and_gc *rt = g_new0(os_and_gc, 1);
    GtkAllocation allocation;
    gtk_widget_get_allocation(widget, &allocation);
    rt->os =  gdk_pixmap_new(gtk_widget_get_window(widget),
                             (int)allocation.width,
                             (int)allocation.height,
                             -1);
    rt->gc =  gdk_gc_new(gtk_widget_get_window(widget));
    gdk_gc_set_fill ((GdkGC *) rt->gc, GDK_SOLID);
    return rt;
}

gpointer pixbuf_get_from_struct(os_and_gc *rt)
{
     if(rt != NULL)
     {
        int w, h;
        drawable_get_size(rt->os, &w, &h);
        if((w <= 0) || (h <= 0))
          return NULL;
        return gdk_pixbuf_get_from_drawable (NULL, (GdkDrawable *)rt->os, 
                                      gdk_colormap_get_system(),
                                      0, 0, 0, 0, w, h);
     }
     return NULL;
}

void free_context(gpointer context)
{
}

void set_clip(GtkWidget *win,  gpointer context)
{
    GdkRectangle rect;
    GdkGC *plot_gc = (GdkGC *) context;

    if(plot_gc)
    {
         rect.x = 0;
         rect.y = 0;
         rect.width = (int) win->allocation.width;
         rect.height = (int) win->allocation.height;
         gdk_gc_set_clip_origin(plot_gc, 0, 0);
         gdk_gc_set_clip_rectangle(plot_gc, &rect);
    }
}

void set_foreground(gpointer context, GdkColor *colore)
{
    if(context == NULL)
       return;
    GdkColor *c1 = colore;
    gdk_gc_set_rgb_fg_color((GdkGC *) context, c1);
}

void draw_rectangle(gpointer win, gpointer context, gboolean ifFill,
                    int x1, int y1, int width, int height)
{
    GdkGC *plot_gc = (GdkGC *) context;
    gdk_draw_rectangle ((GdkDrawable *)win, plot_gc, ifFill,
                         x1, y1, width, height);
}

void draw_rectangle(gpointer win, gpointer context, gboolean ifFill,
                    gpointer rect)
{
    GdkGC *plot_gc = (GdkGC *) context;
    GdkRectangle *r = (GdkRectangle *) rect;
    gdk_draw_rectangle ((GdkDrawable *)win, plot_gc, ifFill,
                         r->x, r->y, r->width, r->height);
}

void draw_rectangle2(gpointer win, gpointer context, gboolean ifFill,
                    int x1, int y1, int width, int height, float col[])
{
    GdkColor co = {0, (guint16)(col[0]*65535.f), (guint16)(col[1]*65535.f), (guint16)(col[2]*65535.f)};
    GdkColor co2 = {0, (guint16)(col[0]*16384.f), (guint16)(col[1]*16384.f), (guint16)(col[2]*16384.f)};

    GdkGC *plot_gc = (GdkGC *) context;
    set_foreground(context, &co2);
    gdk_draw_rectangle ((GdkDrawable *)win, plot_gc, FALSE,
                         x1, y1, width, height);
    set_foreground(context, &co);
    gdk_draw_rectangle ((GdkDrawable *)win, plot_gc, ifFill,
                         x1, y1, width, height);
}

void draw_line(gpointer win, gpointer context, int x1, int y1, int x2, int y2, int lw)
{
    GdkGC *plot_gc = (GdkGC *) context;
    gdk_draw_line ((GdkDrawable *)win, plot_gc, x1, y1, x2, y2);
}

void draw_circle(gpointer win, gpointer context, gboolean ifFill,
              int x1, int y1, int d)
{
    GdkGC *plot_gc = (GdkGC *) context;
    gdk_draw_arc( (GdkDrawable *)win, plot_gc, ifFill,
                     x1, y1, d, d, 0, 360*64 );
}

void draw_arc(gpointer win, gpointer context, gboolean ifFill,
              int x1, int y1, int d, int ang1, int ang2)
{
    GdkGC *plot_gc = (GdkGC *) context;
    gdk_draw_arc( (GdkDrawable *)win, plot_gc, ifFill, x1, y1, d, d, ang1*64, ang2*64 );
}

void draw_sector(gpointer win, gpointer context, gboolean ifFill,
              int x1, int y1, int d, int ang1, int ang2)
{
     draw_arc(win, context, ifFill, x1, y1, d, ang1, ang2);
}

void draw_ell(gpointer win, gpointer context, gboolean ifFill,
              int x1, int y1, int width, int height, int ang1, int ang2)
{
    GdkGC *plot_gc = (GdkGC *) context;
    gdk_draw_arc( (GdkDrawable *)win, plot_gc, ifFill, x1, y1, width, height, 
                  ang1*64, ang2*64 );
}

void draw_polygon(gpointer win, gpointer context, gboolean ifFill,
              GdkPoint *points, int npoints)
{
    GdkGC *plot_gc = (GdkGC *) context;
    gdk_draw_polygon( (GdkDrawable *)win, plot_gc, ifFill, points, npoints );
}

void draw_lines(gpointer win, gpointer context, 
              GdkPoint *points, int npoints)
{
    GdkGC *plot_gc = (GdkGC *) context;
    gdk_draw_lines( (GdkDrawable *)win, plot_gc, points, npoints );
}

void draw_lines_aa(gpointer win, gpointer context, 
              GdkPoint *points, int npoints)
{
    GdkGC *plot_gc = (GdkGC *) context;
    gdk_draw_lines( (GdkDrawable *)win, plot_gc, points, npoints );
}

void draw_segments(gpointer win, gpointer context, 
              GdkSegment *segments, int ns)
{
    GdkGC *plot_gc = (GdkGC *) context;
    gdk_draw_segments( (GdkDrawable *)win, plot_gc, segments, ns );
}

void draw_point(gpointer win, gpointer context, int x, int y)
{
    GdkGC *plot_gc = (GdkGC *) context;
    gdk_draw_point( (GdkDrawable *)win, plot_gc, x, y );
}

void draw_points(gpointer win, gpointer context, GdkPoint *p, int np)
{
    GdkGC *plot_gc = (GdkGC *) context;
    gdk_draw_points((GdkDrawable *)win, plot_gc, p, np);
}

void drawable_get_size(gpointer mypixmap, int *wi, int *he)
{
    *wi = 0;
    *he = 0;
    if(GDK_IS_DRAWABLE(mypixmap))
       gdk_drawable_get_size((GdkDrawable *)mypixmap, wi, he);
    else if (GDK_IS_PIXBUF(mypixmap))
    {
       *wi = gdk_pixbuf_get_width  ((GdkPixbuf *)mypixmap);
       *he = gdk_pixbuf_get_height ((GdkPixbuf *)mypixmap);
    }
}

void set_dash_style(gpointer context)
{
    GdkGC *plot_gc = (GdkGC *) context;
    gdk_gc_set_line_attributes(plot_gc,
                               1,
                               GDK_LINE_ON_OFF_DASH,
                               GDK_CAP_BUTT,
                               GDK_JOIN_MITER);
#ifdef _WIN32
     {
          signed char dash_list[2] = {'\1','\1'};
          gdk_gc_set_dashes (plot_gc, 0, dash_list, 2);
     }
#endif
}

void reset_style(gpointer context)
{
    GdkGC *plot_gc = (GdkGC *) context;
    gdk_gc_set_line_attributes(plot_gc,
                               1,
                               GDK_LINE_SOLID,
                               GDK_CAP_BUTT,
                               GDK_JOIN_MITER);
}

gpointer mr_load_pixmap  (GtkWidget *widget, const gchar *filename)
{
  GdkPixmap *gdkpixmap;
  GdkColormap *colormap = gtk_widget_get_colormap (widget);
  GdkPixbuf *pixb = create_pixbuf(filename);
  gdk_pixbuf_render_pixmap_and_mask_for_colormap (pixb, colormap,
                                           &gdkpixmap, NULL, 1);

  return gdkpixmap;
}

void draw_drawable(gpointer win, gpointer context, gpointer mypixmap,
                   int xdest, int ydest)
{
    draw_drawable(win, context, mypixmap, 0, 0, xdest, ydest,-1, -1);
}

void draw_drawable(gpointer win, gpointer context, gpointer mypixmap,
                   int xsrc, int ysrc, int xdest, int ydest, int w, int h)
{
    GdkGC *plot_gc = (GdkGC *) context;

    gdk_draw_drawable((GdkDrawable *)win, plot_gc, (GdkPixmap *)mypixmap,
                       xsrc, ysrc, xdest, ydest, w, h);
}

gpointer create_initial_pix(const gchar *filename)
{
    return(create_pixbuf(filename));
}

void palla_sfumata(gpointer context, gpointer outPix, 
                         int x, int y, int raggio, GdkColor *col)
{
      int j, i0, i, j2, i2, ix1, iy1, raggio2;
      double dpunto, distm, rad;

      rad = (double) raggio;
      int plux = (int) (rad/2.0);
      raggio2 = raggio * raggio;

      dpunto = (rad*sqrt(2.)/2 + plux) * (rad*sqrt(2.)/2 + plux);
      distm = sqrt((double)(dpunto+dpunto));
      if(distm < 0.5)
         distm = 1.;

      float flim = 0.90f; 

      int red   = (int)col->red;
      int green = (int)col->green;
      int blue  = (int)col->blue;
      for(j = raggio; --j >= -raggio;)
      {
          j2 = j * j;
          i0 = (int)( sqrt((double)(raggio2 - j2)) + 0.5);
          j2 = (j + plux) * (j + plux);
          iy1 = j + y; // + 1;
          for(i = -i0; i <= i0; i++)
          {
              i2 = (i - plux) * (i - plux);
              dpunto = sqrt((double)(j2+i2));
              if ( dpunto <= 0 )
                   dpunto = 1.;
              ix1 = i + x; // + 1;

              if((ix1 >=0) && (iy1 >=0))
              {
                   float coef = (float) ((distm - dpunto) / distm);
                   guint16 r1 =  (guint16) MRblend(red,   65535, coef, flim);
                   guint16 g1 =  (guint16) MRblend(green, 65535, coef, flim);
                   guint16 b1 =  (guint16) MRblend(blue,  65535, coef, flim);
                   GdkColor c1 = {0, r1, g1, b1};
                   set_foreground ( context, &c1);
                   draw_point( outPix, context, ix1, iy1);
              }
          }
       }
}

#else

gpointer draw_rgb_image (gpointer , gpointer context, 
                     int x, int y, int w, int h,
                     unsigned char *fbuffer, int stride)
{
     int stride1 = 4*w;
#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 6, 0)
     stride1 = cairo_format_stride_for_width (CAIRO_FORMAT_RGB24,w);
#endif
     unsigned char *buf = g_new0(unsigned char, stride1*h);
     memset(buf, -1, stride1*h);
//
     for(int i=0; i<stride1*h; i+=stride)
     {
         buf[i]   = fbuffer[i+2];
         buf[i+1] = fbuffer[i+1];
         buf[i+2] = fbuffer[i];
     }
     g_free(fbuffer);
     cairo_surface_t *s = cairo_image_surface_create_for_data(buf,
                          CAIRO_FORMAT_RGB24,w, h, stride*w);
     cairo_t *cr = (cairo_t *)context;
     cairo_set_source_surface (cr, s, x, y);
     cairo_rectangle (cr, x, y, w, h);
     cairo_fill (cr);
     return buf;
}

void render_pixbuf_to_drawable(gpointer myp, gpointer ,
                               gpointer context, int xdest, int ydest)
{
      int wi, he;
      drawable_get_size(myp, &wi, &he);
      cairo_t *cr = (cairo_t *) context;
      cairo_set_source_surface (cr, (cairo_surface_t *)myp, xdest, ydest);
      cairo_rectangle (cr, xdest, ydest, wi, he);
      cairo_fill (cr);
}

void flush_os(gpointer outPix, gpointer outWindow, gpointer )
{
   if(outPix && outWindow)
   {
#if (GTK_MAJOR_VERSION > 2) && (GTK_MINOR_VERSION >= 22)
     cairo_region_t *reg = gdk_window_get_visible_region(GDK_WINDOW(outWindow));
     GdkDrawingContext *dc = gdk_window_begin_draw_frame
           (GDK_WINDOW(outWindow), reg);
     cairo_t *tgc = gdk_drawing_context_get_cairo_context(dc);
     cairo_set_source_surface(tgc, (cairo_surface_t *)outPix, 0, 0);
     cairo_paint(tgc);
     gdk_window_end_draw_frame(GDK_WINDOW(outWindow), dc);
#else
     cairo_t *tgc = gdk_cairo_create((GdkWindow *)outWindow);
     cairo_set_source_surface(tgc, (cairo_surface_t *)outPix, 0, 0);
     cairo_paint(tgc);
     cairo_destroy(tgc);
#endif
   }
}

void draw_string(gpointer outPix,PangoFontDescription *fontdesc,
                 gpointer context, int xb, int yb, const char *string,
                 GdkColor *fore,GdkColor *back, PangoAlignment align)
{
     PangoRectangle *r2 = g_new0(PangoRectangle, 1);
     cairo_t *cr = (cairo_t *)context;
     PangoLayout *layout = pango_cairo_create_layout (cr);
     pango_layout_set_font_description (layout, fontdesc);
//     char *newstring = g_markup_escape_text (string, -1);
//     pango_layout_set_markup (layout, newstring, -1);
     if(parse_text(string))
        pango_layout_set_markup (layout, string, -1);
     else
        pango_layout_set_text (layout, string, -1);
     pango_layout_get_pixel_extents(layout, NULL, r2);
     pango_layout_set_alignment (layout, align);
     if(back != NULL)
     {
        set_foreground (cr, back);
        draw_rectangle(outPix, context, TRUE, xb, yb, r2->width, r2->height);
     }

     set_foreground (cr, fore);
     cairo_move_to (cr, xb, yb);
     pango_cairo_show_layout (cr, layout);
     g_object_unref (layout);
     g_free(r2);
}

void draw_Vstring(gpointer outPix, PangoFontDescription *fontdesc,
                gpointer context, int xb, int yb, const char *string,
                GdkColor *fore, GdkColor *back, int yalign)
{
     PangoLayout *layout;
     PangoRectangle r2;
     int xi, yi, i,lar, alt;
     char string1[40];

     if(context == NULL)
        return;
     get_metrics(fontdesc, &lar, &alt);
     cairo_t *cr = (cairo_t *) context;
     layout = pango_cairo_create_layout (cr);
     pango_layout_set_font_description (layout, fontdesc);
     xi = xb;
     yi = yb;
     switch(yalign)
     {
     case 0:
          yi = yb - (int)strlen(string)*alt/2;
          break;
     case -1:
          yi = yb - (int)strlen(string)*alt;
          break;
     default:
          yi = yb;
          break;
     }
     for(i=0; i<(int)strlen(string);i++)
     {
         ScriviStringa(string1, sizeof(string1), "<tt> %c </tt>", string[i]);
         pango_layout_set_markup (layout, string1, -1);
         if(back != NULL)
         {
            pango_layout_get_pixel_extents(layout, NULL, &r2);
            set_foreground (cr, back);
            draw_rectangle(outPix, context, TRUE, xi, yi, r2.width, r2.height);
         }
         set_foreground (cr, fore);
         cairo_move_to (cr, xi, yi);
         pango_cairo_show_layout (cr, layout);
         yi += alt;
     }
     g_object_unref (layout);
}

void draw_Vstring(gpointer outPix, PangoFontDescription *fontdesc,
                 gpointer context, int xb, int yb, const char *string,
                 GdkColor *fore, int align)
{
   int width, height;
   float *color = ToColor3f(fore);
   double angle = 270, xi;
   cairo_t *cr = (cairo_t *) context;
   cairo_surface_t *surface = (cairo_surface_t *) outPix;
   int w = cairo_image_surface_get_width (surface);
   int h = cairo_image_surface_get_height (surface);

   PangoLayout *layout = pango_cairo_create_layout (cr);
   pango_layout_set_font_description (layout, fontdesc);
   if(parse_text(string))
      pango_layout_set_markup (layout, string, -1);
   else
      pango_layout_set_text (layout, string, -1);
//   pango_layout_set_markup (layout, string, -1);
   cairo_save (cr);
   cairo_translate (cr, w/2 , h/2 );
   cairo_set_source_rgb (cr, color[0], color[1], color[2]);
   cairo_rotate (cr, angle * G_PI / 180.);
   pango_cairo_update_layout (cr, layout);
   pango_layout_get_size (layout, &width, &height);
//   double yi = -(double)w/2 + (double)xb -((double)height / PANGO_SCALE)/2;
   double yi = -(double)w/2 + (double)xb;
   switch (align)
   {
   case 0:
          xi = (h/2-((double)width / PANGO_SCALE)/2)-yb;
          break;
   case -1:
          xi = h/2-yb -((double)width / PANGO_SCALE);
          break;
   default:
          xi =  h/2-yb;
          break;
   }
   cairo_move_to (cr, xi, yi);
   pango_cairo_show_layout (cr, layout);
   cairo_restore (cr);
   g_object_unref (layout);
   g_free(color);
}

void destroy_os_and_gc(gpointer os, gpointer gc)
{
    if(gc)
       cairo_destroy((cairo_t *)gc);
    if(os)
       cairo_surface_destroy((cairo_surface_t *)os);
}

os_and_gc *get_os_and_gc(GtkWidget* widget)
{
    GtkAllocation allocation;
    gtk_widget_get_allocation(widget, &allocation);
    int w = (int)allocation.width;
    int h = (int)allocation.height;
    os_and_gc *rt = g_new0(os_and_gc, 1);
    cairo_surface_t *image = cairo_image_surface_create(CAIRO_FORMAT_RGB24, w, h);
    rt->os = (gpointer) image;
    cairo_t *cr = cairo_create (image);
    rt->gc = (gpointer) cr;
    cairo_set_line_width ((cairo_t *) rt->gc, 1.0);
    return rt;
}

#if GTK_MAJOR_VERSION < 3
os_and_gc *create_cairo_stuff(int wi, int he, GtkWidget *w)
{
    os_and_gc *rt = g_new0(os_and_gc, 1);
    rt->os = gdk_pixmap_new(gtk_widget_get_window(w), wi, he,-1);
    rt->gc = gdk_cairo_create ((GdkDrawable *)rt->os);
    return rt;
}

gpointer pixbuf_get_from_struct(os_and_gc *rt)
{
     if(rt != NULL)
     {
        int w, h;
        drawable_get_size(rt->os, &w, &h);
        if((w <= 0) || (h <= 0))
          return NULL;
        cairo_surface_t *image = (cairo_surface_t *)rt->os;
        if(image == NULL)
           return NULL;

        GdkWindow *window = (GdkWindow *) rt->gc;
        GdkPixmap *pixmap = gdk_pixmap_new (window, w, h, -1);
        cairo_t *cr = gdk_cairo_create (pixmap);
        cairo_set_source_surface (cr, image, 0, 0);
        cairo_paint (cr);
        cairo_destroy (cr);
        GdkPixbuf *pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, w, h);
        pixbuf = gdk_pixbuf_get_from_drawable (NULL, pixmap, gdk_colormap_get_system(), 0, 0, 0, 0, w, h);
        g_object_unref(pixmap);
        return pixbuf;
     }
     return NULL;
}

GdkPixbuf *pixbuf_get_from_cairo_stuff(gpointer p1, int wi, int he)
{
     GdkPixbuf *pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, wi, he);
     pixbuf = gdk_pixbuf_get_from_drawable (NULL, (GdkDrawable *)p1, 
         gdk_colormap_get_system(), 0, 0, 0, 0, wi, he);
     return pixbuf;
}

#else
os_and_gc *create_cairo_stuff(int wi, int he, GtkWidget *w)
{
    os_and_gc *rt = g_new0(os_and_gc, 1);
    rt->os = cairo_image_surface_create(CAIRO_FORMAT_RGB24, wi, he);
    rt->gc = cairo_create ((cairo_surface_t *)rt->os);
    return rt;
}

gpointer pixbuf_get_from_struct(os_and_gc *rt)
{
     if(rt != NULL)
     {
        int w, h;
        drawable_get_size(rt->os, &w, &h);
        if((w <= 0) || (h <= 0))
          return NULL;
        cairo_surface_t *image = (cairo_surface_t *)rt->os;
        if(image == NULL)
           return NULL;
        GdkPixbuf *pixbuf = gdk_pixbuf_get_from_surface (image, 0, 0, w, h);
        return pixbuf;
     }
     return NULL;
}

GdkPixbuf *pixbuf_get_from_cairo_stuff(gpointer p1, int wi, int he)
{
     cairo_surface_t *image = (cairo_surface_t *)p1;
     if(image == NULL)
        return NULL;
     return(gdk_pixbuf_get_from_surface (image, 0, 0, wi, he));
}
#endif

void free_context(gpointer context)
{
    cairo_t *cr = (cairo_t *)context;
    cairo_destroy(cr);
}

void set_clip(GtkWidget *,  gpointer )
{
}

void set_foreground(gpointer context, GdkColor *colore)
{
    if(context == NULL)
       return;
    double red   = (double) colore->red   / 65535.;
    double green = (double) colore->green / 65535.;
    double blue  = (double) colore->blue  / 65535.;
    cairo_set_source_rgb((cairo_t *)context, red, green, blue);
}

void draw_rectangle(gpointer , gpointer context, gboolean ifFill,
                    int x1, int y1, int width, int height)
{
    cairo_t *cr = (cairo_t *) context;
    cairo_set_line_width (cr, 1);
    cairo_rectangle(cr, x1, y1, width, height);
    if(ifFill)
       cairo_fill(cr);
    else
       cairo_stroke(cr);
}

void draw_rectangle(gpointer , gpointer context, gboolean ifFill,
                    gpointer rect)
{
    cairo_t *cr = (cairo_t *) context;
    cairo_set_line_width (cr, 1);
#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 10, 0)
    cairo_rectangle_int_t *r = (cairo_rectangle_int_t *) rect;
    cairo_rectangle(cr, r->x, r->y, r->width, r->height);
#else
typedef struct { int x, y, w, h;}  rettangolo;
    rettangolo *r = (rettangolo *)rect;
    GdkPoint xy[4];
    xy[0].x = r->x;
    xy[0].y = r->y;
    xy[1].x = r->x;
    xy[1].y = r->y+r->h;
    xy[2].x = r->x+ r->w;
    xy[2].y = r->y+r->h;
    xy[3].x = r->x+ r->w;
    xy[3].y = r->y;
    for (int i=1; i<4; i++)
         cairo_line_to( cr, xy[i].x, xy[i].y );
    cairo_close_path (cr);
#endif

    if(ifFill)
       cairo_fill(cr);
    else
       cairo_stroke(cr);
}

void draw_rectangle2(gpointer , gpointer context, gboolean ifFill,
                    int x1, int y1, int width, int height, float col[])
{
    GdkColor co = {0, (guint16)(col[0]*65535.f), (guint16)(col[1]*65535.f), (guint16)(col[2]*65535.f)};
    GdkColor co2 = {0, (guint16)(col[0]*16384.f), (guint16)(col[1]*16384.f), (guint16)(col[2]*16384.f)};
    cairo_t *cr = (cairo_t *) context;
    cairo_set_line_width (cr, 1);
    cairo_rectangle(cr, x1+1, y1+1, width, height);
    set_foreground(context, &co2);
    cairo_stroke(cr);
    cairo_rectangle(cr, x1, y1, width, height);
    set_foreground(context, &co);
    if(ifFill)
       cairo_fill(cr);
    else
       cairo_stroke(cr);
}

void draw_line(gpointer , gpointer context, int x1, int y1, int x2, int y2, int lw)
{
    cairo_t *cr = (cairo_t *) context;
    if(lw <= 1)
       cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
    cairo_set_line_width (cr, (lw>0)?lw:1);
    cairo_move_to(cr, x1, y1);
    cairo_line_to(cr, x2, y2);
    cairo_stroke(cr);
    if(lw <= 1)
       cairo_set_antialias(cr, CAIRO_ANTIALIAS_DEFAULT);
}

void draw_circle(gpointer , gpointer context, gboolean ifFill,
              int x1, int y1, int d)
{
    cairo_t *cr = (cairo_t *) context;
    int r = d/2;
    int xc = x1 + r;
    int yc = y1 + r;
    cairo_move_to(cr, xc+r, yc);
    cairo_arc (cr, xc, yc, r, 0, 2*M_PI);
    if(ifFill)
       cairo_fill(cr);
    else
       cairo_stroke(cr);
}

void draw_arc(gpointer , gpointer context, gboolean ifFill,
              int x1, int y1, int d, int ang1, int ang2)
{
    cairo_t *cr = (cairo_t *) context;
    int r = d/2;
    int xc = x1 + r;
    int yc = y1 + r;

    double angle1 = -(double)ang1 * (M_PI/180.0);
    double angle2 = angle1 - (double)ang2 * (M_PI/180.0);

#ifdef USE_HAVE_SUBPATH
    cairo_new_sub_path (cr); 
#else
    cairo_move_to (cr, xc, yc);
#endif
    if(ifFill)
       cairo_line_to (cr, xc, yc);
    cairo_arc_negative(cr, xc, yc, r, angle1, angle2);
    if(ifFill)
    {
       cairo_line_to (cr, xc, yc);
       cairo_close_path(cr);
       cairo_fill(cr);
    }
    else
       cairo_stroke(cr);
}

void draw_sector(gpointer , gpointer context, gboolean ifFill,
              int x1, int y1, int d, int ang1, int ang2)
{
    cairo_t *cr = (cairo_t *) context;
    int r = d/2;
    int xc = x1 + r;
    int yc = y1 + r;

    if(ang2 < 0)
       ang1 = 360 - ang1;
    ang2 = MRabs(ang2);
    double angle1 = (double)ang1 * (M_PI/180.0);
    double angle2 = (double)(ang1+ang2) * (M_PI/180.0);

#ifdef HAVE_SUBPATH
    cairo_new_sub_path (cr); 
#else
    cairo_move_to (cr, x1, y1);
#endif
    cairo_line_to (cr, xc, yc);
    cairo_arc(cr, xc, yc, r, angle1, angle2);
    cairo_line_to (cr, xc, yc);
    cairo_close_path(cr);
    if(ifFill)
       cairo_fill(cr);
    else
       cairo_stroke(cr);
}

void draw_polygon(gpointer , gpointer context, gboolean ifFill,
              GdkPoint *points, int npoints)
{
    cairo_t *cr = (cairo_t *) context;
    cairo_move_to(cr, points[0].x, points[0].y);
    for (int i=1; i<npoints; i++)
         cairo_line_to( cr, points[i].x, points[i].y );
    cairo_close_path (cr);
    if(ifFill)
       cairo_fill(cr);
    else
       cairo_stroke(cr);
}

void draw_lines(gpointer , gpointer context, 
              GdkPoint *points, int npoints, int lw)
{
    cairo_t *cr = (cairo_t *) context;
    if(lw <= 1)
       cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
    double clw = cairo_get_line_width(cr);
    cairo_set_line_width(cr, (lw>0)?lw:1);
    cairo_move_to(cr, points[0].x, points[0].y);
    for (int i=1; i<npoints; i++)
         cairo_line_to( cr, points[i].x, points[i].y );
    cairo_stroke(cr);
    if(lw <= 1)
       cairo_set_antialias(cr, CAIRO_ANTIALIAS_DEFAULT);
    cairo_set_line_width(cr, clw);
}

void draw_lines_aa(gpointer , gpointer context, 
              GdkPoint *points, int npoints)
{
    cairo_t *cr = (cairo_t *) context;
    cairo_move_to(cr, points[0].x, points[0].y);
    for (int i=1; i<npoints; i++)
         cairo_line_to( cr, points[i].x, points[i].y );
    cairo_stroke(cr);
}

void draw_segments(gpointer , gpointer context,
              GdkSegment *segments, int npoints)
{
    cairo_t *cr = (cairo_t *) context;
    cairo_antialias_t aa = cairo_get_antialias(cr);
    cairo_set_line_width (cr, 1);
    cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
    for (int i=0; i<npoints; i++)
    {
         cairo_move_to(cr, segments[i].x1, segments[i].y1);
         cairo_line_to(cr, segments[i].x2, segments[i].y2);
    }
    cairo_stroke(cr);
    cairo_set_antialias(cr, aa);
}

void draw_point(gpointer , gpointer context, int x, int y)
{
    cairo_t *cr = (cairo_t *) context;
    cairo_move_to (cr, (double) x + 0.5, (double) y); 
    cairo_line_to (cr, (double) x + 0.5, (double) y + 1.0); 
    cairo_stroke(cr);
/*
    cairo_set_line_width (cr, 1);
    cairo_set_line_cap  (cr, CAIRO_LINE_CAP_ROUND); 
    cairo_move_to (cr, x, y);
    cairo_line_to (cr, x, y);
    cairo_stroke (cr);
    cairo_set_line_cap  (cr, CAIRO_LINE_CAP_BUTT); 
*/
}

void draw_points(gpointer , gpointer context, GdkPoint *p, int np)
{
    cairo_t *cr = (cairo_t *) context;
    for (int i=0; i<np; i++)
    {
         cairo_move_to (cr, (double) p[i].x + 0.5, (double) p[i].y);
         cairo_line_to (cr, (double) p[i].x + 0.5, (double) p[i].y + 1.0);
    }
    cairo_stroke(cr);
}

void draw_ell(gpointer , gpointer context, gboolean ifFill,
              int x1, int y1, int width, int height, int ang1, int ang2)
{
    cairo_t *cr = (cairo_t *) context;
    int x = x1 + width/2;
    int y = y1 + height/2;

/*
    if(ang2 < 0)
       ang1 = 360 - ang1;
    ang2 = MRabs(ang2);

    double angle1 = (double)ang1 * (M_PI/180.0);
    double angle2 = (double)ang2 * (M_PI/180.0);
*/
    double angle1 = -(double)ang1 * (M_PI/180.0);
    double angle2 = angle1 - (double)ang2 * (M_PI/180.0);
    cairo_save (cr);

#ifdef USE_HAVE_SUBPATH
    cairo_new_sub_path (cr);
#else
    cairo_move_to (cr, x1, y1);
#endif

    cairo_translate (cr, x, y);
    cairo_scale (cr, width / 2., height / 2.);
    cairo_arc_negative (cr, 0., 0., 1., angle1, angle2);
    cairo_restore (cr);
    if(ifFill)
       cairo_fill(cr);
    else
       cairo_stroke(cr);
}

void drawable_get_size(gpointer image, int *wi, int *he)
{
    *wi = cairo_image_surface_get_width  ((cairo_surface_t *)image);
    *he = cairo_image_surface_get_height ((cairo_surface_t *)image);
}

void set_dash_style(gpointer context)
{
    cairo_t *cr = (cairo_t *) context;
    double dashes[] = {2, 2};
    int ndash  = sizeof (dashes)/sizeof(dashes[0]);
    double offset = -2.0;
    cairo_set_dash (cr, dashes, ndash, offset);
}

gpointer mr_load_pixmap  (GtkWidget *, const gchar *filename)
{
  gchar *pathname = NULL;

  if (!filename || !filename[0])
      return NULL;

  pathname = find_pixmap_file (filename);

  if (!pathname)
      return NULL;
  cairo_surface_t *image = cairo_image_surface_create_from_png (pathname);
  return image;
}

void draw_drawable(gpointer win, gpointer context, gpointer mypixmap,
                   int xdest, int ydest)
{
    draw_drawable(win, context, mypixmap, 0, 0, xdest, ydest,-1, -1);
}

void draw_drawable(gpointer win, gpointer , gpointer mypixmap, 
                   int , int , int xdest, int ydest, int , int )
{
    cairo_t *cr;
    if(GDK_IS_WINDOW(win))
    {
#if (GTK_MAJOR_VERSION > 2) && (GTK_MINOR_VERSION >= 22)
       GdkRectangle rect;
       rect.width= gdk_window_get_width (GDK_WINDOW(win));
       rect.height = gdk_window_get_height (GDK_WINDOW(win));
       gdk_window_get_position (GDK_WINDOW(win), &rect.x, &rect.y);
       cairo_region_t *reg = (cairo_region_t *)region_rectangle(&rect);
       GdkDrawingContext *dc = gdk_window_begin_draw_frame
           (GDK_WINDOW(win), reg);
       cairo_t *tgc = gdk_drawing_context_get_cairo_context(dc);
       cairo_set_source_surface(tgc, (cairo_surface_t *)mypixmap, 0, 0);
       cairo_paint(tgc);
       gdk_window_end_draw_frame(GDK_WINDOW(win), dc);
       return;
#else
       cr = gdk_cairo_create((GdkWindow *) win);
#endif
    }
    else
       cr = cairo_create((cairo_surface_t *) win);
    cairo_surface_t *image = (cairo_surface_t *)mypixmap;
//270512    cairo_set_source_surface (cr, image, 0, 0);
    cairo_set_source_surface (cr, image, xdest, ydest);
    cairo_paint (cr);
    cairo_destroy(cr);
}

void reset_style(gpointer context)
{
    cairo_t *cr = (cairo_t *) context;
    cairo_set_line_width (cr, 1);
    cairo_set_dash (cr, NULL, 0, 0);
}

gpointer create_initial_pix(const gchar *filename)
{
    return(create_surface_from_png(filename));
}

gpointer create_surface_from_png(const gchar *filename)
{
    gchar *found_filename = find_pixmap_file(filename);
    if(!found_filename)
       return NULL;
    gpointer retval = cairo_image_surface_create_from_png (found_filename);
    g_free(found_filename);
    return retval;
}

void palla_sfumata(gpointer context, gpointer , 
                         int x, int y, int raggio, GdkColor *col)
{
cairo_t *cr = (cairo_t *) context;
double x0 = (double)x + (double)raggio * 0.36;
double y0 = (double)y - (double)raggio * 0.36;
double x1 = (double)x + (double)raggio * 0.25;
double y1 = (double)y - (double)raggio * 0.25;
double r0 = MRmax(1.0, (double)raggio/15.);
float color[3];
color[0] = (float)col->red   / 65535.f;
color[1] = (float)col->green / 65535.f;
color[2] = (float)col->blue  / 65535.f;
cairo_pattern_t *pat = cairo_pattern_create_radial (x0, y0, r0,
                                   x1, y1, (double) raggio);
cairo_pattern_add_color_stop_rgba (pat, 0, 1, 1, 1, 1);
cairo_pattern_add_color_stop_rgba (pat, 0.5, color[0], color[1], color[2], 1);
Darker(color);
cairo_pattern_add_color_stop_rgba (pat, 1.0, color[0], color[1], color[2], 1);
cairo_set_source (cr, pat);
cairo_arc (cr, x, y, raggio, 0, 2 * M_PI);
cairo_fill (cr);
cairo_pattern_destroy (pat);
}

#endif

// Regions management

#define USE_CAIRO_REGIONS 1
#ifndef CAIRO_VERSION
   #undef USE_CAIRO_REGIONS
#elif CAIRO_VERSION < CAIRO_VERSION_ENCODE(1, 10, 0)
   #undef USE_CAIRO_REGIONS
#endif
#ifdef USE_CAIRO_REGIONS

gpointer region_new()
{
    return cairo_region_create ();
}

gpointer region_rectangle(gpointer rectangle)
{
    return cairo_region_create_rectangle((cairo_rectangle_int_t *) rectangle);
}

void region_get_clipbox (gpointer region, GdkRectangle *rectangle)
{
    cairo_rectangle_int_t rect;
    cairo_region_get_extents ((cairo_region_t *) region, &rect);
    rectangle->x = rect.x-1;
    rectangle->y = rect.y-1;
    rectangle->width = rect.width+2;
    rectangle->height = rect.height+2;
}

gboolean point_in_region(gpointer region, int x, int y)
{
    return cairo_region_contains_point((cairo_region_t *)region, x, y);
}

void region_destroy(gpointer region)
{
     cairo_region_destroy((cairo_region_t *)region);
}

#else

gpointer region_new()
{
    return gdk_region_new ();
}

gpointer region_rectangle(gpointer rectangle)
{
    return gdk_region_rectangle((GdkRectangle *) rectangle);
}

void region_get_clipbox (gpointer region, GdkRectangle *rectangle)
{
    gdk_region_get_clipbox ((GdkRegion *) region, rectangle);
}

gboolean point_in_region(gpointer region, int x, int y)
{
    return gdk_region_point_in((GdkRegion *)region, x, y);
}

void region_destroy(gpointer region)
{
     gdk_region_destroy((GdkRegion *)region);
}

#endif
// end regions

