MODULE hydrogens
!
! S add_hydrogens(atom,legm,vat,htype,ier)        Aggiunge atomi di idrogeno al modello
! F hydrogen_type(atom,legm,katom) result(htype)  Restituisce un codice che identifica il tipo di idrogeni 
! F distance_hydrogen(atom,hybr)                  Restituisce la distanza X-H riportata in letteratura
! S save_hydrogens(atom,atomH)                    Salva in atomH solo gli H
! S update_hydrogens(atom,atomH)                  Aggiorna in atom solo gli H
!
implicit none

enum, bind(c)
  enumerator :: CH3=1,CH2,CH,C_ALCHINO,C_Ar,X_CH2
  enumerator :: O_H
  enumerator :: N1_H,N2_H
  enumerator :: S_H
  enumerator :: B_H
  enumerator :: C_DEF,N_DEF
endenum

integer, parameter :: NHDISTANCES = 13

! distanze X-H da raggi X
real, dimension(NHDISTANCES), private :: distHx = (/0.969, & ! CH3 metilico (CH3)
                                                    0.980, & ! CH2 metilenico (CH2)
                                                    0.980, & ! CH metinico (CH)
                                                    0.930, & ! C_ALCHINO: in alchino terminale (C_ALCHINO)
                                                    0.930, & ! C_Ar: composti aromatici (C_Ar)
                                                    0.930, & ! X=CH2 terminale (X_CH2)
                                                    0.820, & ! distanza O-H (O_H)         
                                                    0.865, & ! distanza N-H in ammina primaria (N1_H)
                                                    0.898, & ! distanza N-H in ammina secondaria (N2_H)
                                                    1.20,  & ! distanza S-H (S_H)
                                                    1.10,  & ! distanza B-H (B_H)
                                                    0.980, & ! C_DEF (C_DEF)
                                                    0.880/)  ! N_DEF (N_DEF)
 
! distanze X-H normalizzate
real, dimension(NHDISTANCES), parameter, private :: distHn0 = (/1.10, &  ! CH3 metilico
                                                       1.10, &  ! CH2 metilenico
                                                       1.10, &  ! CH metinico
                                                       1.08, &  ! C_ALCHINO: in alchino terminale
                                                       1.08, &  ! C_Ar: composti aromatici
                                                       1.08, &  ! X=CH2 terminale
                                                       0.95, &  ! distanza O-H         
                                                       1.01, &  ! distanza N-H in ammina primaria      
                                                       1.01, &  ! distanza N-H in ammina secondaria        
                                                       1.31, &  ! distanza S-H 
                                                       1.10, &  ! distanza B-H
                                                       1.10, &  ! C_DEF 
                                                       1.01/)   ! N_DEF

real, dimension(NHDISTANCES) :: distHn = distHn0

integer, parameter, private :: NMAXSERIAL = 10000

CONTAINS

   subroutine add_hydrogens(atom,legm,elem,cell,spg,vat,htype,ier)
!
!  Aggiunge atomi di idrogeno al modello
!  Gli H vengono labellati secondo la convezione del cristallo singolo 
!  e introdotti dopo l'atomo a cui sono legati
!
!  htype = 1 -->  H acetylenico (sp)
!  htype = 2 -->  H aromatic    (sp2)
!  htype = 3 -->  terminal CH2 (sp2)
!  htype = 4 -->  H metinico  (sp3)
!  htype = 5 -->  H metilenico  (sp3)
!  htype = 6 -->  H metilico    (sp3)
!  htype = 7 -->  H ossidrilico (sp3)
!  htype = 8 -->  terminal BH
!
!  ier = numero di atomi per il quale c'è stato errore
!
   USE connect_mod
   USE trig_constants, only: rtod
   USE cgeom
   USE unit_cell
   USE atom_type_util
   USE contacts
   USE nr
   USE elements
   USE fragmentmod
   USE spginfom
   USE arrayutil
   type(atom_type), dimension(:), allocatable, intent(inout)    :: atom    ! tutti gli atomi del modello
   type(bond_type), dimension(:), allocatable, intent(inout)    :: legm    ! tutti i legami
   type(element_type), dimension(:), allocatable, intent(inout) :: elem
   integer, dimension(:), intent(in), optional                  :: vat     ! atomi sui quali aggiungere gli H
   integer, intent(in), optional                                :: htype   ! tipo di idrogeno
   type(cell_type), intent(in)                                  :: cell
   type(spaceg_type), intent(in)                                :: spg
   integer, dimension(:), allocatable                           :: veth,vtype
   integer                                                      :: nat
   integer                                                      :: nath
   integer                                                      :: ier 
   integer                                                      :: i,j
   type(container_type), dimension(:), allocatable              :: conn,connbr,conn1
   integer                                                      :: kat
   integer                                                      :: k1,k2
   real, dimension(3)                                           :: ci
   integer                                                      :: natadd
   integer                                                      :: natnew,natold
   real                                                         :: dist0
   real                                                         :: angH
   real, parameter                                              :: sdH = 0.3  ! stand. dev.
   real, dimension(3)                                           :: xatb
   real                                                         :: tangle
   integer                                                      :: kleg
   integer, dimension(2)                                        :: k11
   type(atom_type), dimension(:), allocatable                   :: atomc
   type(atom_type), dimension(:), allocatable                   :: atoms  
   type(bond_type), dimension(:), allocatable                   :: legms 
   type(bond_type)                                              :: legH
   type(atom_type)                                              :: atH
   integer, parameter                                           :: NMAXSERIAL = 10000
   integer, dimension(0:NMAXSERIAL)                             :: letterH,serialH
   integer                                                      :: kser
   integer                                                      :: ndimv
   integer, allocatable, dimension(:)                           :: vetpos,iord
   integer                                                      :: zkat
   integer                                                      :: nconnbr
   logical                                                      :: getbr
   integer                                                      :: ierat
   type(atom_type), dimension(3)                                :: atn
   type(bond_type), dimension(:), allocatable                   :: legn
   integer                                                      :: k,nhadd
   integer                                                      :: natolds,nlegolds
!
   ier = 0
   nat = numatoms(atom)
   if (nat == 0) return
   allocate(legn(3))
   if (.not.is_atomic_specie(atom,C_at)) return ! return if Carbon is absent 
!
   call init_serialH(atom,serialH,letterH)
   getbr = .false.
!
   if (present(vat)) then
!
!      Se vat e' presente cerca gli h solo negli atomi di vat
       nath = size(vat)
       allocate(veth(nath))
       veth(:) = vat
   else
!
!      Se vat e' assente, cerca gli H su tutti gli atomi
       nath = nat
       allocate(veth(nath))
       veth(:) = (/(i,i=1,nat)/)
   endif
!
!  Espandi la simmetria dei legami
   call grow_fragments(atom,atoms,legm,legms,cell,spg,lsym=.false.,kexp=(/EXPAND_ALL,0/))  
   !call init_for_symm(atom,legm,atoms,legms) ! uncomment to avoid symmetry adding hydrogens
   atoms(:)%asym = 0; atoms(:)%op = op_type() ! reset symmetry
   atom(:) = atoms(:nat)  ! aggiorna l'u.a. modificata dalla grow
   natolds = size(atoms)
   nlegolds = numbonds(legms)
!
!  conversione in coord. cartesiane
   call frac_to_cart_copy(atoms,atomc,cell%get_ortom())
!
   allocate(vtype(nath))
   if (present(htype)) then
       vtype(:) = htype
   else
!
!      Calcolo automatico del tipo di H da legare
       call bond_type_perception(atoms,legms,cell)
       do i=1,nath
          kat = veth(i)
          vtype(i) = atom_type_id(atomc,legms,kat)  ! usa tutti gli atomi della grow fragment
       enddo
   endif
!
   call bond_to_connect(size(atoms),legms,conn)    
!
   ndimv = 3*nath
   !allocate(atnew(ndimv),legmnew(ndimv),vetpos(nat+ndimv))
   allocate(vetpos(nat+ndimv))
   vetpos(1:nat) = (/(i,i=1,nat)/)   ! vettore che memorizza la posizione finale degli atomi
   natadd = 0
   natold = size(atom)
!
   do i=1,nath
      kat = veth(i)
      nhadd = 0
      select case(vtype(i))
         case (C_ALCHINO)       ! H acetilenico (sp)
           dist0 = distance_hydrogen(atoms(kat),C_ALCHINO)
           call new_acetilenic_atom(atn(1),legn(1),kat,atomc,conn,dist0,cell=cell,cart=.true.,ier=ierat)
           if (ierat == 0) then
               nhadd = 1
               natadd = natadd + 1
               atn(1)%inte = atom(kat)%inte/atom(kat)%z()
               atn(1)%biso = 1.2*atom(kat)%biso
               !atnew(natadd) = atn(1)
               !legn(1)%n2 = nat+natadd
               legn(1)%n2 = numatoms(atomc) + 1
               !legmnew(natadd) = legn(1)
               kser = serial_number(atoms(kat))
               !call set_label_hydrogen(atnew(natadd),kser,serialH,letterH,.false.)
               call set_label_hydrogen(atn(1),kser,serialH,letterH,.false.)
               vetpos(nat+natadd) = vetpos(kat) + 1
               vetpos(kat+1:nat) = vetpos(kat+1:nat) + 1
           else
               ier = ier + 1
           endif

         case (C_Ar)       ! H aromatico  (sp2)
           dist0 = distance_hydrogen(atoms(kat),C_Ar)
           call new_aromatic_atom(atn(1),legn(1),kat,atomc,conn,dist0,cell=cell,cart=.true.,ier=ierat)
           if (ierat == 0) then
               nhadd = 1
               natadd = natadd + 1
               atn(1)%inte = atom(kat)%inte/atom(kat)%z()
               atn(1)%biso = 1.2*atom(kat)%biso
               !atnew(natadd) = atn(1)
               !legn(1)%n2 = nat+natadd
               legn(1)%n2 = numatoms(atomc) + 1
               !legmnew(natadd) = legn(1)
               kser = serial_number(atoms(kat))
               !call set_label_hydrogen(atnew(natadd),kser,serialH,letterH,.false.)
               call set_label_hydrogen(atn(1),kser,serialH,letterH,.false.)
               vetpos(nat+natadd) = vetpos(kat) + 1
               vetpos(kat+1:nat)= vetpos(kat+1:nat) + 1
           else
               ier = ier + 1
           endif

         case (X_CH2)       ! terminal X=CH2 (sp2)
           if (conn(kat)%nat == 1) then
               ierat = 0
               dist0 = distance_hydrogen(atoms(kat),X_CH2)
               angH = 120   ! angolo H-C-H
               k1 = conn(kat)%pos(1)    ! atomo X
               k2 = kat
!
!              CH2 e' coplanare con R-X-R': cerca un atomo k11 legato ad X
               select case (conn(k1)%nat)
                  case (1)            ! X forma solo un legame
                    xatb(:) = (/0.5,0.5,0.5/)  ! genera un atomo qualunque

                  case (2:)          ! X forma 2 o piu' legami
                    kleg = 0 
                    do j=1,conn(k1)%nat
                       if (conn(k1)%pos(j) /= kat) then
                           kleg = kleg + 1
                           k11(kleg) = conn(k1)%pos(j)
                       endif
                       if (kleg == 1) exit
                    enddo
                    xatb(:) = atomc(k11(1))%xc

               end select
               tangle = 0.0
               kser = serial_number(atoms(kat))
               nhadd = 2
!
!              Primo H
               ci(:) = (/dist0,angH,tangle/)
               natadd = natadd + 1
               call Get_Cartesian_from_Z(ci,atn(1)%xc,atomc(k2)%xc,atomc(k1)%xc,xatb)
               legn(1) = bond_type(kat,numatoms(atomc) + 1,dist0,sdH,0)
               call set_label_hydrogen(atn(1),kser,serialH,letterH,.true.)
               vetpos(nat+natadd) = vetpos(kat) + 1
               zkat = atom(kat)%z()
               atn(1)%inte = atom(kat)%inte/zkat
               atn(1)%biso = 1.2*atom(kat)%biso
!
!              Secondo H dal primo
               ci(:) = (/dist0,angH,180./)
               natadd = natadd + 1
               call Get_Cartesian_from_Z(ci,atn(2)%xc,atomc(k2)%xc,atomc(k1)%xc,atn(1)%xc)
               legn(2) = bond_type(kat,numatoms(atomc)+2,dist0,sdH,0)
               call set_label_hydrogen(atn(2),kser,serialH,letterH,.true.)
               vetpos(nat+natadd) = vetpos(kat) + 2
               atn(2)%inte = atom(k2)%inte/zkat
               atn(2)%biso = 1.2*atom(k2)%biso
               vetpos(kat+1:nat) = vetpos(kat+1:nat) + 2
           else
               ier = ier + 1
               ierat = ier  !!!TO REMOVE
           endif

         case (CH)       ! H metinico (sp3)
           dist0 = distance_hydrogen(atoms(kat),CH)
           call new_metinic_atom(atn(1),legn(1),kat,atomc,conn,dist0,cell=cell,cart=.true.,ier=ierat)
           if (ierat == 0) then
               nhadd = 1
               atn(1)%inte = atom(kat)%inte/atom(kat)%z()
               atn(1)%biso = 1.2*atom(kat)%biso
               !atnew(natadd+1) = atn(1)
               !legn(1)%n2 = nat+natadd+1
               legn(1)%n2 = numatoms(atomc)+1
               !legmnew(natadd+1) = legn(1)
               kser = serial_number(atoms(kat))
               !call set_label_hydrogen(atnew(natadd+1),kser,serialH,letterH,.false.)
               call set_label_hydrogen(atn(1),kser,serialH,letterH,.false.)
               vetpos(nat+natadd+1) = vetpos(kat) + 1
               vetpos(kat+1:nat) = vetpos(kat+1:nat) + 1
               natadd = natadd + 1
           else
               ier = ier + 1
           endif

         case (CH2)       ! H metilenico (sp3)
           dist0 = distance_hydrogen(atoms(kat),CH2)
           call new_metilenic_atom(atn,legn,kat,atomc,conn,dist0,cell=cell,cart=.true.,ier=ierat)
           if (ierat == 0) then
               nhadd = 2
               atn(:2)%inte = atom(kat)%inte/atom(kat)%z()
               atn(:2)%biso = 1.2*atom(kat)%biso
               legn(:2)%n2 = numatoms(atomc)+[1,2]
               !legmnew(natadd+1:natadd+2) = legn(:2)
               kser = serial_number(atoms(kat))
               do k=1,2
                  !call set_label_hydrogen(atnew(natadd+k),kser,serialH,letterH,.false.)
                  call set_label_hydrogen(atn(k),kser,serialH,letterH,.false.)
                  vetpos(nat+natadd+k) = vetpos(kat) + k
               enddo
               natadd = natadd + 2
               vetpos(kat+1:nat) = vetpos(kat+1:nat) + 2
           else
               ier = ier + 1
           endif

         case (CH3)       ! H metilici  (sp3)
           dist0 = distance_hydrogen(atoms(kat),CH3)
           call new_metilic_atom(atn,legn,kat,atomc,conn,dist0,cell=cell,cart=.true.,ier=ierat)
           if (ierat == 0) then
               nhadd = 3
               atn(:)%inte = atom(kat)%inte/atom(kat)%z()
               atn(:)%biso = 1.5*atom(kat)%biso
               !atnew(natadd+1:natadd+3) = atn(:)
               !legn(:)%n2 = nat+natadd+[1,2,3]
               legn(:)%n2 = numatoms(atomc) + [1,2,3]
               !legmnew(natadd+1:natadd+3) = legn
               kser = serial_number(atoms(kat))
               do k=1,3
                  !call set_label_hydrogen(atnew(natadd+k),kser,serialH,letterH,.false.)
                  call set_label_hydrogen(atn(k),kser,serialH,letterH,.false.)
                  vetpos(nat+natadd+k) = vetpos(kat) + k
               enddo
               natadd = natadd + 3
               vetpos(kat+1:nat) = vetpos(kat+1:nat) + 3
           else
               ier = ier + 1
           endif

         case (N1_H,N2_H)
           !write(0,*)'Primary/secondary ammine on atom: ',trim(atomc(kat)%lab)
           dist0 = distance_hydrogen(atoms(kat),vtype(i))
           if (vtype(i) == N1_H) then
               nhadd = 2
           else
               nhadd = 1
           endif
           if (nhadd == 2) then  ! only for primary amine
               if (.not.getbr) then ! compute connbr and conn1 for find_Hacceptor
                   call bond_to_connect(nat,legm,conn1)    
                   call get_branch(atom,cell,conn1,legm,nconnbr,connbr)
                   getbr = .true.
               endif
               call find_Hacceptor(atom,kat,cell,spg,legH,atH,conn1,connbr,nconnbr)
           endif
           call new_aminic_atom(atn(:nhadd),legn(:nhadd),kat,atomc,conn,dist0,cell,.true.,legH,atH,ier=ierat)
           if (ierat == 0) then
               atn(:)%inte = atom(kat)%inte/atom(kat)%z()
               atn(:)%biso = 1.5*atom(kat)%biso
               !atnew(natadd+1:natadd+nhadd) = atn(:)
               !legn(:)%n2 = nat+natadd+[1,2,3]
               do k=1,nhadd
               !   legn(k)%n2 = nat+natadd+k
                  legn(k)%n2 = numatoms(atomc) + k
               enddo
               !legmnew(natadd+1:natadd+nhadd) = legn
               kser = serial_number(atoms(kat))
               do k=1,nhadd
                  !call set_label_hydrogen(atnew(natadd+k),kser,serialH,letterH,.false.)
                  call set_label_hydrogen(atn(k),kser,serialH,letterH,.false.)
                  vetpos(nat+natadd+k) = vetpos(kat) + k
               enddo
               natadd = natadd + nhadd
               vetpos(kat+1:nat) = vetpos(kat+1:nat) + nhadd
           else
               ier = ier + 1
           endif

         case (O_H,S_H)       ! H ossidrilico (sp3)
           if (conn(kat)%nat == 1) then
               if (.not.getbr) then ! compute connbr and conn1 for find_Hacceptor
                   call bond_to_connect(nat,legm,conn1)    
                   call get_branch(atom,cell,conn1,legm,nconnbr,connbr)
                   getbr = .true.
               endif
               call find_Hacceptor(atom,kat,cell,spg,legH,atH,conn1,connbr,nconnbr)
               dist0 = distance_hydrogen(atoms(kat))
               call new_oxydrilic_atom(atn(1),legn(1),kat,atomc,conn,dist0,cell,.true.,legH,atH,ier=ierat)
               if (ierat == 0) then
                   nhadd = 1
                   atn(1)%inte = atom(kat)%inte/atom(kat)%z()
                   atn(1)%biso = 1.2*atom(kat)%biso
                   !atnew(natadd+1) = atn(1)
                   !legn(1)%n2 = nat+natadd+1
                   legn(1)%n2 = numatoms(atomc) + 1
                   !legmnew(natadd+1) = legn(1)
                   kser = serial_number(atoms(kat))
                   !call set_label_hydrogen(atnew(natadd+1),kser,serialH,letterH,.false.)
                   call set_label_hydrogen(atn(1),kser,serialH,letterH,.false.)
                   vetpos(nat+natadd+1) = vetpos(kat) + 1
                   vetpos(kat+1:nat) = vetpos(kat+1:nat) + 1
                   natadd = natadd + 1
               else
                   ier = ier + 1
               endif
           endif

         case (B_H)       ! terminal B-H
           if (conn(kat)%nat == 4 .or. conn(kat)%nat == 5) then
               ierat = 0
               dist0 = distance_hydrogen(atoms(kat))
               do j=1,3
                  xatb(j) = sum(atomc(conn(kat)%pos)%xc(j))/conn(kat)%nat   ! baricentro degli atomi legati a kat
               enddo
!
!              Disegna l'atomo di H lungo la linea che congiunge il baricentro a kat
               nhadd = 1
               natadd = natadd + 1
               !atnew(natadd)%xc = atomc(kat)%xc - (xatb(:) - atomc(kat)%xc)*dist0/distanzac(xatb,atomc(kat)%xc)
               atn(1)%xc = atomc(kat)%xc - (xatb(:) - atomc(kat)%xc)*dist0/distanzac(xatb,atomc(kat)%xc)
               !legmnew(natadd) = bond_type(kat,nat+natadd,dist0,sdH,0)
               legn(1) = bond_type(kat,numatoms(atomc)+1,dist0,sdH,0)
               kser = serial_number(atoms(kat))
               !call set_label_hydrogen(atnew(natadd),kser,serialH,letterH,.false.)
               call set_label_hydrogen(atn(1),kser,serialH,letterH,.false.)
               vetpos(nat+natadd) = vetpos(kat) + 1
               vetpos(kat+1:nat) = vetpos(kat+1:nat) + 1
               !atnew(natadd)%inte = atom(k2)%inte/atom(kat)%z()
               !atnew(natadd)%biso = 1.2*atom(k2)%biso
               atn(1)%inte = atom(k2)%inte/atom(kat)%z()
               atn(1)%biso = 1.2*atom(k2)%biso
           else
               ier = ier + 1
               ierat = ier
           endif

         case default
            ier = ier + 1
            ierat = ier

      end select

      if (ierat == 0) then
!
!         Add H if is not in the content
          if (is_element(elem,'H') == 0) then 
              call add_element(elem,H_at,nhadd,spg%nsymop)
!corr          else
!corr!
!corr!             update content of H atoms
!corr              posH = is_element(elem,'H')
!corr              elem(posH)%nw = elem(posH)%nw + nhadd*spg%nsymop
          endif
!
!         Assign element type to new atoms
          do k=1,nhadd
             call atn(k)%set_specie_from_z(H_at,elem)
          enddo
!
!         Addd new atoms and bonds to the atomc and legms
          call add_atoms_to_list(atomc,atn(:nhadd),natnew)
          call combine_legm(legms,legn,nleg=nhadd)
          call connect_update_add(conn,legn(:nhadd))
      endif
   enddo
   if (natadd > 0) then
!
!      Now copy new atoms from atomc/legms to atom/legm
       call cart_to_frac(atomc(natolds+1:),cell%get_ortoi())
       call add_atoms_to_list(atom,atomc(natolds+1:),natnew)
       legms(:natadd) = legms(nlegolds+1:)
       legms(:natadd)%n2 = legms(:natadd)%n2 - (natolds - natold)   ! update n2
       call combine_legm(legm,legms,nleg=natadd)
!
!      Ordina gli atomi in modo che gli H compaiano dopo gli atomi a cui sono legati
       allocate(iord(natnew))
       call indexx(vetpos(:natnew),iord)
       atom(:) = atom(iord)
!
!      update order numbers in legm
       iord(iord) = (/(i,i=1,natnew)/)   ! create index for bonds_update
       call bonds_update(legm,iord)
   endif
!corr!
!corr!  Aggiungi gli atomi generati
!corr   if (natadd > 0) then
!corr       if (is_element(elem,H_at) == 0) then             ! H e' nel contenuto?
!corr           call add_element(elem,H_at,natadd,spg%nsymop)
!corr       endif
!corr!
!corr!      Assegna nz agli atomi di H aggiunti
!corr       do i=1,natadd
!corr          call atnew(i)%set_specie_from_z(H_at,elem)
!corr       enddo
!corr!
!corr!      Aggiungi H e legami al modello
!corr       call cart_to_frac(atnew(:natadd),cell%get_ortoi())
!corr       natold = size(atom)
!corr       call add_atoms_to_list(atom,atnew(:natadd),natnew)
!corr!
!corr!      Ordina gli atomi in modo che gli H compaiano dopo gli atomi a cui sono legati
!corr       allocate(iord(natnew))
!corr       call indexx(vetpos(:natnew),iord)
!corr       atom(:) = atom(iord)
!corr       call reallocate_legm(legmnew,natadd)
!corr       call combine_legm(legm,legmnew)
!corr!
!corr!      update order numbers in legm
!corr       iord(iord) = (/(i,i=1,natnew)/)   ! create index for bonds_update
!corr       call bonds_update(legm,iord)
!corr   endif
!
   end subroutine add_hydrogens

!---------------------------------------------------------------------      

   subroutine init_serialH(atom,serialH,letterH)
!
!  Inizializza variabili serialH e letterH per labellare poi gli H
!
   !USE atom_type_util
   USE atom_basic
   USE strutil
   type(atom_type), dimension(:), intent(in) :: atom
   integer, dimension(-1:), intent(out)      :: serialH,letterH
   integer, dimension(size(atom))            :: zval
   integer                                   :: i
   integer                                   :: ier
   character(len=1)                          :: lett
   integer                                   :: kser,kch
!
!  serial dice se il seriale i senza lettera e' gia' usato (es. H1 e' usato?)
   serialH(:) = 0    
!
!  letterH dice qual'e' per il seriale i l'ultima lettera usata. letter(-1) per seriale assente
   letterH(:) = ichar('A') - 1      ! ultima lettera usata
!
!  Estrai seriale e lettera dalle labels degli idrogeni esistenti
!corr   zval(:) = atomic_number(atom)
   zval(:) = atom%z()
   do i=1,size(atom)
      if (zval(i) == 1) then
          call get_from_label(atom(i)%lab,numb=kser,ierror=ier,letter=lett)
          if (kser >= 0) then
              if (kser > NMAXSERIAL) then
                  kser = kser/10**(nint(log10(real(kser)/NMAXSERIAL)))  
                  if (kser > NMAXSERIAL) kser = kser/10
              endif
              if (ch_is_alpha(lett)) then
                  lett = upper(lett)
                  kch = ichar(lett)
                  if (letterH(kser) < kch) letterH(kser)=kch
              else
                  serialH(kser) = 1
              endif
          endif
      endif
   enddo
!
   end subroutine init_serialH

!---------------------------------------------------------------------      

   subroutine set_label_hydrogen(atomH,kser,serial,letter,addletter)
   !USE atom_type_util
   USE atom_basic
   USE strutil
   type(atom_type), intent(inout)        :: atomH     ! H-atom da labellare
   integer, intent(inout)                :: kser      ! seriale dell'atomo a cui è legato l'H
   integer, dimension(-1:), intent(inout) :: serial    ! per ogni seriale se il seriale senza lettera e' stato usato
   integer, dimension(-1:), intent(inout) :: letter    ! per ogni seriale indica l'ultima lettera usata
   logical, intent(in)                   :: addletter ! se vero la label deve avere la lettera
   logical                               :: setletter
!
!  Se troppo grande tronca kser: es. 86754 diventa 8675
   if (kser > NMAXSERIAL) then
       kser = kser/10**(nint(log10(real(kser)/NMAXSERIAL)))  
       if (kser > NMAXSERIAL) kser = kser/10
   endif
!
   if (kser >= 0) then
       atomH%lab = 'H'//trim(i_to_s(kser))
   else
       atomH%lab = 'H'
   endif
!
   setletter = addletter
!
   if (.not.addletter) then
       if (serial(kser) == 0) then   ! se il numero senza lettera non è stato usato ....
           serial(kser) = 1
       else                          ! altrimenti forza una lettera
           setletter = .true.
       endif
   endif
!
   if (setletter) then
       letter(kser) = letter(kser)+1
       atomH%lab = trim(atomH%lab)//achar(letter(kser))
   endif
!
   end subroutine set_label_hydrogen

!---------------------------------------------------------------------      

   integer function atom_type_id(atom,legm,katom) result(htype)
!
!  Restituisce un codice che identifica il tipo di idrogeni legati all'atomo katom
!
   USE connect_mod
   USE atom_type_util
   USE elements
   type(atom_type), dimension(:), allocatable, intent(in) :: atom    ! gli atomi 
   type(bond_type), dimension(:), allocatable, intent(in) :: legm    ! tutti i legami
   integer, intent(in)                                    :: katom   ! atomo da considerare
   integer                                                :: zval
   integer, dimension(:), allocatable                     :: vleg,vleg2
   integer                                                :: nleg,nleg2
   integer                                                :: i,j
   logical                                                :: nfound
   integer                                                :: katom2,katom3
   integer                                                :: sumord
!
   htype = 0
   zval = atom(katom)%z()
!
!  Sono gestito solo C, O e N ammidici
   select case (zval)
      case (C_at)  ! C
        call get_bonds_of_atom(legm,katom,vleg,nleg)
        if (nleg > 0) then
            sumord = sum_order(legm(vleg))
            select case (nleg)
              case (1)    ! 1 legame
                select case (sumord)
                  case (1)
                    htype = CH3   ! carbonio metilico (sp3)
                  case (2)
                    htype = X_CH2   ! terminal X=CH2 (sp2)
                  case (3)
                    htype = C_ALCHINO   ! carbonio sp   
                end select

              case (2)     ! 2 legami
                select case (sumord)
                  case (2)   
                    if (all(legm(vleg)%ord == 4)) then  ! C in aromatic cycle, e.g. 5-member ring
                        htype = C_Ar   ! carbonio sp2
                    else
                        htype = CH2   ! carbonio sp3
                    endif
                  case (3)
                        htype = C_Ar   ! carbonio sp2
                end select
 
              case (3)    ! 3 legami
                if (sumord == 3) then
                    if (.not.any(legm(vleg)%ord > 3)) then  ! only if not aromatic add H
                        htype = CH   ! C metinico (sp3)
                    endif
                endif

            end select
        endif

      case (O_at,S_at)  ! O or S
        call get_bonds_of_atom(legm,katom,vleg,nleg)
        if (nleg == 1) then
            if (sum(legm(vleg)%ord) == 1) then
                if (legm(vleg(1))%n1 == katom) then
                    katom2 = legm(vleg(1))%n2
                else
                    katom2 = legm(vleg(1))%n1
                endif
                nfound = .false.
                if (atom(katom2)%z() == C_at) then
                    if (zval == O_at) then
                        htype = O_H
                    else
                        htype = S_H
                    endif
                endif
            endif
        endif

      case (N_at)  ! N ammidici: search for N-C=O or N-N=0 moiety
        call get_bonds_of_atom(legm,katom,vleg,nleg)
        if (nleg > 0) then
            if (nleg == 2 .and. (all(legm(vleg(:nleg))%ord == AR_SINGLE)))then
                htype = C_Ar   ! N-H in aromatic cycle (es. pyrrole)
            else
                sumord = sum_order(legm(vleg))
                if ((nleg == 1 .and. sumord == 1) .or. (nleg == 2 .and. sumord == 2)) then ! check for primary or secondary ammide/ammine
!
!                   Check for ammide
                    call get_connected_atoms(legm,katom,vleg,nleg)       
                    nfound = .false.
                    loopN:do i=1,nleg
                       katom2 = vleg(i)
                       if (atom(katom2)%z() == C_at) then  ! N legato a C
                           call get_connected_atoms(legm,katom2,vleg2,nleg2)       
                           if (nleg2 == 3 .or. nleg2 == 2) then  ! il carbonio deve formare 2-3 legami
                               do j=1,nleg2
                                  katom3 = vleg2(j)
                                  !if (atom(katom3)%z()== O_at) then      ! bond must be C=O type
                                  if (atom(katom3)%z() == O_at .or. atom(katom3)%z() == N_at) then     ! bond must be C=O or N=0 type
                                      if (legm(bond_position(legm,katom2,katom3))%ord == 2) then
                                          nfound = .true.
                                          exit loopN
                                      endif
                                  endif
                               enddo
                           endif
                       endif
                    enddo loopN
                    if (nfound) then
                        if (nleg == 1) then
                            htype = X_CH2  ! primary amide: similar to C=C-R bond
                        elseif (nleg == 2) then
                            htype = C_Ar   ! secondary amide: similar to aromatic atom
                        endif
                    else
                        if (nleg == 1) then
                            htype = N1_H
                        elseif (nleg == 2) then
                            htype = N2_H
                        endif
                    endif
                endif
            endif
        endif
        
   end select
!
   end function atom_type_id

!-------------------------------------------------------------------------------      

   integer function sum_order(legm) result(sumb)
   USE connect_mod
   type(bond_type), dimension(:), intent(in) :: legm   
   integer                                   :: i
   sumb = 0
   do i=1,size(legm)
      select case(legm(i)%ord)
        case (1:3)     ! non aromatic bond
          sumb = sumb + legm(i)%ord
        case (4:5)     ! aromatic bond
          sumb = sumb + legm(i)%ord - 3 
      end select
   enddo
   end function sum_order

!-------------------------------------------------------------------------------      

   real function distance_hydrogen(atom,code,rxdist)  result(dist)
!
!  Restituisce la distanza X-H riportata in letteratura
!
   USE atom_basic
   USE elements
   type(atom_type), intent(in)   :: atom   ! atomo
   integer, intent(in), optional :: code   ! ibridazione
   logical, intent(in), optional :: rxdist ! vuoi la distanza di raggi X
   integer                       :: coder
   logical                       :: rxdis
!
   if (present(code)) then
       coder = code
   else
       coder = 1
   endif
!
   if (present(rxdist)) then
       rxdis = rxdist
   else
       rxdis = .true.
   endif
!
   select case (atom%z())
      case (C_at)  ! C
         select case (coder)
           case (CH3,CH2,CH,C_ALCHINO,C_Ar,X_CH2)
             dist = distH(coder)
           case default 
             dist = distH(C_DEF)
         end select

      case (B_at)  ! B
         dist = distH(B_H)

      case (N_at)  ! N
         select case(coder)
           case (X_CH2,CH,CH2,N2_H)    ! ammina secondaria
             dist = distH(N2_H)
           case (CH3,N1_H)             ! ammina primaria
             dist = distH(N1_H)
           case default 
             dist = distH(N_DEF)
         end select

      case (O_at)  ! O
         dist = distH(O_H)

      case (S_at) ! S
         dist = distH(S_H)

      case default
         dist = distH(C_DEF)

   end select
!
   CONTAINS

      real function distH(code)
      integer, intent(in) :: code
!
      if (rxdis) then
          distH = distHx(code)
      else
          distH = distHn(code)
      endif
!
      end function distH
!
   end function distance_hydrogen

!-------------------------------------------------------------------------------      

   subroutine normalise_hydrogens(atom,legm,cell,ier)
   USE connect_mod
   USE atom_type_util
   USE unit_cell
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   type(cell_type), intent(in)                               :: cell
   integer, intent(out)                                      :: ier
   type(atom_type), dimension(:), allocatable                :: atomc
   type(bond_type), dimension(:), allocatable                :: legmc
   integer                                                   :: nleg
   integer                                                   :: i
   integer, dimension(size(atom))                            :: zval
   integer                                                   :: htype
   integer                                                   :: nH,nX
   real                                                      :: distXH
   integer                                                   :: nleg0
   real, dimension(3,3) :: ortoinv
!
   ier = 0
!
   nleg = numbonds(legm)
   if (nleg == 0) then
       ier = 1
       return
   endif
!
!corr   zval(:) = atomic_number(atom)
   zval(:) = atom%z()
!
!  Isola i legami senza H da usare per calcolare l'htype
   call resize_bonds(legmc,nleg)
   nleg0 = 0
   do i=1,nleg
      if (zval(legm(i)%n1) /= 1 .and. zval(legm(i)%n2) /= 1) then
          nleg0 = nleg0 + 1
          legmc(nleg0) = legm(i)
      endif
   enddo
   if (nleg == nleg0) then
       ier = 2
       return  ! esco perche' non ci sono H
   endif
   call resize_bonds(legmc,nleg0)
!
!  conversione in coord. cartesiane
!corr   call frac_to_cart_copy(atom,atomc,orthomatrix(cell))
   call frac_to_cart_copy(atom,atomc,cell%get_ortom())
!
!corr   ortoinv = orthomatrixi(cell) ! usato da fractional_coord
   ortoinv = cell%get_ortoi() ! usato da fractional_coord
   do i=1,numbonds(legm)
      nH = 0
      if (zval(legm(i)%n1) == 1) then
          nH = legm(i)%n1
          nX = legm(i)%n2
      elseif (zval(legm(i)%n2) == 1) then
          nH = legm(i)%n2
          nX = legm(i)%n1
      endif
      if (nH > 0) then
!
!         Determina l'htype e la distanza a cui portare nH
          htype = atom_type_id(atomc,legmc,nX)    ! N:B. vuole legmc in input e non legm
          distXH = distance_hydrogen(atom(nX),htype,rxdist=.false.)
!
          call set_distance(atomc(nX)%xc,atomc(nH)%xc,distXH)   ! modifica la distanza di nH
          atom(nH) = fractional_coord(atomc(nH),ortoinv)        
          legm(i)%dist = distXH
          !write(0,*)'norm:',trim(atomc(nH)%lab)//'-'//trim(atomc(nX)%lab),htype,distXH
      endif
   enddo
!
   end subroutine normalise_hydrogens

!-------------------------------------------------------------------------------      

   subroutine set_distance(x1,x2,dist)
   USE cgeom
   real, dimension(3), intent(in)    :: x1
   real, dimension(3), intent(inout) :: x2
   real                              :: dist
!
   x2 = x1 + (x2 - x1)*dist/distanzac(x1,x2)
!
   end subroutine set_distance

!-------------------------------------------------------------------------------      

   subroutine save_hydrogens(atom,atomH)
!
!  Salva in atomH solo gli H
!
   USE atom_type_util
   type(atom_type), dimension(:), allocatable, intent(in)  :: atom
   type(atom_type), dimension(:), allocatable, intent(out) :: atomH
   integer, dimension(:), allocatable                      :: zval
   integer                                                 :: nat
   integer                                                 :: i
   integer                                                 :: nh
!
   nat = numatoms(atom)
   if (nat == 0) return
!
   allocate(zval(nat)) 
!corr   zval(:) = atomic_number(atom)
   zval(:) = atom%z()
   nh = count(zval(:) == 1) ! numero di H
   if (nh == 0) return     
   allocate(atomH(nh))
!
!  Ora copia in atomH solo gli H
   nh = 0
   do i=1,nat
      if (zval(i) == 1) then
          nh = nh + 1
          atomH(nh) = atom(i)
      endif
   enddo
!
   end subroutine save_hydrogens

!-------------------------------------------------------------------------------      

   subroutine update_hydrogens(atom,atomH)
!
!  Aggiorna in atom solo gli H
!
   USE atom_type_util
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(atom_type), dimension(:), allocatable, intent(in)    :: atomH
   integer, dimension(:), allocatable                        :: zval
   integer                                                   :: nat
   integer                                                   :: i
   integer                                                   :: nh
!   
   nat = numatoms(atom)
   if (nat == 0) return
   if (numatoms(atomH) == 0) return
!
   allocate(zval(nat)) 
!corr   zval(:) = atomic_number(atom)
   zval(:) = atom%z()
!
   nh = 0
   do i=1,nat
      if (zval(i) == 1) then
          nh = nh + 1
          atom(i) = atomH(nh)
      endif
   enddo
!
   end subroutine update_hydrogens

!----------------------------------------------------------------------------------------------------

   subroutine new_acetilenic_atom(atnew,legn,kat,atom,conn,dist0,cell,cart,ier)
!
!  Add metinic atom
!
   USE atom_type_util
   USE connect_mod
   USE unit_cell
   USE cgeom
   USE arrayutil
   type(atom_type), intent(out)                    :: atnew
   type(bond_type), intent(out)                    :: legn
   integer, intent(in)                             :: kat
   type(atom_type), dimension(:), intent(in)       :: atom
   type(container_type), dimension(:), intent(in) :: conn
   real, intent(in)                                :: dist0
   type(cell_type), intent(in)                     :: cell
   logical, intent(in)                             :: cart
   integer, intent(out)                            :: ier
   type(atom_type), dimension(:), allocatable      :: atomc
   integer                                         :: k1

   if (conn(kat)%nat == 1) then
       ier = 0
       k1 = conn(kat)%pos(1)
!
       if (.not.cart) then
           call frac_to_cart_copy(atom([k1,kat]),atomc,cell%get_ortom())
       else
           allocate(atomc(2),source=atom([k1,kat]))
       endif
!
!      aggiungi atomo lungo la retta kat-k1 a distanza dist0
       atnew%xc = atomc(2)%xc - (atomc(1)%xc - atomc(2)%xc)*dist0/distanzac(atomc(1)%xc,atomc(2)%xc)
       legn = bond_type(kat,size(atom)+1,dist0)
   else
       ier = 1
   endif
   end subroutine new_acetilenic_atom

!----------------------------------------------------------------------------------------------------

   subroutine new_metinic_atom(atnew,legn,kat,atom,conn,dist0,cell,cart,ier)
!
!  Add metinic atom
!
   USE atom_type_util
   USE connect_mod
   USE unit_cell
   USE cgeom
   USE arrayutil
   type(atom_type), intent(out)                    :: atnew
   type(bond_type), intent(out)                    :: legn
   integer, intent(in)                             :: kat
   type(atom_type), dimension(:), intent(in)       :: atom
   type(container_type), dimension(:), intent(in) :: conn
   real, intent(in)                                :: dist0
   type(cell_type), intent(in)                     :: cell
   logical, intent(in)                             :: cart
   integer, intent(out)                            :: ier
   type(atom_type), dimension(:), allocatable      :: atomc
   real, dimension(4)                              :: vp
   real :: distp
   integer :: k1,k2,k3,nat
   real, dimension(3) :: xatb
      real :: tangle1,tangle,angH

   if (conn(kat)%nat == 3) then
!
!      Controlla che kat non giaccia nel piano k1-k2-k3
       k1 = conn(kat)%pos(1)
       k2 = conn(kat)%pos(2)
       k3 = conn(kat)%pos(3)
!
       if (.not.cart) then
           call frac_to_cart_copy(atom([k1,k2,k3,kat]),atomc,cell%get_ortom())
       else
           allocate(atomc(4),source=atom([k1,k2,k3,kat]))
       endif
!
!      piano passante per k1,k2,k3
       vp(:) = plane3points(atomc(1)%xc,atomc(2)%xc,atomc(3)%xc)
!
!      distanza kat-piano
       distp = abs(dot_product(vp(:3),atomc(4)%xc)+vp(4))
       if (distp > 0.1) then ! kat puo' considerarsi fuori dal piano
           ier = 0
!
!          Calcola il baricentro dei 3 atomi connessi a kat
           xatb(:) = (atomc(1)%xc(:) + atomc(2)%xc(:) + atomc(3)%xc(:))/3
!
           nat = size(atom)
!
!          Disegna l'atomo di H lungo la linea che congiunge il baricentro a kat
           !atnew%xc = atomc(4)%xc - (xatb(:) - atomc(4)%xc)*dist0/distanzac(xatb,atomc(4)%xc)
!!!!!!!!new
           tangle1 = Angle_Dihedral(atomc(3)%xc,atomc(4)%xc,atomc(1)%xc,atomc(2)%xc)
           !write(0,*)'TANGLE=',tangle1,atomc(3)%lab,atomc(4)%lab,atomc(1)%lab,atomc(2)%lab
           if (tangle1 < 0) then
               tangle = -(360 +tangle1) / 2.0
           else
               tangle = (360 -tangle1) / 2.0
           endif
           !write(0,*)'TANGLE new atom=',tangle,dist0
           angH = 109.47
           call Get_Cartesian_from_Z([dist0,angH,tangle],atnew%xc,atomc(4)%xc,atomc(1)%xc,atomc(2)%xc)
           !write(0,*)'new atom=',atnew%xc
           tangle1 = Angle_Dihedral(atnew%xc,atomc(4)%xc,atomc(1)%xc,atomc(2)%xc)
           !write(0,*)'TANGLE=',tangle1,atomc(4)%lab,atomc(1)%lab,atomc(2)%lab
!!!!!!!!new
           legn = bond_type(kat,nat+1,dist0)
       else
           ier = 1    ! kat nel piano: kat potrebbe essere sp2
       endif
   else
       ier = 1
   endif
   end subroutine new_metinic_atom

!----------------------------------------------------------------------------------------------------

   subroutine new_metilenic_atom(atnew,legn,kat,atom,conn,dist0,cell,cart,ier)
!
!  Add metilenic atom
!
   USE atom_type_util
   USE connect_mod
   USE unit_cell
   USE cgeom
   USE arrayutil
   type(atom_type), dimension(:), intent(out)      :: atnew
   type(bond_type), dimension(:), intent(out)      :: legn
   integer, intent(in)                             :: kat
   type(atom_type), dimension(:), intent(in)       :: atom
   type(container_type), dimension(:), intent(in) :: conn
   real, intent(in)                                :: dist0
   type(cell_type), intent(in)                     :: cell
   logical, intent(in)                             :: cart
   integer, intent(out)                            :: ier
   type(atom_type), dimension(:), allocatable      :: atomc
   real                                            :: ang,angH
   real, dimension(3)                              :: ci,xatb
   integer                                         :: k1,k2,k3,nat
!
   if (conn(kat)%nat == 2) then
       k1 = conn(kat)%pos(1)
       k2 = kat
       k3 = conn(kat)%pos(2)
       if (.not.cart) then
           call frac_to_cart_copy(atom([k1,k2,k3]),atomc,cell%get_ortom())
       else
           allocate(atomc(3),source=atom([k1,k2,k3]))
       endif
!
!      Genera atomo atb lungo la bisettrice
       ang = rtod*angleC(atomc(1)%xc,atomc(2)%xc,atomc(3)%xc)                         
       angH = (360-ang) / 2  
       ci(:) = (/0.5,angH,180./)
       call Get_Cartesian_from_Z(ci,xatb,atomc(2)%xc,atomc(1)%xc,atomc(3)%xc)
!
       nat = size(atom)
!
!      Genara il primo H
       angH = 109.47  ! angolo tra i 2 H
       ci(:) = (/dist0,angH/2.0,90./)
       call Get_Cartesian_from_Z(ci,atnew(1)%xc,atomc(2)%xc,xatb,atomc(3)%xc)
       legn(1) = bond_type(kat,nat+1,dist0)
       if (size(atnew) > 1) then
!
!          Genara il secondo H
           angH = 109.47  ! angolo tra i 2 H
           ci(:) = (/dist0,angH/2.0,-90./)
           call Get_Cartesian_from_Z(ci,atnew(2)%xc,atomc(2)%xc,xatb,atomc(3)%xc)
           legn(2) = bond_type(kat,nat+2,dist0)
       endif
       ier = 0
   else
       ier = 1
   endif
   end subroutine new_metilenic_atom

!----------------------------------------------------------------------------------------------------

   subroutine new_metilic_atom(atnew,legn,kat,atom,conn,dist0,cell,cart,ier)
!
!  Add metilic atom
!
   USE connect_mod
   USE trig_constants, only:rtod
   USE cgeom
   USE atom_type_util
   USE unit_cell
   USE arrayutil
   type(atom_type), dimension(:), intent(out)      :: atnew
   type(bond_type), dimension(:), intent(out)      :: legn
   integer, intent(in)                             :: kat
   type(atom_type), dimension(:), intent(in)       :: atom
   type(container_type), dimension(:), intent(in) :: conn
   real, intent(in)                                :: dist0
   type(cell_type), intent(in)                     :: cell
   logical, intent(in)                             :: cart
   integer, intent(out)                            :: ier
   integer                                         :: k1,k2
   real                                            :: angH
   real, dimension(3)                              :: ci
   type(atom_type), dimension(:), allocatable      :: atomc,atomc1
   real                                            :: tangle
   integer                                         :: nat
   real, dimension(3)                              :: xatb
   integer, dimension(2)                           :: k11
   integer                                         :: j,kleg
!
   if (conn(kat)%nat == 1) then
       angH = 109.47
       k1 = conn(kat)%pos(1)
       k2 = kat
       if (.not.cart) then
           call frac_to_cart_copy(atom([k1,k2]),atomc,cell%get_ortom())
       else
           allocate(atomc(2),source=atom([k1,k2]))
       endif
!
!      Calcola angolo di torsione tale da avere configurazione sfalsata
       select case (conn(k1)%nat)

          case (1)  ! k1 forma solo un legame o più legami (caso non gestibile)
!
!           crea un atomo e una torsione qualunque (es. 180)
            xatb(:) = (/0.5,0.5,0.5/)
            tangle = 180.0
                    
          case (2:)   ! k1 forma 2 o piu' legami
            kleg = 0 
            do j=1,conn(k1)%nat
               if (conn(k1)%pos(j) /= kat) then
                   kleg = kleg + 1
                   k11(kleg) = conn(k1)%pos(j)
               endif
               if (kleg == 2) exit
            enddo
            if (kleg == 1) then
                tangle = 180.0
                if (.not.cart) then
                    call frac_to_cart_copy(atom([k11(1)]),atomc1,cell%get_ortom())
                else
                    allocate(atomc1(1),source=atom([k11(1)]))
                endif
            else
                if (.not.cart) then
                    call frac_to_cart_copy(atom([k11(1),k11(2)]),atomc1,cell%get_ortom())
                else
                    allocate(atomc1(2),source=atom([k11(1),k11(2)]))
                endif
                tangle = Angle_Dihedral(atomc1(1)%xc,atomc(1)%xc,atomc(2)%xc,atomc1(2)%xc)
!
!               tangle = 180: atomi complanari o quasi. non fare niente
                !if (abs(abs(tangle) - 180) >= 2) then  
                if (abs(abs(tangle) - 180) >= 5) then  
                    tangle = tangle / 2    ! per sfalsare la configurazione
                else
                    tangle = (180. - angH) / 2  ! case of aniline
                endif
            endif
            xatb(:) = atomc1(1)%xc

       end select
       nat = size(atom)
!
!      Primo H
       ci(:) = (/dist0,angH,tangle/)
       call Get_Cartesian_from_Z(ci,atnew(1)%xc,atomc(2)%xc,atomc(1)%xc,xatb)
       legn(1) = bond_type(kat,nat+1,dist0)
       if (size(atnew) > 1) then
!
!          Genera il secondo H dal primo
           ci(:) = (/dist0,angH,120./)
           call Get_Cartesian_from_Z(ci,atnew(2)%xc,atomc(2)%xc,atomc(1)%xc,atnew(1)%xc)
           legn(2) = bond_type(kat,nat+2,dist0)
           if (size(atnew) > 2) then
!
!              Genera il terzo H dal primo
               ci(:) = (/dist0,angH,-120./)
               call Get_Cartesian_from_Z(ci,atnew(3)%xc,atomc(2)%xc,atomc(1)%xc,atnew(1)%xc)
               legn(3) = bond_type(kat,nat+3,dist0)
            endif
       endif
       ier = 0
   else
       ier = 1
   endif
!
   end subroutine new_metilic_atom

!----------------------------------------------------------------------------------------------------

   subroutine new_aminic_atom(atnew,legn,kat,atom,conn,dist0,cell,cart,legH,atH,ier)
!
!  Add metilic atom
!
   USE connect_mod
   USE trig_constants, only:rtod
   USE cgeom
   USE atom_type_util
   USE unit_cell
   USE arrayutil
   type(atom_type), dimension(:), intent(out)      :: atnew
   type(bond_type), dimension(:), intent(out)      :: legn
   integer, intent(in)                             :: kat
   type(atom_type), dimension(:), intent(in)       :: atom
   type(container_type), dimension(:), intent(in) :: conn
   real, intent(in)                                :: dist0
   type(cell_type), intent(in)                     :: cell
   logical, intent(in)                             :: cart
   type(bond_type), intent(in)                     :: legH
   type(atom_type), intent(in)                     :: atH
   integer, intent(out)                            :: ier
!corr   integer                                         :: k1,k2
!corr   real                                            :: angH
!corr   real, dimension(3)                              :: ci
!corr   type(atom_type), dimension(:), allocatable      :: atomc,atomc1
!corr   real                                            :: tangle
!corr   integer                                         :: nat
!corr   real, dimension(3)                              :: xatb
!corr   integer, dimension(2)                           :: k11
!corr   integer                                         :: j,kleg
   type(container_type), dimension(:), allocatable :: conn1
   type(atom_type), dimension(:), allocatable :: atom1
!
   ier = 0
   if (conn(kat)%nat == 1) then
!!!TODO: check the angle in ammine
!           write(0,*)'H bonds:',legH%n1
       if (legH%n1 > 0) then    ! H-bond is present
!
!          add first hydrogen along H-bond
           call new_oxydrilic_atom(atnew(1),legn(1),kat,atom,conn,dist0,cell,.true.,legH,atH,ier)
           if (ier == 0) then
!
!              add new atom to atom/conn
               allocate(conn1(size(conn)),source=conn)
               call connect_update_add(conn1,legn(:1))
               allocate(atom1(size(conn)),source=atom)
               call push_back_atom(atom1,atnew(1))
               call new_metilenic_atom(atnew(2:2),legn(2:2),kat,atom1,conn1,dist0,cell,.true.,ier) !add 2th atom
           endif
       else
           call new_metilic_atom(atnew,legn,kat,atom,conn,dist0,cell,cart,ier)
       endif
!corr       return
!corr       angH = 109.47   !!!!TODO: check this angle 
!corr       k1 = conn(kat)%pos(1)
!corr       k2 = kat
!corr       if (.not.cart) then
!corr           call frac_to_cart_copy(atom([k1,k2]),atomc,cell%get_ortom())
!corr       else
!corr           allocate(atomc(2),source=atom([k1,k2]))
!corr       endif
!corr!
!corr!      Calcola angolo di torsione tale da avere configurazione sfalsata
!corr       select case (conn(k1)%nat)
!corr
!corr          case (1)  ! k1 forma solo un legame o più legami (caso non gestibile)
!corr!
!corr!           crea un atomo e una torsione qualunque (es. 180)
!corr            xatb(:) = (/0.5,0.5,0.5/)
!corr            tangle = 180.0
!corr                    
!corr          case (2:)   ! k1 forma 2 o piu' legami
!corr            kleg = 0 
!corr            do j=1,conn(k1)%nat
!corr               if (conn(k1)%pos(j) /= kat) then
!corr                   kleg = kleg + 1
!corr                   k11(kleg) = conn(k1)%pos(j)
!corr               endif
!corr               if (kleg == 2) exit
!corr            enddo
!corr            if (kleg == 1) then
!corr                tangle = 180.0
!corr                if (.not.cart) then
!corr                    call frac_to_cart_copy(atom([k11(1)]),atomc1,cell%get_ortom())
!corr                else
!corr                    allocate(atomc1(1),source=atom([k11(1)]))
!corr                endif
!corr            else
!corr                if (.not.cart) then
!corr                    call frac_to_cart_copy(atom([k11(1),k11(2)]),atomc1,cell%get_ortom())
!corr                else
!corr                    allocate(atomc1(2),source=atom([k11(1),k11(2)]))
!corr                endif
!corr                tangle = Angle_Dihedral(atomc1(1)%xc,atomc(1)%xc,atomc(2)%xc,atomc1(2)%xc)
!corr!
!corr!               tangle = 180: atomi complanari o quasi. non fare niente
!corr                if (abs(abs(tangle) - 180) >= 5) then  
!corr                    tangle = tangle / 2         ! per sfalsare la configurazione
!corr                else
!corr                    tangle = (180. - angH) / 2  ! case of aniline
!corr                endif
!corr            endif
!corr            xatb(:) = atomc1(1)%xc
!corr
!corr       end select
!corr       nat = size(atom)
!corr!
!corr!      Primo H
!corr       ci(:) = (/dist0,angH,tangle/)
!corr       call Get_Cartesian_from_Z(ci,atnew(1)%xc,atomc(2)%xc,atomc(1)%xc,xatb)
!corr       legn(1) = bond_type(kat,nat+1,dist0)
!corr       if (size(atnew) > 1) then
!corr!
!corr!          Genera il secondo H dal primo
!corr           ci(:) = (/dist0,angH,120./)
!corr           call Get_Cartesian_from_Z(ci,atnew(2)%xc,atomc(2)%xc,atomc(1)%xc,atnew(1)%xc)
!corr           legn(2) = bond_type(kat,nat+2,dist0)
!corr           if (size(atnew) > 2) then  !quaternary ammonium cation
!corr!
!corr!              Genera il terzo H dal primo
!corr               ci(:) = (/dist0,angH,-120./)
!corr               call Get_Cartesian_from_Z(ci,atnew(3)%xc,atomc(2)%xc,atomc(1)%xc,atnew(1)%xc)
!corr               legn(3) = bond_type(kat,nat+3,dist0)
!corr            endif
!corr       endif
   elseif (conn(kat)%nat == 2) then
       call new_metilenic_atom(atnew,legn,kat,atom,conn,dist0,cell,cart,ier)
   else
       ier = 1
   endif
!
   end subroutine new_aminic_atom

!----------------------------------------------------------------------------------------------------

   subroutine new_aromatic_atom(atnew,legn,kat,atom,conn,dist0,cell,cart,ier)
!
!  Add aromatic atom
!
   USE connect_mod
   USE trig_constants, only:rtod
   USE cgeom
   USE atom_type_util
   USE unit_cell
   USE arrayutil
   type(atom_type), intent(out)                    :: atnew
   type(bond_type), intent(out)                    :: legn
   integer, intent(in)                             :: kat
   type(atom_type), dimension(:), intent(in)       :: atom
   type(container_type), dimension(:), intent(in) :: conn
   real, intent(in)                                :: dist0
   type(cell_type), intent(in)                     :: cell
   logical, intent(in)                             :: cart
   integer, intent(out)                            :: ier
   integer                                         :: k1,k2,k3
   real                                            :: ang,angH
   real, dimension(3)                              :: ci
   type(atom_type), dimension(:), allocatable      :: atomc
!
   if (conn(kat)%nat == 2) then
       ier = 0
!
!      Calcola l'angolo al quale aggiungere H
       k1 = conn(kat)%pos(1)
       k2 = kat
       k3 = conn(kat)%pos(2)
       if (.not.cart) then
           call frac_to_cart_copy(atom([k1,k2,k3]),atomc,cell%get_ortom())
       else
           allocate(atomc(3),source=atom([k1,k2,k3]))
       endif
       ang = rtod*angleC(atomc(1)%xc,atomc(2)%xc,atomc(3)%xc)
       angH = (360-ang) / 2  ! il legame con H va lungo la bisettrice dell'ang K1K2K3
!
!      Genera l'atomo e il legame
       ci(:) = (/dist0,angH,180./)
       call Get_Cartesian_from_Z(ci,atnew%xc,atomc(2)%xc,atomc(1)%xc,atomc(3)%xc)
       legn = bond_type(kat,size(atom)+1,dist0)
       !atnew%inte = atom(kat)%inte/atomic_number(atom(kat))
       !atnew%biso = 1.2*atom(kat)%biso
   else
       ier = 1
   endif
!
   end subroutine new_aromatic_atom

!----------------------------------------------------------------------------------------------------
  
   subroutine new_oxydrilic_atom(atnew,legn,kat,atom,conn,dist0,cell,cart,legH,atH,ier)
!
!  Add oxydrilic atom
!
   USE connect_mod
   USE trig_constants, only:rtod
   USE cgeom
   USE atom_type_util
   USE unit_cell
   USE arrayutil
   type(atom_type), intent(out)                    :: atnew
   type(bond_type), intent(out)                    :: legn
   integer, intent(in)                             :: kat
   type(atom_type), dimension(:), intent(in)       :: atom
   type(container_type), dimension(:), intent(in) :: conn
   real, intent(in)                                :: dist0
   type(cell_type), intent(in)                     :: cell
   logical, intent(in)                             :: cart
   type(bond_type), intent(in)                     :: legH
   type(atom_type), intent(in)                     :: atH
   integer, intent(out)                            :: ier
   real                                            :: angH
   integer                                         :: k1,k2,k3
   type(atom_type), dimension(:), allocatable      :: atomc
   integer                                         :: j
   real                                            :: tangle
   real, dimension(3)                              :: xatb
   integer                                         :: kleg
   integer, dimension(2)                           :: k11
   real, dimension(3)                              :: ci
!
   if (conn(kat)%nat == 1) then
       angH = 109.47   ! angolo X-O-H
       k1 = conn(kat)%pos(1)
       k2 = kat
       if (legH%n1 > 0) then    ! legame idrogeno presente
           k3 = 0
           do j=1,conn(k1)%nat
              if (conn(k1)%pos(j) /= kat) then
                  k3 = conn(k1)%pos(j)
              endif
           enddo
           if (k3 > 0) then
               if (.not.cart) then
                   call frac_to_cart_copy(atom([k1,k2,k3]),atomc,cell%get_ortom())
               else
                   allocate(atomc(3),source=atom([k1,k2,k3]))
               endif
               !tangle = Angle_Dihedral(atomc(3)%xc,atomc(1)%xc,atomc(2)%xc,atH%xc)
               tangle = -Angle_Dihedral(atH%xc,atomc(2)%xc,atomc(1)%xc,atomc(3)%xc)
               xatb(:) = atomc(3)%xc
           else   ! this force new atom in the plane k1-k2-atH
               if (.not.cart) then
                   call frac_to_cart_copy(atom([k1,k2]),atomc,cell%get_ortom())
               else
                   allocate(atomc(2),source=atom([k1,k2]))
               endif
               xatb(:) = atH%xc  
               tangle = 0.0
           endif
       else
!
!          Calcola angolo di torsione tale da avere configurazione sfalsata
           select case (conn(k1)%nat)

              case (1)  ! k1 forma solo un legame 
                if (.not.cart) then
                    call frac_to_cart_copy(atom([k1,k2]),atomc,cell%get_ortom())
                else
                    allocate(atomc(2),source=atom([k1,k2]))
                endif
!
!               crea un atomo e una torsione qualunque (es. 180)
                xatb(:) = (/0.5,0.5,0.5/)
                tangle = 180.0
           
              case (2:)   ! k1 forma 2 o piu' legami
                kleg = 0 
                do j=1,conn(k1)%nat
                   if (conn(k1)%pos(j) /= kat) then
                       kleg = kleg + 1
                       k11(kleg) = conn(k1)%pos(j)
                   endif
                   if (kleg == 2) exit
                enddo
                if (kleg == 1) then
                    if (.not.cart) then
                        call frac_to_cart_copy(atom([k1,k2,k11(1)]),atomc,cell%get_ortom())
                    else
                        allocate(atomc(3),source=atom([k1,k2,k11(1)]))
                    endif
                    tangle = 180.0
                else
                    if (.not.cart) then
                        call frac_to_cart_copy(atom([k1,k2,k11(1),k11(2)]),atomc,cell%get_ortom())
                    else
                        allocate(atomc(4),source=atom([k1,k2,k11(1),k11(2)]))
                    endif
                    tangle = Angle_Dihedral(atomc(3)%xc,atomc(1)%xc,atomc(2)%xc,atomc(4)%xc)
!
!                   tangle = 180: atomi complanari o quasi. non fare niente
                    if (abs(abs(tangle) - 180) >= 2) then  
                         tangle = tangle / 2    ! per sfalsare la configurazione
                    endif
                endif
                xatb(:) = atomc(3)%xc
           end select
       endif
       ci(:) = (/dist0,angH,tangle/)
       call Get_Cartesian_from_Z(ci,atnew%xc,atomc(2)%xc,atomc(1)%xc,xatb)
       legn = bond_type(kat,size(atom)+1,dist0)
       ier = 0
   else
       ier = 1
   endif
!
   end subroutine new_oxydrilic_atom

!----------------------------------------------------------------------------------------------------

   subroutine new_doubleb_atom(atnew,legn,kat,atom,conn,dist0,cell,cart,ier)
!
!  Add metilic atom
!
   USE connect_mod
   USE trig_constants, only:rtod
   USE cgeom
   USE atom_type_util
   USE unit_cell
   USE arrayutil
   type(atom_type), dimension(:), intent(out)      :: atnew
   type(bond_type), dimension(:), intent(out)      :: legn
   integer, intent(in)                             :: kat
   type(atom_type), dimension(:), intent(in)       :: atom
   type(container_type), dimension(:), intent(in) :: conn
   real, intent(in)                                :: dist0
   type(cell_type), intent(in)                     :: cell
   logical, intent(in)                             :: cart
   integer, intent(out)                            :: ier
   integer                                         :: k1,k2
   real                                            :: angH
   real, dimension(3)                              :: ci
   type(atom_type), dimension(:), allocatable      :: atomc,atomc1
   real                                            :: tangle
   integer                                         :: nat
   real, dimension(3)                              :: xatb
   integer, dimension(2)                           :: k11
   integer                                         :: j,kleg
!
   if (conn(kat)%nat == 1) then
       angH = 109.47
       k1 = conn(kat)%pos(1)
       k2 = kat
       if (.not.cart) then
           call frac_to_cart_copy(atom([k1,k2]),atomc,cell%get_ortom())
       else
           allocate(atomc(2),source=atom([k1,k2]))
       endif
!
!      Calcola angolo di torsione tale da avere configurazione sfalsata
       select case (conn(k1)%nat)

          case (1)  ! k1 forma solo un legame o più legami (caso non gestibile)
!
!           crea un atomo e una torsione qualunque (es. 180)
            xatb(:) = (/0.5,0.5,0.5/)
            tangle = 180.0
                    
          case (2:)   ! k1 forma 2 o piu' legami
            kleg = 0 
            do j=1,conn(k1)%nat
               if (conn(k1)%pos(j) /= kat) then
                   kleg = kleg + 1
                   k11(kleg) = conn(k1)%pos(j)
               endif
               if (kleg == 2) exit
            enddo
            if (kleg == 1) then
                tangle = 180.0
                if (.not.cart) then
                    call frac_to_cart_copy(atom([k11(1)]),atomc1,cell%get_ortom())
                else
                    allocate(atomc1(1),source=atom([k11(1)]))
                endif
            else
                if (.not.cart) then
                    call frac_to_cart_copy(atom([k11(1),k11(2)]),atomc1,cell%get_ortom())
                else
                    allocate(atomc1(2),source=atom([k11(1),k11(2)]))
                endif
                tangle = Angle_Dihedral(atomc1(1)%xc,atomc(1)%xc,atomc(2)%xc,atomc1(2)%xc)
!
!               tangle = 180: atomi complanari o quasi. non fare niente
                if (abs(abs(tangle) - 180) >= 2) then  
                    tangle = tangle / 2    ! per sfalsare la configurazione
                endif
            endif
            xatb(:) = atomc1(1)%xc

       end select
       nat = size(atom)
!
!      Primo H
       ci(:) = (/dist0,angH,tangle/)
       call Get_Cartesian_from_Z(ci,atnew(1)%xc,atomc(2)%xc,atomc(1)%xc,xatb)
       legn(1) = bond_type(kat,nat+1,dist0)
       if (size(atnew) > 1) then
!
!          Genera il secondo H dal primo
           ci(:) = (/dist0,angH,120./)
           call Get_Cartesian_from_Z(ci,atnew(2)%xc,atomc(2)%xc,atomc(1)%xc,atnew(1)%xc)
           legn(2) = bond_type(kat,nat+2,dist0)
           if (size(atnew) > 2) then
!
!              Genera il terzo H dal primo
               ci(:) = (/dist0,angH,-120./)
               call Get_Cartesian_from_Z(ci,atnew(3)%xc,atomc(2)%xc,atomc(1)%xc,atnew(1)%xc)
               legn(3) = bond_type(kat,nat+3,dist0)
            endif
       endif
       ier = 0
   else
       ier = 1
   endif
!
   end subroutine new_doubleb_atom

!----------------------------------------------------------------------------------------------------

   subroutine add_atom_bound_to(kat,atom,legm,cell,elem,kel,btype,ier)
!   
!  Add new atom connected to kat 
!
   USE connect_mod
   USE atom_type_util
   USE unit_cell
   USE bondtmod
   USE elements
   USE atom_basic
   USE arrayutil
   integer, intent(in)                                       :: kat   ! bound atom to kat atom
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom  ! list of atoms
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm  ! list of bonds
   type(cell_type), intent(in)                               :: cell
   type(element_type), dimension(:), allocatable, intent(in)              :: elem
   integer, intent(in)                                       :: kel   ! element type to add
   integer, intent(in)                                       :: btype ! bond type to add
   integer, intent(out)                                      :: ier
   type(container_type), dimension(:), allocatable :: conn
   integer :: idat, natn
   real :: dist0
   type(atom_type), dimension(1) :: atnew
   type(bond_type), dimension(1) :: legn
   integer :: ierd
   type(bond_type)                      :: legH
   type(atom_type)                      :: atH
!
   ier = 0
!  FIXME - gestire nleg = 0 e ghost atoms
!
!  Identify atom type
   call bond_type_perception(atom,legm,cell)  !FIXME - apply only on frag containing kat or check if already applied
   idat = atom_type_id(atom,legm,kat)  
   call bond_to_connect(size(atom),legm,conn)
   if (idat == 0) then
       if (atom(kat)%z() == N_at) then
           if (conn(kat)%nat == 3) idat = CH         !quaternary ammonium cation
       endif
   endif
   
!corr          btype = 1
!corr   call find_distance(atomic_number(atom(kat)),elem(kel)%z,btype,dist0,ierd)
   call find_distance(atom(kat)%z(),elem(kel)%z,btype,dist0,ierd)
            !write(0,*)'type=',idat,'dist=',dist0
   if (ierd /= 0) dist0 = 1.4   !!!FIXME - find better solution
   select case (idat)
     case (C_ALCHINO)
       call new_acetilenic_atom(atnew(1),legn(1),kat,atom,conn,dist0,cart=.false.,cell=cell,ier=ier)
     case (CH)
       call new_metinic_atom(atnew(1),legn(1),kat,atom,conn,dist0,cart=.false.,cell=cell,ier=ier)
     case (CH2)
       call new_metilenic_atom(atnew,legn,kat,atom,conn,dist0,cart=.false.,cell=cell,ier=ier)
     case (CH3)
       call new_metilic_atom(atnew,legn,kat,atom,conn,dist0,cart=.false.,cell=cell,ier=ier)
     case (C_Ar)
       call new_aromatic_atom(atnew(1),legn(1),kat,atom,conn,dist0,cart=.false.,cell=cell,ier=ier)
     case (O_H,S_H)
       call new_oxydrilic_atom(atnew(1),legn(1),kat,atom,conn,dist0,cell,.false.,legH,atH,ier)
     case (N1_H,N2_H)
       call new_aminic_atom(atnew,legn,kat,atom,conn,dist0,cell,.false.,legH,atH,ier)
     case default
       ier = 1
   end select
!
   if (ier == 0) then
       atnew(1) = fractional_coord(atnew(1),cell%get_ortoi())
       call add_atom(atom,atnew(1),legm,legn)
       natn = size(atom)
       !corr call set_specie(natn,atom,kel,elem)
       call atom(natn)%set_specie_from_el(kel,elem)
       call atom_string(atom,elem,[natn])
       call set_biso(atom(natn))
       call set_intensity(atom(natn))
   endif
!
   end subroutine add_atom_bound_to

END MODULE hydrogens
