#define GTK_DISABLE_DEPRECATED 1

#include <gtk/gtk.h>

#include <cstdio>
#include <time.h>
#include <cmath>
#include <vector>
#include <string>
#include <algorithm>
#include <functional>

#ifdef _WIN32
#include <sys/timeb.h>
#else
#include <sys/time.h>
#endif

#define INFO_WINDOW            1
#define QUEST_WINDOW           2
#define QUEST_WINDOW_R         3         
#define QUEST_WINDOW_YES_NO    4         
#define QUEST_WINDOW_NO_YES    5         
#define THREE_BUTTONS          6         
#define WARN_WINDOW            7         
#define ERR_WINDOW             8         
#define SEVERE_ERR_WINDOW      9  
#define PERCENTO '%'

#define DHPerResidue      8.0  // Average number of Hydrogen atoms per residue
#define DCPerResidue      5.0  // Average number of Carbon atoms per residue
#define DNPerResidue      1.35 // Average number of Nitrogen atoms per residue
#define DOPerResidue      1.5  // Average number of Oxygen atoms per residue
#define DSPerResidue      0.05 // Average number of Sulphur atoms per residue

#define MaxLen          150
#define DResidueWeight  108.7  // Weight of one residue in Daltons

#include <sys/stat.h>
#include <cctype>
#ifdef _WIN32
#include <direct.h>
#include <io.h>

#ifdef _MSC_VER
typedef int mode_t;
#endif
    const mode_t tXt = _S_IREAD | _S_IWRITE | _S_IEXEC;
    const mode_t rwXt = _S_IREAD | _S_IWRITE | _S_IEXEC;
inline void ChangeMode  (const char *file, mode_t mode)
{
    _chmod (file, mode);
}

#else
#include <unistd.h>
#include <sys/types.h>
    const mode_t tXt = S_IRWXU | S_IRWXG | S_IRWXO;
    const mode_t rwXt = S_IRUSR | S_IRGRP | S_IROTH | S_IWUSR | S_IWGRP | S_IWOTH;
inline void ChangeMode  (const char *file, mode_t mode)
{
    chmod (file, mode);
}
#endif

typedef char nomefile[256];
#define MaxLenFilename sizeof(nomefile)

typedef struct {
    int op;        // symmetry operator
    int tra[3];    // translation
               } op_type;

typedef struct {
      float xc[3];   // x,y,z 
      float biso;    // fattore termico
      float bij[6];  // fattori termici anisotropi
      float och;     // occupanza chimica
      float ocry;    // occupanza cristall.
      float inte;    // intensita' di picco
      int nz;        // nz
      char lab[16];  // label
      int rcod[5];   // posizione nella matrice jac. del parametro se affinato; 0 se non affinato
      int ptab;      // pointer to table of elements
      float xsd[3];  // standard deviation on x,y,z
      float bsd;     // standard deviation on biso
      float osd;     // standard deviation on och
      int   asym;    // corrisponding atom in the a.u.
      op_type  op;   // symmetry operator
      int doc;       // dynamical occupancy correction
                }  atom_type;

typedef struct {
     int    hkl[3];  // indici hkl
     int    m;       // molteplicità del riflesso
     float  tthd[2]; // 2-theta del riflesso in gradi 
     float  slaq;    // ((sen(theta) / lambda))**2 = rho**2
     float  fo;      // fattori di struttura osservati 
     float  fc;      // fattori di struttura calcolati dal modello 
     int    ph;      // fasi calcolate dal modello
     float  fv;      // fattori di struttura veri
     int    phv;     // fase vera
     float  lp[2];   // correzione di Lorentz-polarizzazione 
     float  as[2];   // correzione di assorbimento 
     float  fwhm[2]; // fwhm 
     float  po;      // P.O. correction
     float  pk;      // peak range
     float  rapI;    // intensity ratio
     int    jcode;   // reflection code
     int    rcod;    // code for reflection refinement
               } reflection_type;

typedef struct {
               int   n1, n2;  // atomi legati
               float dist;    // distanza corrente
               float sigma;   // sigma
               int   ord;     // bond order
               } f_bond_type;

typedef struct {
                 GdkPixbuf *icon;
                 gchar *text;
               } PixText;

typedef struct {
      int                        z             ;// atomic number Z
      char                       name[20]      ;// name
      char                       lab[8]        ;// label
      float                      weight        ;// atomic weight
      float                      c_radius      ;// covalent radius
      float                      w_radius      ;// van der Waals radius
      float                      ax[4]         ;// coefficients a per calcolo scattering factors (x-ray)
      float                      bx[4]         ;// coefficients b per calcolo scattering factors (x-ray)
      float                      cx            ;// coefficients c per calcolo scattering factors (x-ray)
      float                      fMo[2]        ;// f',f''
      float                      fCu[2]        ;// f',f''
      float                      ae[4]         ;// coefficients a per calcolo scattering factors (electron)
      float                      be[4]         ;// coefficients b per calcolo scattering factors (electron)
      float                      ce            ;// coefficients c per calcolo scattering factors (electron)
      float                      fact          ;// fattore di scattering dei neutroni
      float                      al[4]         ;// used scattering factors
      float                      bs[4]         ;// used scattering factors
      float                      cl            ;// used scattering factors
      float                      nw            ;// number of elements
      float                      mac           ;// Mass attenuation coefficients (cm2 g-1)
      int                        radtype       ;// tipo di radiazione usata
      int                        charge        ;// charge 
      int                        ptab          ;// pointer to table of elements
      float                      zeff;
      float                      ifMottFormula;
               } element_type;

typedef struct {
     float  val;    // valore del parametro
     float  sd;     // standard deviation
     int    rcod;   // posizione nella matrice jac. del parametro se affinato; 0 se non affinato
     char   str[10];// stringa associata al parametro
               } r_param_type;

int dir_set_current(const gchar *dir);

const char* GetBrowser(void);
const char* GetPDFViewer(void);
int dir_create(const gchar *dir);
void CopiaStringa(char *dest, size_t sz, const char *src);
void CopiaStringan(char *dest, size_t sz, const char *src, size_t n);
char *DuplicaStringa(const char *s);
void AppendiStringa(char *dest, size_t sz, const char *src);
void AppendiStringan(char *dest, size_t sz, const char *src, size_t n);
FILE *fOpen(const char *path, const char *mode);
void ScriviStringa(char *str, size_t sz, const char *format, ...);
#if !__STDC_WANT_SECURE_LIB__
int  fscanf_s(FILE *fp, const char *format, ...);
#endif
int Cancella(const char *pathname);
const char *Ambiente(const gchar *varname);
void GetMillisec(int *timer);
void OraFormatotm(const time_t *timep, tm *result);
void OraFormatoasc(const struct tm *tm, char *buf);
void OraCorrente(struct tm *newtime);
int mr_access(const char *filename, char cmode);
bool str_white(const char *str);
std::string strstrip(std::string &s);
char *getPWD();
char* getHomeDir (void);
void formatta_msg(char *msgout, size_t lmsg, const char *MsgIn, int lriga);
int num_el (const char *str[]);
char *get_prog_dir(const char *prognam);
void strshift(char tmpd[], int ish);
void get_servdir(nomefile dire);
void put_servdir(nomefile dire);
char *get_mtime(const char *name);
char *get_ctime();
char *get_jtime();
char *get_files_dir(const char *progdir, const char *filesdirname);
char *get_pkg_dir(const char *progdir);
int b_unset (int word, int bit);
int b_set (int word, int bit);
int b_test (int word, int bit);
void CutStrc(char *str, int *iz, int lriga);
void  mr_CopyFile(const char *fileold, const char *filenew);
char  *filtra(const char *str, const char *c);
std::string filtranl(const char *str);
std::string filtrastr(const std::string &s, const char c);
char  *str_cut(char *str, const char *c);
int is_valid_symbol(const char *el, int N=0);
float *get_elem_color(int num);
float *get_amino_color(int num);
float *get_chain_color(int num);
float *get_struct_color(int num);
float *get_gen_color(int num);
const char *get_AminoCode(int num);
const char *get_nth_element(int num);
const char *get_nth_element_name(int num);
int ZFromPtab(int ptab);
const char *get_nth_amino(int num);
const char *get_nth_chain(int num);
const char *get_nth_struct(int num);
const char *get_nth_base(int num);
int IsAmino(const char *resname);
int IsAminoNucle(const char *resname, const char *element_name=NULL);
int IsDNA(const char *resname, const char *element_name=NULL);
int nCPU();
const char *SystemArch();
std::string dateinfo(char *filinfo);
std::string ArchInfo();
std::string fromUTF8(std::string localname);
//const gchar *fromUTF8(const gchar *localname);
const gchar *toUTF8(const gchar *localname);

#define MRtoupper(a) ((a) >= 'a' && ((a) <= 'z') ? ((a)-('a'-'A')):(a))
#define  CHARS                    sizeof(char)
//#define MRsign(x,y)               ( ((y) >= (0)) ? (MRabs(x)) : -(MRabs(x)) )

#define SQR(a) (a*a)

//float Distanza (const float *p1, const float *p2);
//float Angle (const float *p1, const float *p2);
//float Angolo (const float *p1, const float *p2, const float *p3);
//float Torsione (const float *p1, const float *p2, const float *p3, const float *p4);
//float calcDistAng (float **p, int op);
//float Det(float m[3][3]);
//float *CellaReciproca(float Cell[]);
//void MatriceReciproca(float cell[], float matR[3][3]);
//void CalcMats(float cella[], float mat[3][3], float matI[3][3],
//              float cells[], float matR[3][3], bool cInv);
void Desaturate(float color[]);
void TuneHSV(float color[], float inten);
void Darker(float color[]);
void TuneH(int nTot, int  frac, float color[]);
void TuneH(float nTot, float  frac, float color[]);
const char *lower_case(const char *stringa);

int IsUpdateAvailable(const char *filnam, bool ifSilent, int *ver=NULL);

// Elements etc.
typedef struct {
  char szSymbol[8];
  char szString[20];
  float color[4];
  int   Z;
} JavElement;

const int N_ELEMENTS = 300; //(int) (sizeof(JAV_ELEMENTY)/sizeof(JavElement));
static JavElement JAV_ELEMENTY[N_ELEMENTS] = {
     {"H",  "Hydrogen",      {0.850980f, 0.850980f, 1.000000f, 1.0f},   1}, 
     {"He", "Helium",        {0.850980f, 1.000000f, 1.000000f, 1.0f},   2},
     {"Li", "Lithium",       {0.800000f, 0.501961f, 1.000000f, 1.0f},   3},
     {"Be", "Beryllium",     {0.760784f, 1.000000f, 0.000000f, 1.0f},   4},
     {"B",  "Boron",         {1.000000f, 0.709804f, 0.709804f, 1.0f},   5},
     {"C",  "Carbon",        {0.564706f, 0.564706f, 0.564706f, 1.0f},   6},
     {"N",  "Nitrogen",      {0.188235f, 0.313726f, 0.972549f, 1.0f},   7},
     {"O",  "Oxygen",        {1.000000f, 0.050980f, 0.050980f, 1.0f},   8},
     {"F",  "Fluorine",      {0.564706f, 0.878431f, 0.313726f, 1.0f},   9},
     {"Ne", "Neon",          {0.701961f, 0.890196f, 0.960784f, 1.0f},  10},
     {"Na", "Sodium",        {0.670588f, 0.360784f, 0.949020f, 1.0f},  11},
     {"Mg", "Magnesium",     {0.541176f, 1.000000f, 0.000000f, 1.0f},  12},
     {"Al", "Aluminium",     {0.749020f, 0.650980f, 0.650980f, 1.0f},  13},
     {"Si", "Silicon",       {0.941176f, 0.784314f, 0.627451f, 1.0f},  14},
     {"P",  "Phosphorus",    {1.000000f, 0.501961f, 0.000000f, 1.0f},  15},
     {"S",  "Sulphur",       {1.000000f, 1.000000f, 0.188235f, 1.0f},  16},
     {"Cl", "Chlorine",      {0.121569f, 0.941176f, 0.121569f, 1.0f},  17},
     {"Ar", "Argon",         {0.501961f, 0.819608f, 0.890196f, 1.0f},  18},
     {"K",  "Potassium",     {0.560784f, 0.250980f, 0.831373f, 1.0f},  19},
     {"Ca", "Calcium",       {0.239216f, 1.000000f, 0.000000f, 1.0f},  20},
     {"Sc", "Scandium",      {0.901961f, 0.901961f, 0.901961f, 1.0f},  21},
     {"Ti", "Titanium",      {0.749020f, 0.760784f, 0.780392f, 1.0f},  22},
     {"V",  "Vanadium",      {0.650980f, 0.650980f, 0.670588f, 1.0f},  23},
     {"Cr", "Chromium",      {0.541176f, 0.600000f, 0.780392f, 1.0f},  24},
     {"Mn", "Manganese",     {0.611765f, 0.478431f, 0.780392f, 1.0f},  25},
     {"Fe", "Iron",          {0.878431f, 0.400000f, 0.200000f, 1.0f},  26},
     {"Co", "Cobalt",        {0.941176f, 0.564706f, 0.627451f, 1.0f},  27},
     {"Ni", "Nickel",        {0.313726f, 0.815686f, 0.313726f, 1.0f},  28},
     {"Cu", "Copper",        {0.784314f, 0.501961f, 0.200000f, 1.0f},  29},
     {"Zn", "Zinc",          {0.490196f, 0.501961f, 0.690196f, 1.0f},  30},
     {"Ga", "Gallium",       {0.760784f, 0.560784f, 0.560784f, 1.0f},  31},
     {"Ge", "Germanium",     {0.400000f, 0.560784f, 0.560784f, 1.0f},  32},
     {"As", "Arsenic",       {0.741176f, 0.501961f, 0.890196f, 1.0f},  33},
     {"Se", "Selenium",      {1.000000f, 0.631373f, 0.000000f, 1.0f},  34},
     {"Br", "Bromine",       {0.650980f, 0.160784f, 0.160784f, 1.0f},  35},
     {"Kr", "Krypton",       {0.360784f, 0.721569f, 0.819608f, 1.0f},  36},
     {"Rb", "Rubidium",      {0.439216f, 0.180392f, 0.690196f, 1.0f},  37},
     {"Sr", "Strontium",     {0.000000f, 1.000000f, 0.000000f, 1.0f},  38},
     {"Y",  "Ytrium ",       {0.580392f, 1.000000f, 1.000000f, 1.0f},  39},
     {"Zr", "Zirconium",     {0.580392f, 0.878431f, 0.878431f, 1.0f},  40},
     {"Nb", "Niobium",       {0.450980f, 0.760784f, 0.788235f, 1.0f},  41},
     {"Mo", "Molybdenum",    {0.329412f, 0.709804f, 0.709804f, 1.0f},  42},
     {"Tc", "Technetium",    {0.231373f, 0.619608f, 0.619608f, 1.0f},  43},
     {"Ru", "Ruthenium",     {0.141176f, 0.560784f, 0.560784f, 1.0f},  44},
     {"Rh", "Rhodium",       {0.039216f, 0.490196f, 0.549020f, 1.0f},  45},
     {"Pd", "Palladium",     {0.000000f, 0.411765f, 0.521569f, 1.0f},  46},
     {"Ag", "Silver",        {0.752941f, 0.752941f, 0.752941f, 1.0f},  47},
     {"Cd", "Cadmium",       {1.000000f, 0.850980f, 0.560784f, 1.0f},  48},
     {"In", "Indium",        {0.650980f, 0.458824f, 0.450980f, 1.0f},  49},
     {"Sn", "Tin",           {0.400000f, 0.501961f, 0.501961f, 1.0f},  50},
     {"Sb", "Antimony",      {0.619608f, 0.388235f, 0.709804f, 1.0f},  51},
     {"Te", "Tellurium",     {0.831373f, 0.478431f, 0.000000f, 1.0f},  52},
     {"I",  "Iodine",        {0.580392f, 0.000000f, 0.580392f, 1.0f},  53},
     {"Xe", "Xenon",         {0.258824f, 0.619608f, 0.690196f, 1.0f},  54},
     {"Cs", "Caesium",       {0.341176f, 0.090196f, 0.560784f, 1.0f},  55},
     {"Ba", "Barium",        {0.000000f, 0.788235f, 0.000000f, 1.0f},  56},
     {"La", "Lanthanum",     {0.439216f, 0.831373f, 1.000000f, 1.0f},  57},
     {"Ce", "Cerium",        {1.000000f, 1.000000f, 0.780392f, 1.0f},  58},
     {"Pr", "Praseodymium",  {0.850980f, 1.000000f, 0.780392f, 1.0f},  59},
     {"Nd", "Neodymium",     {0.780392f, 1.000000f, 0.780392f, 1.0f},  60},
     {"Pm", "Promethium",    {0.639216f, 1.000000f, 0.780392f, 1.0f},  61},
     {"Sm", "Samarium",      {0.560784f, 1.000000f, 0.780392f, 1.0f},  62},
     {"Eu", "Europium",      {0.380392f, 1.000000f, 0.780392f, 1.0f},  63},
     {"Gd", "Gadolinium",    {0.270588f, 1.000000f, 0.780392f, 1.0f},  64},
     {"Tb", "Terbium",       {0.188235f, 1.000000f, 0.780392f, 1.0f},  65},
     {"Dy", "Dysprosium",    {0.121569f, 1.000000f, 0.780392f, 1.0f},  66},
     {"Ho", "Holmium",       {0.000000f, 1.000000f, 0.611765f, 1.0f},  67},
     {"Er", "Erbium",        {0.000000f, 0.901961f, 0.458824f, 1.0f},  68},
     {"Tm", "Thulium",       {0.000000f, 0.831373f, 0.321569f, 1.0f},  69},
     {"Yb", "Ytterbium",     {0.000000f, 0.749020f, 0.219608f, 1.0f},  70},
     {"Lu", "Lutetium",      {0.000000f, 0.670588f, 0.141176f, 1.0f},  71},
     {"Hf", "Hafnium",       {0.301961f, 0.760784f, 1.000000f, 1.0f},  72},
     {"Ta", "Tantalum",      {0.301961f, 0.650980f, 1.000000f, 1.0f},  73},
     {"W",  "Tungsten",      {0.129412f, 0.580392f, 0.839216f, 1.0f},  74},
     {"Re", "Rhenium",       {0.149020f, 0.490196f, 0.670588f, 1.0f},  75},
     {"Os", "Osmium",        {0.149020f, 0.400000f, 0.588235f, 1.0f},  76},
     {"Ir", "Iridium",       {0.090196f, 0.329412f, 0.529412f, 1.0f},  77},
     {"Pt", "Platinum",      {0.815686f, 0.815686f, 0.878431f, 1.0f},  78},
     {"Au", "Gold",          {1.000000f, 0.819608f, 0.137255f, 1.0f},  79},
     {"Hg", "Mercury",       {0.721569f, 0.721569f, 0.815686f, 1.0f},  80},
     {"Tl", "Thalium",       {0.650980f, 0.329412f, 0.301961f, 1.0f},  81},
     {"Pb", "Lead",          {0.341176f, 0.349020f, 0.380392f, 1.0f},  82},
     {"Bi", "Bismuth",       {0.619608f, 0.309804f, 0.709804f, 1.0f},  83},
     {"Po", "Polonium",      {0.670588f, 0.360784f, 0.000000f, 1.0f},  84},
     {"At", "Astatine",      {0.458824f, 0.309804f, 0.270588f, 1.0f},  85},
     {"Rn", "Radon",         {0.258824f, 0.509804f, 0.588235f, 1.0f},  86},
     {"Fr", "Francium",      {0.258824f, 0.000000f, 0.400000f, 1.0f},  87},
     {"Ra", "Radium",        {0.000000f, 0.490196f, 0.000000f, 1.0f},  88},
     {"Ac", "Actinium",      {0.439216f, 0.670588f, 0.980392f, 1.0f},  89},
     {"Th", "Thorium",       {0.000000f, 0.729412f, 1.000000f, 1.0f},  90},
     {"Pa", "Protactinium",  {0.000000f, 0.631373f, 1.000000f, 1.0f},  91},
     {"U",  "Uranium",       {0.000000f, 0.560784f, 1.000000f, 1.0f},  92},
     {"Np", "Neptunium",     {0.000000f, 0.501961f, 1.000000f, 1.0f},  93},
     {"Pu", "Plutonium",     {0.000000f, 0.419608f, 1.000000f, 1.0f},  94},
     {"Am", "Americium",     {0.329412f, 0.360784f, 0.949020f, 1.0f},  95},
     {"Cm", "Curium",        {0.470588f, 0.360784f, 0.890196f, 1.0f},  96},
     {"Bk", "Berkelium",     {0.541176f, 0.309804f, 0.890196f, 1.0f},  97},
     {"Cf", "Californium",   {0.631373f, 0.211765f, 0.831373f, 1.0f},  98},
     {"Es", "Einsteinium",   {0.701961f, 0.121569f, 0.831373f, 1.0f},  99},
     {"Fm", "Fermium",       {0.701961f, 0.121569f, 0.729412f, 1.0f}, 100},
     {"Md", "Mendelevium",   {0.701961f, 0.050980f, 0.650980f, 1.0f}, 101},
     {"No", "Nobelium",      {0.741176f, 0.050980f, 0.529412f, 1.0f}, 102},
     {"Lr", "Lawrencium",    {0.780392f, 0.000000f, 0.400000f, 1.0f}, 103},
     {"Rf", "Rutherfordium", {0.800000f, 0.000000f, 0.349020f, 1.0f}, 104},
     {"Db", "Dubnium",       {0.819608f, 0.000000f, 0.309804f, 1.0f}, 105},
     {"Sg", "Seaborgium",    {0.850980f, 0.000000f, 0.270588f, 1.0f}, 106},
     {"Bh", "Bohrium",       {0.878431f, 0.000000f, 0.219608f, 1.0f}, 107},
     {"Hs", "Hassium",       {0.901961f, 0.000000f, 0.180392f, 1.0f}, 108},
     {"Mt", "Meitnerium",    {0.921569f, 0.000000f, 0.149020f, 1.0f}, 109},
     {"Ds", "Darmstadtium",  {0.933333f, 0.000000f, 0.137255f, 1.0f}, 110},
     {"Rg", "Roentgenium",   {0.980392f, 0.000000f, 0.156863f, 1.0f}, 111}//,
#if 0
     {"D",    "Deuterium",        {0.000000f, 0.000000f, 0.000000f, 1.0f},   1},
     {"H1-",  "Hydrogen(1-)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},   1},
     {"Li1+", "Lithium(1+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},   3},
     {"Be2+", "Beryllium(2+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},   4},
     {"O1-",  "Oxygen(1-)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},   8},
     {"F1-",  "Fluorine(1-)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},   9},
     {"Na1+", "Sodium(1+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},   11},
     {"Mg2+", "Magnesium(2+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},   12},
     {"Al3+", "Aluminium(3+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},   13},
     {"Si4+", "Silicon(4+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},   14},
     {"Cl1-", "Chlorine(1-)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},   17},
     {"K1+",  "Potassium(1+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},   19},
     {"Ca2+", "Calcium(2+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},   20},
     {"Sc3+", "Scandium(3+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  21},
     {"Ti2+", "Titanium(2+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  22},
     {"Ti3+", "Titanium(3+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  22},
     {"Ti4+", "Titanium(4+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  22},
     {"V2+",  "Vanadium(2+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  23},
     {"V3+",  "Vanadium(3+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  23},
     {"V5+",  "Vanadium(5+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  23},
     {"Cr2+", "Chromium(2+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  24},
     {"Cr3+", "Chromium(3+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  24},
     {"Mn2+", "Manganese(2+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  25},
     {"Mn3+", "Manganese(3+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  25},
     {"Mn4+", "Manganese(4+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  25},
     {"Fe2+", "Iron(2+)",         {0.000000f, 0.000000f, 0.000000f, 1.0f},  26},
     {"Fe3+", "Iron(3+)",         {0.000000f, 0.000000f, 0.000000f, 1.0f},  26},
     {"Co2+", "Cobalt(2+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  27},
     {"Co3+", "Cobalt(3+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  27},
     {"Ni2+", "Nickel(2+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  28},
     {"Ni3+", "Nickel(3+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  28},
     {"Cu1+", "Copper(1+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  29},
     {"Cu2+", "Copper(2+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  29},
     {"Zn2+", "Zinc(2+)",         {0.000000f, 0.000000f, 0.000000f, 1.0f},  30},
     {"Ga3+", "Gallium(3+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  31},
     {"Ge4+", "Germanium(4+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  32},
     {"Br1-", "Bromine(1-)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  35},
     {"Rb1+", "Rubidium(1+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  37},
     {"Sr2+", "Strontium(2+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  38},
     {"Y3+",  "Ytrium(3+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  39},
     {"Zr4+", "Zirconium(4+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  40},
     {"Nb3+", "Niobium(3+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  41},
     {"Nb5+", "Niobium(5+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  41},
     {"Mo3+", "Molybdenum(3+)",   {0.000000f, 0.000000f, 0.000000f, 1.0f},  42},
     {"Mo5+", "Molybdenum(5+)",   {0.000000f, 0.000000f, 0.000000f, 1.0f},  42},
     {"Mo6+", "Molybdenum(6+)",   {0.000000f, 0.000000f, 0.000000f, 1.0f},  42},
     {"Ru3+", "Ruthenium(3+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  44},
     {"Ru4+", "Ruthenium(4+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  44},
     {"Rh3+", "Rhodium(3+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  45},
     {"Rh4+", "Rhodium(4+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  45},
     {"Pd2+", "Palladium(2+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  46},
     {"Pd4+", "Palladium(4+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  46},
     {"Ag1+", "Silver(1+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  47},
     {"Ag2+", "Silver(2+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  47},
     {"Cd2+", "Cadmium(2+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  48},
     {"In3+", "Indium(3+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  49},
     {"Sn2+", "Tin(2+)",          {0.000000f, 0.000000f, 0.000000f, 1.0f},  50},
     {"Sn4+", "Tin(4+)",          {0.000000f, 0.000000f, 0.000000f, 1.0f},  50},
     {"Sb3+", "Antimony(3+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  51},
     {"Sb5+", "Antimony(5+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  51},
     {"I1-",  "Iodine(1-)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  53},
     {"Cs1+", "Caesium(1+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  55},
     {"Ba2+", "Barium(2+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  56},
     {"La3+", "Lanthanum(3+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  57},
     {"Ce3+", "Cerium(3+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  58},
     {"Ce4+", "Cerium(4+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  58},
     {"Pr3+", "Praseodymium(3+)", {0.000000f, 0.000000f, 0.000000f, 1.0f},  59},
     {"Pr4+", "Praseodymium(4+)", {0.000000f, 0.000000f, 0.000000f, 1.0f},  59},
     {"Nd3+", "Neodymium(3+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  60},
     {"Pm3+", "Promethium(3+)",   {0.000000f, 0.000000f, 0.000000f, 1.0f},  61},
     {"Sm3+", "Samarium(3+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  62},
     {"Eu2+", "Europium(2+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  63},
     {"Eu3+", "Europium(3+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  63},
     {"Gd3+", "Gadolinium(3+)",   {0.000000f, 0.000000f, 0.000000f, 1.0f},  64},
     {"Tb3+", "Terbium(3+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  65},
     {"Dy3+", "Dysprosium(3+)",   {0.000000f, 0.000000f, 0.000000f, 1.0f},  66},
     {"Ho3+", "Holmium(3+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  67},
     {"Er3+", "Erbium(3+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  68},
     {"Tm3+", "Thulium(3+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  69},
     {"Yb2+", "Ytterbium(2+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  70},
     {"Yb3+", "Ytterbium(3+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  70},
     {"Lu3+", "Lutetium(3+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  71},
     {"Hf4+", "Hafnium(4+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  72},
     {"Ta5+", "Tantalum(5+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  73},
     {"W6+",  "Tungsten(6+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  74},
     {"Os4+", "Osmium(4+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  76},
     {"Ir3+", "Iridium(3+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  77},
     {"Ir4+", "Iridium(4+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  77},
     {"Pt2+", "Platinum(2+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  78},
     {"Pt4+", "Platinum(4+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  78},
     {"Au1+", "Gold(1+)",         {0.000000f, 0.000000f, 0.000000f, 1.0f},  79},
     {"Au3+", "Gold(3+)",         {0.000000f, 0.000000f, 0.000000f, 1.0f},  79},
     {"Hg1+", "Mercury(1+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  80},
     {"Hg2+", "Mercury(2+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  80},
     {"Tl1+", "Thalium(1+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  81},
     {"Tl3+", "Thalium(3+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  81},
     {"Pb2+", "Lead(2+)",         {0.000000f, 0.000000f, 0.000000f, 1.0f},  82},
     {"Pb4+", "Lead(4+)",         {0.000000f, 0.000000f, 0.000000f, 1.0f},  82},
     {"Bi3+", "Bismuth(3+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  83},
     {"Bi5+", "Bismuth(5+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  83},
     {"Ra2+", "Radium(2+)",       {0.000000f, 0.000000f, 0.000000f, 1.0f},  88},
     {"Ac3+", "Actinium(3+)",     {0.000000f, 0.000000f, 0.000000f, 1.0f},  89},
     {"Th4+", "Thorium(4+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  90},
     {"U3+",  "Uranium(3+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  92},
     {"U4+",  "Uranium(4+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  92},
     {"U6+",  "Uranium(6+)",      {0.000000f, 0.000000f, 0.000000f, 1.0f},  92},
     {"Np3+", "Neptunium(3+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  93},
     {"Np4+", "Neptunium(4+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  93},
     {"Np6+", "Neptunium(6+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  93},
     {"Pu3+", "Plutonium(3+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  94},
     {"Pu4+", "Plutonium(4+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  94},
     {"Pu6+", "Plutonium(6+)",    {0.000000f, 0.000000f, 0.000000f, 1.0f},  94}
#endif
};

const int N_TRUE_ELEMENTS = 111;

typedef struct {
  char szSymbol[5];
  char szCode[2];
  char szString[30];
  float color[4];
} JavAmino;

static JavAmino JAV_AMINO[] = {
     {" Ala", "A", "Alanine", {1.000000f, 0.000000f, 0.000000f, 1.0f}},
     {" Arg", "R", "Arginine", {0.000000f, 0.000000f, 1.000000f, 1.0f}},
     {" Asn", "N", "Asparagine", {0.000000f, 1.000000f, 0.000000f, 1.0f}},
     {" Asp", "D", "Aspartic acid", {1.000000f, 1.000000f, 0.000000f, 1.0f}},
     {" Cys", "C", "Cysteine", {1.000000f, 0.000000f, 1.000000f, 1.0f}},
     {" Gln", "Q", "Glutamine", {0.000000f, 1.000000f, 1.000000f, 1.0f}},
     {" Glu", "E", "Glutamic acid", {1.000000f, 0.752941f, 0.752941f, 1.0f}},
     {" Gly", "G", "Glycine", {0.000000f, 0.752941f, 0.752941f, 1.0f}},
     {" His", "H", "Histidine", {0.650980f, 0.000000f, 0.650980f, 1.0f}},
     {" Ile", "I", "Isoleucine", {0.200000f, 0.498039f, 1.000000f, 1.0f}},
     {" Leu", "L", "Leucine", {1.000000f, 0.498039f, 0.000000f, 1.0f}},
     {" Lys", "K", "Lysine", {0.000000f, 0.600000f, 0.000000f, 1.0f}},
     {" Met", "M", "Methionine", {0.000000f, 0.000000f, 0.705882f, 1.0f}},
     {" Phe", "F", "Phenylalanine", {0.705882f, 0.000000f, 0.000000f, 1.0f}},
     {" Pro", "P", "Proline", {0.349020f, 0.800000f, 1.000000f, 1.0f}},
     {" Ser", "S", "Serine", {0.800000f, 0.800000f, 0.800000f, 1.0f}},
     {" Thr", "T", "Threonine", {0.745098f, 0.627451f, 0.431373f, 1.0f}},
     {" Trp", "W", "Tryptophan", {1.000000f, 0.549020f, 1.000000f, 1.0f}},
     {" Tyr", "Y", "Tyrosine", {1.000000f, 0.439216f, 0.258824f, 1.0f}},
     {" Val", "V", "Valine", {0.509804f, 0.509804f, 0.823529f, 1.0f}},
     {" Mse", "M", "Selenomethionine",{0.000000f, 0.000000f, 0.705882f, 1.0f}}
};

const int N_AMINO = (int) (sizeof(JAV_AMINO)/sizeof(JavAmino));

static JavAmino JAV_DNA_NUCLEOTIDES[] = {
     {"  DA", "A", "Adenine", {1.000000f, 1.000000f, 0.000000f, 1.0f}},
     {"  DC", "C", "Cytosine",{0.933333f, 0.650980f, 0.070588f, 1.0f}},
     {"  DG", "G", "Guanine", {0.750000f, 0.000000f, 0.000000f, 1.0f}},
     {"  DT", "T", "Thymine", {0.000000f, 0.750000f, 0.000000f, 1.0f}},
     {"   U", "U", "Uracil",  {0.000000f, 0.750000f, 0.000000f, 1.0f}},
};

const int N_DNA_NUCLEOTIDES = (int) (sizeof(JAV_DNA_NUCLEOTIDES)/sizeof(JavAmino));

#define Is_Amino(x)      ((x)<=N_AMINO)
#define Is_Dna(x)        ((((x)>N_AMINO)) && ((x)<=N_AMINO+N_DNA_NUCLEOTIDES))
#define IsAdenine(x)     ((x)==N_AMINO+1)
#define IsCytosine(x)    ((x)==N_AMINO+2)
#define IsGuanine(x)     ((x)==N_AMINO+3)
#define IsThymine(x)     ((x)==N_AMINO+4)
#define IsUracil(x)      ((x)==N_AMINO+5)
#define IsPyrimidine(x)  (IsCytosine(x) || IsThymine(x) || IsUracil(x))
#define IsPurine(x)      (IsAdenine(x) || IsGuanine(x))
//#define IsPair(x,y)      ((x != y) && (((x+y) == 45) || ((x+y) == 46) || ((x+y) == 48)) )
//#define IsPair(x,y)      ((x != y) && (((x+y) == 45) || ((x+y) == 46)) )
#define IsPair(x,y)      ((x != y) && ((IsPurine(x) && IsPyrimidine(y)) || (IsPurine(y) && IsPyrimidine(x))) )
#define Is_Water(x)      ((x)==0)
#define AANonStd(x)      ((x)==500)
#define NANonStd(x)      ((x)==600)

inline bool Is_CNOS(int code)
{
    return(((code >= 5) && (code <= 7)) || (code == 15));
}

inline bool Is_AminoG(int code)
{
    return (Is_Amino(code) || AANonStd(code));
}

inline bool Is_DnaG(int code)
{
    return (Is_Dna(code) || NANonStd(code));
}

inline bool Is_Acid(int code)
{
    return (Is_DnaG(code) || Is_AminoG(code));
}

typedef struct {
  char szString[6];
  float color[4];
} JavChain;

static JavChain JAV_CHAIN[] = {
       {" A, a", {0.752941f, 0.815686f, 1.000000f, 1.0f}},    
       {" B, b", {0.690196f, 1.000000f, 0.690196f, 1.0f}},    
       {" C, c", {1.000000f, 0.752941f, 0.784314f, 1.0f}},    
       {" D, d", {1.000000f, 1.000000f, 0.501961f, 1.0f}},    
       {" E, e", {1.000000f, 0.752941f, 1.000000f, 1.0f}},    
       {" F, f", {0.690196f, 0.941176f, 0.941176f, 1.0f}},    
       {" G, g", {1.000000f, 0.815686f, 0.439216f, 1.0f}},    
       {" H, h", {0.941176f, 0.501961f, 0.501961f, 1.0f}},    
       {" I, h", {0.960784f, 0.870588f, 0.701961f, 1.0f}},    
       {" J, j", {0.000000f, 0.749020f, 1.000000f, 1.0f}},    
       {" K, k", {0.803922f, 0.360784f, 0.360784f, 1.0f}},    
       {" L, l", {0.400000f, 0.803922f, 0.666667f, 1.0f}},    
       {" M, m", {0.603922f, 0.803922f, 0.196078f, 1.0f}},    
       {" N, n", {0.933333f, 0.509804f, 0.933333f, 1.0f}},    
       {" O, o", {0.000000f, 0.807843f, 0.819608f, 1.0f}},    
       {" P, p", {0.000000f, 1.000000f, 0.498039f, 1.0f}},    
       {" Q, q", {0.235294f, 0.701961f, 0.443137f, 1.0f}},    
       {" R, r", {0.000000f, 0.000000f, 0.545098f, 1.0f}},    
       {" S, s", {0.741176f, 0.717647f, 0.419608f, 1.0f}},    
       {" T, t", {0.000000f, 0.392157f, 0.000000f, 1.0f}},    
       {" U, u", {0.501961f, 0.000000f, 0.000000f, 1.0f}},    
       {" V, v", {0.501961f, 0.501961f, 0.000000f, 1.0f}},    
       {" W, w", {0.501961f, 0.000000f, 0.501961f, 1.0f}},    
       {" X, x", {0.000000f, 0.501961f, 0.501961f, 1.0f}},    
       {" Y, y", {0.721569f, 0.525490f, 0.043137f, 1.0f}},    
       {" Z, z", {0.698039f, 0.133333f, 0.133333f, 1.0f}}     
};

const int N_CHAINS = (int) (sizeof(JAV_CHAIN)/sizeof(JavChain));

typedef struct {
  char szString[8];
  float color[4];
} JavStructure;


static JavStructure JAV_STRUCTURE[] = {
       {"helix",  {1.000000f, 0.000000f, 0.000000f, 1.0f}},    // helix
       {"strand", {1.000000f, 1.000000f, 0.000000f, 1.0f}},    // strands
       {"turn",   {0.000000f, 0.000000f, 1.000000f, 1.0f}},    // turns
       {"coil",   {0.000000f, 1.000000f, 0.000000f, 1.0f}},    // coils
       {"other",  {1.000000f, 0.411765f, 0.705882f, 1.0f}}     // other
};

const int N_STRUCT  = (int) (sizeof(JAV_STRUCTURE)/sizeof(JavStructure));

const int NUM_COLORS = (int) (sizeof(JAV_CHAIN)/sizeof(JavChain) + 
                              sizeof(JAV_STRUCTURE)/sizeof(JavStructure) +
                              sizeof(JAV_AMINO)/sizeof(JavAmino) +
                              sizeof(JAV_DNA_NUCLEOTIDES)/sizeof(JavAmino) +
                              sizeof(JAV_ELEMENTY)/sizeof(JavElement)) + 1;
gboolean destroy_dlg(gpointer data );
int MessageDialog(GtkWidget *parent, GtkMessageType type,
            GtkButtonsType buttons, const gchar *message, ...) ;
int MessageDialog(const char *title, GtkWidget *parent, GtkMessageType type,
            GtkButtonsType buttons, const gchar *message, ...) ;
int MessageDialog(const char *title, int timeout, int def, GtkWidget *parent,
            GtkMessageType type, GtkButtonsType buttons,
            const gchar *message, ...);
double CellVolume(const double cella[]);
double CellVolume(const float cella[]);
int AnalizzaSequenza(const std::string &buf);
int ContentFromSequenceFile(const char *SeqFile, int NumSpec[]);
bool IsWater(const char *resname);
inline int iround(double d)
{
    return static_cast<int>( floor(d+0.5) );
}
float Minimo (float vett[], int n);
float Massimo (float vett[], int n);
bool compare_names(const std::string &fn1, const std::string &fn2);
std::string noBlank(std::string s);

#ifdef __GNUC__
#if ((__GNUC__ >=4 ) && (__GNUC_MINOR__ >= 1))
// left-trim
static inline std::string &ltrim(std::string &s) {
        s.erase(s.begin(), std::find_if(s.begin(), s.end(), std::not1(std::ptr_fun<int, int>(std::isspace))));
        return s;
}

// right-trim
static inline std::string &rtrim(std::string &s) {
        s.erase(std::find_if(s.rbegin(), s.rend(), std::not1(std::ptr_fun<int, int>(std::isspace))).base(), s.end());
        return s;
}

// left and right trim
static inline std::string &trim(std::string &s) {
        return ltrim(rtrim(s));
}
#else
static inline std::string &trim(std::string &s) {
// trim trailing spaces
   size_t endpos = s.find_last_not_of(" \t");
   if( std::string::npos != endpos )
   {
       s = s.substr( 0, endpos+1 );
   }
// trim leading spaces
   size_t startpos = s.find_first_not_of(" \t");
   if( std::string::npos != startpos )
   {
       s = s.substr( startpos );
   }
   return s;
}
#endif
#else
// left-trim
static inline std::string &ltrim(std::string &s) {
        s.erase(s.begin(), std::find_if(s.begin(), s.end(), [](unsigned char c) {return !::isspace(c);}));
        return s;
}
// right-trim
static inline std::string &rtrim(std::string &s) {
        s.erase(std::find_if(s.rbegin(), s.rend(), [](unsigned char c) {return !::isspace(c);}).base(), s.end());
        return s;
}

// left and right trim
static inline std::string &trim(std::string &s) {
        return ltrim(rtrim(s));
}
#endif
int leggixen(const char *file1);
char *find_program_in_path(const char *progname);
void find_pattern_in_dir(const char *filtro, std::vector<std::string> &Files,
                          const char *dir=NULL);
typedef struct {
                  const char *str;
                  int        fort_opt;
                  int        params;
                  int        valore;
               } _OptStruct;

int IsValidOption(std::string opt, _OptStruct Opts[]);
bool get_matrices(const char *input, int RMatrix[3][3], float TMatrix[3]);
void str_replace(std::string &stringa, std::string str1, std::string str2);
std::vector<std::string> split_str(std::string &stringa, std::string separ);
