module ochemdb

implicit none

contains

   subroutine export_for_ochemdb(atom,legm,elem,cell,spg,sname,expand,ftype,lcharge,kpr)
!
!  Generate file csv for statistical analysis of distances
!
   USE elements
   USE atom_type_util
   USE unit_cell
   USE spginfom
   USE connect_mod
   USE strutil
   USE fileutil
   USE nrutil, only: swap
   USE arrayutil
   USE rotationmod
   USE cgeom
       USE gen_frm
   type(atom_type), dimension(:), allocatable, intent(in)    :: atom
   type(bond_type), dimension(:), allocatable                :: legm
   type(element_type), dimension(:), allocatable, intent(in) :: elem
   type(cell_type), intent(in)                               :: cell
   type(spaceg_type), intent(in)                             :: spg
   character(len=*), intent(in)                              :: sname
   logical, intent(in)                                       :: expand
   character(len=*)                                          :: ftype
   logical, intent(in)                                       :: lcharge
   integer, intent(in)                                       :: kpr
   integer                                                   :: i,j,a1,a2,a3,a4
   type(file_handle)                                         :: fdb
   type(angle_type), dimension(:), allocatable               :: ang
   type(container_type), dimension(:), allocatable           :: conn
   integer                                                   :: spos,strcode
   type(sg_info_type)                                        :: infospg
   character(len=1), dimension(0:1)                          :: strtype = ['I','O']
   character(len=:), allocatable                             :: crys
   type(atom_type), dimension(:), allocatable                :: atoms
   type(bond_type), dimension(:), allocatable                :: legms,bonds1l    !,bondsd
   integer, dimension(:), allocatable                        :: charge
   integer                                                   :: nring,nrot,newrot,chis,numb0l,numb1l,numb1lt
   type(container_type), dimension(:), allocatable           :: ring
   type(rotation_type), dimension(:), allocatable            :: rot
   type(torsion_type), dimension(:), allocatable             :: tors
   integer, dimension(4)                                     :: atv
   logical                                                   :: fdist,fang,ftors,fspg
   character(len=:), allocatable, dimension(:)               :: atclass
   integer, parameter                                        :: NMAXATOM = 500
   integer, parameter                                        :: NMAXBOND = 3000
   integer                                                   :: nang
!
   fdist = index(ftype,'D') > 0
   fang = index(ftype,'A') > 0
   ftors = index(ftype,'T') > 0
   fspg = index(ftype,'S') > 0
   if (numatoms(atom) == 0) then
       if (kpr >= 0) write(kpr,'(a)')'WARNING: NO ATOMS IN THE STRUCTURE!'
       return
   endif
   if (numbonds(legm) > NMAXBOND .and. spg%num == 1) then
       if (kpr >= 0) write(kpr,'(a,i0)')'WARNING: TOO BONDS IN P1 STRUCTURE! NBONDS = ',numbonds(legm)
       return
   endif
!
   call init_for_symm(atom,legm,atoms,legms)
   numb0l = numbonds(legms)
   numb1l = numb0l
   numb1lt = numb0l
   if (expand .and. numatoms(atom) < NMAXATOM) then
       if (is_duplicated_atoms(atoms,0.1,cell,spg)) then
           if (kpr >= 0) write(kpr,'(a)')'WARNING: DUPLICATED ATOMS FOUND! STRUCTURE WAS REJECTED.'
           return
       endif
       !!call apply_symmetry(atom,atoms,legm,legms,level=[1.0,1.0,1.0],1,cell,spg)
       call expand_contacts(atom,cell,spg,atoms,legms,.false.)        ! expand at 1th level
       numb1lt = numbonds(legms) ! save total number of atoms at 1th level
       numb1l = numb1lt
       !call export_structure('pippo1.mol',atoms,cell,spg,elem,MOL_FILE,   &
       !                      'prova',1.54056,legms)
       !       do i=1,numbonds(legms)
       !          write(0,*)'BOND1=',i,legms(i)%n1,legms(i)%n2
       !       enddo
       if (numb1lt > numb0l) then
!
!          remove duplicate bonds and update numb1l
           call find_duplicate_bonds_sym(atoms,legms,numb0l+1,numb1l)
           !call print_connect(atoms%lab,legms,kpri=70)
           !write(0,*)'NUMB=',numb1l,numbonds(legms)
           !corr numb1l = numbonds(legms)  ! n. of bonds at 1th level
           call expand_contacts(atom,cell,spg,atoms,legms,.false.)    ! expand at 2th level
              !do i=1,numbonds(legms)
              !   write(0,*)'BOND2=',i,legms(i)%n1,legms(i)%n2
              !enddo
       endif
   endif
       !call print_atoms(atoms,kpr=70)
       !do i=1,numatoms(atoms)
       !   write(70,'(a,a,a,i3)')'ATOM=',atoms(i)%glab(i),'ASYM=',atoms(i)%asym
       !enddo
     !allocate(bondsd(numb1l),source=legms(:numb1l))
   !call export_structure('pippo.mol',atoms,cell,spg,elem,MOL_FILE,   &
   !                      'prova',1.54056,legms)
   if (numbonds(legms) > 0) then
!
!      Write file with: ;ID;elem1;elem2;dist;
       call order_type(atoms,legms)
   
       if (fdist) then
           call fdb%fopen(trim(sname)//'d.csv','w')
           do i=1,numb1l
              a1 = legms(i)%n1
              a2 = legms(i)%n2
              write(fdb%handle(),'(a)')';'//trim(sname)//';'  &
                               //trim(atoms(a1)%spec())//';'//trim(atoms(a2)%spec())//';'//r_to_s(legms(i)%dist)//';'
           enddo
           call fdb%fclose()
       endif
!
!      Write file with: ;ID;charge1,charge2,label1,label2;conn1;class1;conn2;class2;ord;
       allocate(charge(numatoms(atoms)))
       if (lcharge) then
           charge = atoms%charge()
       else
           charge = [(i,i=1,numatoms(atoms))]
       endif
       where(charge == 0) charge = -999
       call bond_to_connect(numatoms(atoms),legms,conn)
       call bond_type_perception(atoms,legms,cell,ring,nring)
!
!      delete non organic ring
       do i=1,nring
          if (any_is_metal(atoms(ring(i)%pos))) then
              call cont_set_as_deleted(ring(i))
!corr              write(0,*)'delete ring:',atoms(ring(i)%pos)%lab
          endif
       enddo
       call delete_container(ring)
!
       if (fdist .or. fang .or. ftors) then
           call make_atomclass(atoms,numatoms(atom),legms,conn,ring,atclass)
       endif
       if (fdist) call create_atom_class_filed(atoms,legms,numb1l,conn,charge,sname,atclass)
!
!      Angles
       allocate(bonds1l(numb1lt),source=legms(:numb1lt))  ! make array with bonds up to first level
       call bond_to_angle(bonds1l,atoms,cell%get_g(),ang,angvali=[0.1,1000.0])
       call find_duplicate_angles_sym(atoms,ang,nang)
       if (numangles(ang) > 0) then
           if (fang) then
               call fdb%fopen(trim(sname)//'a.csv','w')
               call order_type(atoms,ang)
               do i=1,nang !!!!!numangles(ang)
                  a1 = ang(i)%n1
                  a2 = ang(i)%n2
                  a3 = ang(i)%n3
                  write(fdb%handle(),'(a)')';'//trim(sname)//';'//trim(atoms(a1)%spec())//';'//trim(atoms(a2)%spec())  &
                                         //';'//trim(atoms(a3)%spec())//';'//r_to_s(ang(i)%val)//';' 
               enddo
               call fdb%fclose()
!
               call create_atom_class_filea(atoms,legms,ang,nang,conn,charge,sname,atclass)
           endif
!
!          Find torsion angles around rotable bonds. For each rotable bonds more torsions could be associated
           if (ftors) then
               call get_rotation_from_leg(atoms,conn,bonds1l,cell,nrot,rot,.false.)
               if (nrot > 0) then
                   call bonds_to_torsions(bonds1l,atoms,cell,tors)
                   newrot = 0
!
!                  Loop on all torsions accepting only those have no H and metals and no linear
                   loop_tors: do i=1,numtorsions(tors)
                      atv = [tors(i)%n1,tors(i)%n2,tors(i)%n3,tors(i)%n4]
                      if (is_hydrogen(atoms(atv))) then
                          tors(i)%n1 = 0
                          cycle
                      endif
                      if (any_is_metal(atoms(atv))) then
                          tors(i)%n1 = 0
                          cycle
                      endif
                      if (abs(rtod*angleC(atoms(atv(1))%xc,atoms(atv(2))%xc,atoms(atv(3))%xc,cell%get_g()) - 180) <= 1.5) then
                          tors(i)%n1 = 0
                          cycle
                      endif
                      if (abs(rtod*angleC(atoms(atv(2))%xc,atoms(atv(3))%xc,atoms(atv(4))%xc,cell%get_g()) - 180) <= 1.5) then
                          tors(i)%n1 = 0
                          cycle
                      endif
!
!                     Now check if torsion is a rotable bond
                      do j=1,nrot
                         if ((atv(2) == rot(j)%ators(2) .and. atv(3) == rot(j)%ators(3)) .or. &
                             (atv(3) == rot(j)%ators(2) .and. atv(2) == rot(j)%ators(3)) ) then
                             newrot = newrot + 1
                             cycle loop_tors
                         endif
                      enddo
                      tors(i)%n1 = 0
                   enddo loop_tors
!
                   if (newrot > 0) then
                       tors(:newrot) = pack(tors,mask=tors%n1 > 0)
                       call resize_torsions(tors,newrot)
                       !tors%val = abs(tors%val)
                       tors%val = tors%val
                       call order_type(atoms,tors)
                          !call print_connect(atoms%lab,tors=tors,kpri=0)
!
                       call fdb%fopen(trim(sname)//'t.csv','w')
                       do i=1,numtorsions(tors)
                          a1 = tors(i)%n1
                          a2 = tors(i)%n2
                          a3 = tors(i)%n3
                          a4 = tors(i)%n4
                          write(fdb%handle(),'(a)')';'//trim(sname)    &
                                //';'//trim(atoms(a1)%spec())//';'//trim(atoms(a2)%spec())  &
                                //';'//trim(atoms(a3)%spec())//';'//trim(atoms(a4)%spec())//';'//r_to_s(tors(i)%val)//';' 
                       enddo
                       call fdb%fclose()
!
                       call create_atom_class_filet(atoms,legms,tors,conn,charge,sname,atclass)
                   endif
!
               endif
           endif
       endif
   endif
!
!  Space group
   if (fspg) then
       if (spg%num == 0) then
           if (kpr >= 0) write(kpr,'(a)') 'WARNING: UNDEFINED SPACE GROUP!'
           return
       endif
       call fdb%fopen(trim(sname)//'spg.csv','w')
       if (.not.spg_check_csys(spg,cell%get_par())) then
           if (kpr >= 0) write(kpr,'(a)') 'WARNING: CELL AND SPG ARE INCOMPATIBLE!'
           write(0,'(a)') 'WARNING: CELL AND SPG ARE INCOMPATIBLE!'//trim(sname)
       endif
       !csys_cell = csys_from_cell(cell%get_par()
       !if (csys_cell == CS_Hexagonal) then
       !    if (kpr >= 0) write(kpr,'(a)') 'WARNING: CELL AND SPG ARE INCOMPATIBLE!'
       !endif
       infospg = get_spg_info(spg%num,spos)
       if (spos > 0) then
           strcode = is_organic(elem%z,nint(elem%nw))
           if (strcode == 2) strcode = 1  ! metallorganic -> organic
           if (spg%csys_code == CS_Trigonal) then
               !if (uc_is_rhombohedral(cell%get_par(),0.05)) then
               if (uc_is_rhombohedral(cell%get_par(),0.5)) then
                   crys = trim(cry_sys(spg%csys_code))//';R'
               !elseif (uc_is_hexagonal(cell%get_par(),0.05)) then
               elseif (uc_is_hexagonal(cell%get_par(),0.5)) then
                   crys = trim(cry_sys(spg%csys_code))//';H'
               else   ! possible problem with cell
                   if (kpr >= 0) write(kpr,'(a)') 'WARNING: AMBIGUOUS SYMMETRY!'
                   return
               endif
           else
               crys = trim(cry_sys(spg%csys_code))//';'
           endif
           chis = 0
           if (is_chiral(spg)) chis = 1
           write(fdb%handle(),'(a,i0,a,i1,a)')';'//trim(sname)//';'//trim(spg%symbol_xhm)//';', &
                 spg%num,';'//trim(infospg%hm2)//';'//strtype(strcode)//';'//crys//';',chis,';'
       endif
       call fdb%fclose()
   endif
!
   end subroutine export_for_ochemdb

!--------------------------------------------------------------------------------------------------------

   subroutine create_atom_class_filed(atom,bond,nbond,conn,charge,sname,atclass)
   USE atom_type_util
   USE connect_mod
   USE fileutil
   USE strutil
   USE arrayutil
   USE unit_cell
   USE spginfom
   USE elements
   type(atom_type), dimension(:), allocatable, intent(in)      :: atom
   type(bond_type), dimension(:), allocatable, intent(in)      :: bond
   integer, intent(in)                                         :: nbond
   type(container_type), dimension(:), allocatable, intent(in) :: conn
   integer, dimension(:), intent(in)                           :: charge
   character(len=*), intent(in)                                :: sname
   character(len=*), dimension(:), intent(in)                  :: atclass
   type(file_handle)                                           :: fdb
   integer                                                     :: i,a1,a2
!
!  Write file with: ;ID;charge1,charge2,label1,label2;conn1;class1;conn2;class2;ord;
   call fdb%fopen(trim(sname)//'d1.csv','w')
   do i=1,nbond
      a1 = bond(i)%n1
      a2 = bond(i)%n2
      write(fdb%handle(),'(a)',advance='no')';'//trim(sname)//';'    &
                       //i_to_s(charge(a1))//';'//i_to_s(charge(a2))//';'  &
                       //atom(a1)%glab()//';'//atom(a2)%glab()//';'
!
!     Write connected atoms to a1 and a2 in ascending order
      if (conn(a1)%nat > 1) then
          write(fdb%handle(),'(a)',advance='no')    &
          trim(s_get_atomlist(atom,conn(a1)%pos,conn(a1)%nat,'-',vmask=conn(a1)%pos/=a2,ncopy=2))
      endif
      write(fdb%handle(),'(a)',advance='no')';'
!corr      write(fdb%handle(),'(a)',advance='no')get_atomclass(a1,atom,bond,conn,ring)
      write(fdb%handle(),'(a)',advance='no')trim(atclass(atom(a1)%asym))
      write(fdb%handle(),'(a)',advance='no')';'
      if (conn(a2)%nat > 1) then
          write(fdb%handle(),'(a)',advance='no')    &
          trim(s_get_atomlist(atom,conn(a2)%pos,conn(a2)%nat,'-',vmask=conn(a2)%pos/=a1,ncopy=2))
      endif
      write(fdb%handle(),'(a)',advance='no')';'
!corr      write(fdb%handle(),'(a)',advance='no')get_atomclass(a2,atom,bond,conn,ring)
      write(fdb%handle(),'(a)',advance='no')trim(atclass(atom(a2)%asym))
      write(fdb%handle(),'(a)',advance='no')';'
      write(fdb%handle(),'(i0)',advance='no')bond(i)%ord
      write(fdb%handle(),'(a)',advance='yes')';'
   enddo
   call fdb%fclose()

   end subroutine create_atom_class_filed

!--------------------------------------------------------------------------------------------------------

   subroutine create_atom_class_filea(atom,bond,ang,nang,conn,charge,sname,atclass)
   USE atom_type_util
   USE connect_mod
   USE fileutil
   USE strutil
   USE arrayutil
   USE unit_cell
   USE spginfom
   USE elements
   type(atom_type), dimension(:), allocatable, intent(in)      :: atom
   type(bond_type), dimension(:), allocatable, intent(in)      :: bond
   type(angle_type), dimension(:), allocatable, intent(in)     :: ang
   integer, intent(in)                                         :: nang
   type(container_type), dimension(:), allocatable, intent(in) :: conn
   integer, dimension(:), intent(in)                           :: charge
!corr   type(container_type), dimension(:), allocatable, intent(in) :: ring
   character(len=*), dimension(:), intent(in)                  :: atclass
   character(len=*), intent(in)                                :: sname
   type(file_handle)                                           :: fdb
   integer                                                     :: i,a1,a2,a3
!
!  Write file with: ;ID;charge1;charge2;charge3;label1;label2;label3;conn1;class1;conn2;class2;conn3;class3;ord1;ord2;
   call fdb%fopen(trim(sname)//'a1.csv','w')
        
   do i=1,nang     !!!!numangles(ang)
      a1 = ang(i)%n1
      a2 = ang(i)%n2
      a3 = ang(i)%n3
      write(fdb%handle(),'(a)',advance='no')';'//trim(sname)//';'    &
                   //i_to_s(charge(a1))//';'//i_to_s(charge(a2))//';'//i_to_s(charge(a3))//';'  &
                   //atom(a1)%glab()//';'//atom(a2)%glab()//';'//atom(a3)%glab()//';'
!
!     Write connected atoms to a1 in ascending order
      if (conn(a1)%nat > 1) then
          write(fdb%handle(),'(a)',advance='no')   &
                trim(s_get_atomlist(atom,conn(a1)%pos,conn(a1)%nat,'-',vmask=conn(a1)%pos/=a2,ncopy=2))
      endif
      write(fdb%handle(),'(a)',advance='no')';'
!corr      write(fdb%handle(),'(a)',advance='no')get_atomclass(a1,atom,bond,conn,ring)
      write(fdb%handle(),'(a)',advance='no')trim(atclass(atom(a1)%asym))
      write(fdb%handle(),'(a)',advance='no')';'
!
!     Write connected atoms to a2 in ascending order
      if (conn(a2)%nat > 2) then
          write(fdb%handle(),'(a)',advance='no')    &
                trim(s_get_atomlist(atom,conn(a2)%pos,conn(a2)%nat,'-',vmask=(conn(a2)%pos/=a1 .and. conn(a2)%pos/=a3),ncopy=2))
      endif
      write(fdb%handle(),'(a)',advance='no')';'
!corr      write(fdb%handle(),'(a)',advance='no')get_atomclass(a2,atom,bond,conn,ring)
      write(fdb%handle(),'(a)',advance='no')trim(atclass(atom(a2)%asym))
      write(fdb%handle(),'(a)',advance='no')';'
!
!     Write connected atoms to a3 in ascending order
      if (conn(a3)%nat > 1) then
          write(fdb%handle(),'(a)',advance='no')    &
                trim(s_get_atomlist(atom,conn(a3)%pos,conn(a3)%nat,'-',vmask=conn(a3)%pos/=a2,ncopy=2))
      endif
      write(fdb%handle(),'(a)',advance='no')';'
!corr      write(fdb%handle(),'(a)',advance='no')get_atomclass(a3,atom,bond,conn,ring)
      write(fdb%handle(),'(a)',advance='no')trim(atclass(atom(a3)%asym))
      write(fdb%handle(),'(a)',advance='no')';'
      write(fdb%handle(),'(i0)',advance='no')bond(bond_position(bond,ang(i)%n1,ang(i)%n2))%ord
      write(fdb%handle(),'(a)',advance='no')';'
      write(fdb%handle(),'(i0)',advance='no')bond(bond_position(bond,ang(i)%n2,ang(i)%n3))%ord
      write(fdb%handle(),'(a)',advance='yes')';'
   enddo
   call fdb%fclose()

   end subroutine create_atom_class_filea

!--------------------------------------------------------------------------------------------------------

   subroutine create_atom_class_filet(atom,bond,tors,conn,charge,sname,atclass)
   USE atom_type_util
   USE connect_mod
   USE fileutil
   USE strutil
   USE arrayutil
   USE unit_cell
   USE spginfom
   USE elements
   type(atom_type), dimension(:), allocatable, intent(in)      :: atom
   type(bond_type), dimension(:), allocatable, intent(in)      :: bond
   type(torsion_type), dimension(:), allocatable, intent(in)   :: tors
   type(container_type), dimension(:), allocatable, intent(in) :: conn
   integer, dimension(:), intent(in)                           :: charge
!corr   type(container_type), dimension(:), allocatable, intent(in) :: ring
   character(len=*), intent(in)                                :: sname
   character(len=*), dimension(:), intent(in)                  :: atclass
   type(file_handle)                                           :: fdb
   integer                                                     :: i,a1,a2,a3,a4
!
!  Write file with: ;ID;charge1;charge2;charge3;charge4,label1;label2;label3;label4;conn1;class1;conn2;class2;conn3;class3;conn4;class4;ord1;ord2;ord3
   call fdb%fopen(trim(sname)//'t1.csv','w')
        
   do i=1,numtorsions(tors)
      a1 = tors(i)%n1
      a2 = tors(i)%n2
      a3 = tors(i)%n3
      a4 = tors(i)%n4
      write(fdb%handle(),'(a)',advance='no')';'//trim(sname)//';'    &
                   //i_to_s(charge(a1))//';'//i_to_s(charge(a2))//';'//i_to_s(charge(a3))//';'//i_to_s(charge(a4))//';' &
                   //atom(a1)%glab()//';'//atom(a2)%glab()//';'//atom(a3)%glab()//';'//atom(a4)%glab()//';'
!
!     Write connected atoms to a1 in ascending order
      if (conn(a1)%nat > 1) then
          write(fdb%handle(),'(a)',advance='no')   &
                trim(s_get_atomlist(atom,conn(a1)%pos,conn(a1)%nat,'-',vmask=conn(a1)%pos/=a2,ncopy=2))
      endif
      write(fdb%handle(),'(a)',advance='no')';'
      write(fdb%handle(),'(a)',advance='no')trim(atclass(atom(a1)%asym))
      write(fdb%handle(),'(a)',advance='no')';'
!
!     Write connected atoms to a2 in ascending order
      if (conn(a2)%nat > 2) then
          write(fdb%handle(),'(a)',advance='no')    &
                trim(s_get_atomlist(atom,conn(a2)%pos,conn(a2)%nat,'-',vmask=(conn(a2)%pos/=a1 .and. conn(a2)%pos/=a3),ncopy=2))
      endif
      write(fdb%handle(),'(a)',advance='no')';'
      write(fdb%handle(),'(a)',advance='no')trim(atclass(atom(a2)%asym))
      write(fdb%handle(),'(a)',advance='no')';'
!
!     Write connected atoms to a3 in ascending order
      if (conn(a3)%nat > 1) then
          write(fdb%handle(),'(a)',advance='no')    &
                trim(s_get_atomlist(atom,conn(a3)%pos,conn(a3)%nat,'-',vmask=conn(a3)%pos/=a2 .and. conn(a3)%pos/=a4,ncopy=2))
      endif
      write(fdb%handle(),'(a)',advance='no')';'
      write(fdb%handle(),'(a)',advance='no')trim(atclass(atom(a3)%asym))
      write(fdb%handle(),'(a)',advance='no')';'
!
!     Write connected atoms to a3 in ascending order
      if (conn(a4)%nat > 1) then
          write(fdb%handle(),'(a)',advance='no')    &
                trim(s_get_atomlist(atom,conn(a4)%pos,conn(a4)%nat,'-',vmask=conn(a4)%pos/=a3,ncopy=2))
      endif
      write(fdb%handle(),'(a)',advance='no')';'
      write(fdb%handle(),'(a)',advance='no')trim(atclass(atom(a4)%asym))
      write(fdb%handle(),'(a)',advance='no')';'

      write(fdb%handle(),'(i0)',advance='no')bond(bond_position(bond,tors(i)%n1,tors(i)%n2))%ord
      write(fdb%handle(),'(a)',advance='no')';'
      write(fdb%handle(),'(i0)',advance='no')bond(bond_position(bond,tors(i)%n2,tors(i)%n3))%ord
      write(fdb%handle(),'(a)',advance='no')';'
      write(fdb%handle(),'(i0)',advance='no')bond(bond_position(bond,tors(i)%n3,tors(i)%n4))%ord
      write(fdb%handle(),'(a)',advance='yes')';'
   enddo
   call fdb%fclose()

   end subroutine create_atom_class_filet

!--------------------------------------------------------------------------------------------------------

   subroutine make_atomclass(atom,nau,bonds,conn,ring,atclass)
   USE atom_basic
   USE connect_mod
   USE arrayutil
   type(atom_type), dimension(:), intent(in)                   :: atom
   integer, intent(in)                                         :: nau
   type(bond_type), dimension(:), allocatable, intent(in)      :: bonds
   type(container_type), dimension(:), allocatable, intent(in) :: conn,ring
   character(len=:), allocatable, dimension(:), intent(out)    :: atclass
   integer                                                     :: i
   integer                                                     :: maxlen
!
   maxlen = maxval(conn(:nau)%nat)
   maxlen = (maxlen*4 + 6)*maxlen
   !write(0,*)'maxlen=',maxlen,maxval(conn(:nau)%nat)
   allocate(character(maxlen)::atclass(nau))
   do i=1,nau
      atclass(i) = get_atomclass(i,atom,bonds,conn,ring)
      !write(0,'(a,a,1x,a)')'AT=',atom(i)%glab(i),trim(atclass(i))
   enddo
!
   end subroutine make_atomclass

!--------------------------------------------------------------------------------------------------------

   function get_atomclass(kat,atom,bonds,conn,ring)  result(sclass)
   USE arrayutil
   USE atom_type_util
   USE strutil
   USE nr
   USE connect_mod
   integer, intent(in)                                         :: kat
   type(atom_type), dimension(:), intent(in)                   :: atom
   type(bond_type), dimension(:), allocatable, intent(in)      :: bonds
   type(container_type), dimension(:), allocatable, intent(in) :: conn,ring
   character(len=:), allocatable                               :: sclass
   integer, dimension(:), allocatable                          :: vc,vcc
   integer                                                     :: nv,i,kpos
   character(len=:), allocatable                               :: str
   integer                                                     :: maxlen,maxl,nsvet,j,maxnumat
   character(len=:), allocatable, dimension(:)                 :: svet,svetc
   integer, dimension(:), allocatable                          :: nums,numat,iord
   logical                                                     :: kfound,arom
!
!  Evaluate if kat is in aromatic ring
   call get_containers(kat,ring,vc,nv)
   call new_array(vcc,nv)
   if (nv > 0) then
       vcc = ring(vc)%nat  ! size of rings 
       do i=1,nv
          arom = ring_is_aromatic(ring(vc(i)),bonds)
          if (arom) exit
       enddo
   else
       arom = .false.
   endif
!
   sclass = get_atom_name(atom(kat),vcc,arom)
   if (conn(kat)%nat > 0) then
!
!      allocate array svet
       maxlen = 0
       do i=1,conn(kat)%nat
          kpos = conn(kat)%pos(i)
          maxl = 2 + conn(kpos)%nat*2 + conn(kpos)%nat*2 + 2 ! estimeted len of string=specific atom+neighbours+ring+bracket for ring
          if (maxl > maxlen) maxlen=maxl
       enddo
       allocate(character(maxlen)::svet(conn(kat)%nat))
       allocate(nums(conn(kat)%nat),source=1)
       allocate(numat(conn(kat)%nat))
!
!      fill array svet and check for duplicate string
       nsvet = 0
       do i=1,conn(kat)%nat
          kpos = conn(kat)%pos(i)
          call get_containers(kpos,ring,vc,nv)
          call new_array(vcc,nv)
          if (nv > 0) then
              vcc = ring(vc)%nat
              arom = ring_is_aromatic(ring(vc(1)),bonds)
          else
              arom = .false.
          endif
          str = get_atom_name(atom(kpos),vcc,arom)//   &
                s_get_atomlist(atom,conn(kpos)%pos,conn(kpos)%nat,vmask=conn(kpos)%pos/=kat,ncopy=2)
          kfound=.false.
          do j=1,nsvet
             if(s_eqidb(str,svet(j))) then
                kfound = .true.
                nums(j) = nums(j)+1
                exit
             endif
          enddo
          if (.not.kfound) then
              nsvet = nsvet + 1
              svet(nsvet) = str
              numat(nsvet) = conn(kpos)%nat
          endif
       enddo
!
!      sort svet for number of atoms and alphabetic
       if (nsvet > 1) then
           allocate(iord(nsvet))
           allocate(character(maxlen+1)::svetc(nsvet))
           maxnumat = maxval(numat(:nsvet))
           do i=1,nsvet
              svetc(i) = char(maxnumat - numat(i))//svet(i)
              !call s_detags(svetc(i))
              !call s_s_delete(svetc(i),'[')
              !call s_s_delete(svetc(i),']')
              call s_set_delete(svetc(i),'[]')
              !write(71,*)'SV='//svetc(i)
           enddo
           call svec_sort_heap_a_index(nsvet,svetc,iord)
           deallocate(svetc)
           allocate(character(maxlen)::svetc(nsvet))
           do i=1,nsvet
              svetc(i) = svet(iord(i))
           enddo
           do i=1,nsvet
              svet(i) = svetc(i)
           enddo
           !svet(:nsvet) = svet(iord)   ! internal error with gfortran
               !write(71,*)nsvet,'IORD=',nums(iord)
               !write(71,*)nsvet,'nums=',nums
           nums(:nsvet) = nums(iord)
           !numat = numat(iord)
       endif
       !allocate(iord(nsvet))
       !call indexx(numat(:nsvet),iord(:nsvet))
       !svet(:nsvet) = svet(iord(nsvet:1:-1))
       !nums(:nsvet) = nums(iord(nsvet:1:-1))
       !numat(:nsvet) = numat(iord(nsvet:1:-1))
       !stest=['Cu','Al','B ','Hg','Ba','Rh','Sb','Ge','Ca','H ']
       !ntest=[   1,   2,   1,   2,  2,    3,   3,   2,   1,  1 ]
       !call new_array(iord,10)
       !call indexx(ntest,iord)
       !ntest = ntest(iord)
       !stest = stest(iord)
       !do i=1,10
       !   write(71,*)'TESTP=',stest(i),ntest(i)
       !enddo
       !do i=1,10
       !   stest(i) = char(maxval(ntest)-ntest(i))//stest(i)
       !enddo
       !call svec_sort_heap_a_index(10,stest,iord)
       !ntest = ntest(iord)
       !stest = stest(iord)
       !do i=1,10
       !   write(71,*)'TESTP=',stest(i),ntest(i)
       !enddo
!
       do i=1,nsvet
          if (nums(i) == 1) then
              sclass=sclass//'('//trim(svet(i))//')'
          else
              sclass=sclass//'('//trim(svet(i))//')'//i_to_s(nums(i))

          endif
       enddo
   endif
!
   end function get_atomclass

!--------------------------------------------------------------------------------------------------------

   function get_atom_name(at,vc,ar)   result(sname)
   USE strutil
   USE atom_basic
   USE arrayutil
   type(atom_type), intent(in)                    :: at
   integer, dimension(:), allocatable, intent(in) :: vc
   logical, intent(in)                            :: ar
   character(len=:), allocatable                  :: sname
   integer                                        :: nvc
!
   nvc = size_array(vc)
   if (nvc == 0) then
       sname = trim(at%spec())
   else
       if (ar) then
           sname = lower(trim(at%spec()))//'['//cat_ivet(vc,',')//']'
       else
           sname = trim(at%spec())//'['//cat_ivet(vc,',')//']'
       endif
   endif
!
   end function get_atom_name

end module ochemdb
