#include <utils.h>
#include "grafico.h"

#define PairDistributionWindow "PairDistribution"

static GtkWidget *Window = NULL;

typedef struct {
    int   rad;
    float rmax;
    float qmax;
               } pdf_option_type;

gboolean ManageExpose (GtkWidget *widget, gpointer obj, gpointer data);
void Attesa(const char *msg);
void FineAttesa(const char *msg);
pdf_option_type pdfG = { 0, 10.f, 20.f};


namespace GraphicSpace 
{
static Grafico *graf = NULL;
}               // end namespace

extern "C" void jav_pdf_(int *km, pdf_option_type *pdf);
extern "C" void jav_export_powder_pattern_(char *f1, int *tipo, size_t lung);


static int Wm = 1;
static std::string Filename(" ");
static GdkEventMask EventMask = (GdkEventMask)
                               (GDK_BUTTON_PRESS_MASK |
                                GDK_BUTTON_RELEASE_MASK |
                                GDK_BUTTON_MOTION_MASK |
                                GDK_POINTER_MOTION_MASK |
                                GDK_SCROLL_MASK );

static gboolean on_ConfigureEvent (GtkWidget *widget, GdkEventConfigure *,
                             gpointer data)
{
using namespace GraphicSpace;
    graf->Configure(widget);
    return TRUE;
}

static void on_settings(GtkWidget *button, gpointer data)
{
using namespace GraphicSpace;
   GtkWidget *father = gtk_widget_get_toplevel(button);
   int response = GPOINTER_TO_INT(data);
   if(response != 0)
   {
      GtkWidget *w = get_widget(father, "radiation");
      pdfG.rad   = combo_get_val(w) - 1;
      w = get_widget(father, "rmax");
      pdfG.rmax  = (float) spin_button_get_value(w);
      w = get_widget(father, "qmax");
      pdfG.qmax  = (float) spin_button_get_value(w);
      Attesa("Computing PDF. Please wait ...");
      ShowWaitCursor(father);
      jav_pdf_(&Wm, &pdfG);
      HideWaitCursor(father);
   }
   if(response <= 1)
   {
      GtkWidget *grandfather = get_widget(father, "father");
      g_object_set_data(G_OBJECT(grandfather), "settings_dlg", NULL);
      gtk_widget_destroy(father);
   }
}

void PDFSettings(GtkWidget *but, gpointer data)
{
using namespace GraphicSpace;
   const char *rads[] = {"X-ray","Neutron","Electron"};
   int nrads = (int) G_N_ELEMENTS(rads);

   GtkWidget *father = gtk_widget_get_toplevel(but);
   GtkWidget *dlg = get_widget(father, "settings_dlg");
   if(GTK_IS_WIDGET(dlg))
   {
      gdk_window_raise(gtk_widget_get_window(dlg));
      return;
   }
   dlg = gtk_dialog_new ();

   gtk_window_set_title (GTK_WINDOW (dlg), "PDF Settings");
   gtk_container_set_border_width(GTK_CONTAINER(dlg), 8);
   GtkWidget *vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
   gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
   gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(father));
   gtk_container_set_border_width(GTK_CONTAINER(vbox1), 8);
   g_object_set_data(G_OBJECT(dlg), "father", father);
   g_object_set_data(G_OBJECT(father), "settings_dlg", dlg);

   GtkWidget *table = table_new(3, 2, FALSE);
   gtk_box_pack_start(GTK_BOX(vbox1), table, FALSE, FALSE, 4);
   table_set_col_spacings(table, 12);
   table_set_row_spacings(table, 12);
   int k = 0;

   GtkWidget *label = LABEL_NEW("Wavelength", NULL);
   set_alignment(label, 0., 0.5);
   table_attach_exp(table, label, 0, 1, k, k+1);
   GtkWidget *combo = create_combo(NULL, rads, nrads, pdfG.rad+1, NULL,
              NULL, false);
   g_object_set_data(G_OBJECT(dlg), "radiation", combo);
   table_attach_exp(table, get_widget(combo, "parent"), 1, 2, k, k+1);
   k++;

   label = LABEL_NEW("Rmax", NULL);
   set_alignment(label, 0., 0.5);
   table_attach_exp(table, label, 0, 1, k, k+1);
   GtkWidget *spin = SPIN_NEW(0, 100, pdfG.rmax, 0.1f, 1);
   g_object_set_data(G_OBJECT(dlg), "rmax", spin);
   table_attach_exp(table, spin, 1, 2, k, k+1);
   set_width_chars(GTK_ENTRY(spin), 6);
   k++;

   label = LABEL_NEW("Qmax", NULL);
   set_alignment(label, 0., 0.5);
   table_attach_exp(table, label, 0, 1, k, k+1);
   spin = SPIN_NEW(0, 100, pdfG.qmax, 0.1f, 1);
   g_object_set_data(G_OBJECT(dlg), "qmax", spin);
   table_attach_exp(table, spin, 1, 2, k, k+1);
   set_width_chars(GTK_ENTRY(spin), 6);
   k++;

  GtkWidget *button = OK_button();
  gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_OK);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(on_settings), GINT_TO_POINTER(1));
  button = APPLY_button();
  gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_APPLY);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(on_settings), GINT_TO_POINTER(2));
  button = CLOSE_button();
  gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_CANCEL);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(on_settings), GINT_TO_POINTER(0));
  g_signal_connect_swapped(G_OBJECT(dlg), "delete-event", G_CALLBACK(gtk_button_clicked), button);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)),
      GTK_BUTTONBOX_SPREAD);
  gtk_widget_show_all(dlg);

}

void SavePDF(GtkWidget *but, gpointer data)
{
using namespace gtkext;
   GtkWidget *dlg = gtk_widget_get_toplevel(but);
   const char *filter = "ASCII powder pattern(*.xy)";
   FilselItem local_it;
   local_it = saveas(dlg, filter, Filename.c_str(), false);
   if(local_it.exitCode > 0)
   {
       int  tipo = local_it.nfilt + 1;
       char *f1 = g_build_filename(local_it.cwd, local_it.fileinp, NULL);
       jav_export_powder_pattern_(f1, &tipo, strlen(f1));
       //g_print("file %s, tipo %d\n", f1, tipo);
   }
}

static void on_reset_zoom(GtkWidget *button, GtkWidget *widget)
{
using namespace GraphicSpace;
     graf->ResetZoom(button, widget);
}

static gboolean mouse_release  (GtkWidget       *widget,
                                GdkEventButton  *event,
                                GtkWidget       *but)
{
using namespace GraphicSpace;
  graf->MouseRelease(widget, but);
  return FALSE;
}

std::string display_xy(int x, int y)
{
using namespace GraphicSpace;
  _iPoint ip = {x, y};
  char *stringa;
  float r, g;

  _Point p = graf->FromScreen(ip);
  r = p.x;
  g = p.y;
  stringa = g_strdup_printf("R: %.4f      G: %.4f", r, g);
  return stringa;
}

static gboolean mouse_move (GtkWidget *w, GdkEventMotion *event, gpointer data)
{
using namespace GraphicSpace;
  int x, y, butt, stato;
  x = (int) event->x;
  y = (int) event->y;
  stato = (int) event->state;
  butt = -1;

  if((stato & GDK_BUTTON3_MASK) == GDK_BUTTON3_MASK )
     butt = 3;
  else if ((stato & GDK_BUTTON1_MASK) == GDK_BUTTON1_MASK )
     butt = 1;

  if(butt < 0)
  {
     gtk_statusbar_push(GTK_STATUSBAR(data), 1, toUTF8(display_xy(x, y).c_str()));
     return TRUE;
  }
  else
     graf->MouseMove(w, x, y);
  return FALSE;
}

static gboolean mouse_press (GtkWidget *widget, GdkEventButton  *ev, gpointer data)
{
using namespace GraphicSpace;
  graf->SetStartMove(true);
  return FALSE; 
}

GtkWidget *create_pdf_dlg(GtkWidget *father)
{
  GtkWidget *dlg = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dlg), "Pair Distribution Function (PDF)");
  gtk_container_set_border_width(GTK_CONTAINER(dlg), 8);
  GtkWidget *vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
  gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(father));

  GtkWidget *viewer = gtk_drawing_area_new ();
  g_object_set_data(G_OBJECT(viewer), "offscreen", NULL);
  g_object_set_data(G_OBJECT(dlg), "drawarea", viewer);
  gtk_box_pack_start(GTK_BOX(vbox1), viewer, TRUE, TRUE, 2);

  gtk_widget_set_size_request(viewer, 550, 350);
  gtk_widget_add_events(viewer, EventMask);

  GtkWidget *statusbar = gtk_statusbar_new ();
  g_object_set_data (G_OBJECT (dlg), "statusbar", statusbar);
  gtk_box_pack_start(GTK_BOX(vbox1), statusbar, FALSE, FALSE, 2);

  (void) g_signal_connect (G_OBJECT (viewer), "configure_event",
                      G_CALLBACK (on_ConfigureEvent), NULL);
  (void) g_signal_connect (G_OBJECT (viewer), DRAW_EVENT,
                      G_CALLBACK (ManageExpose), NULL);
  (void) g_signal_connect (G_OBJECT (viewer), "motion_notify_event",
                      G_CALLBACK (mouse_move), statusbar);
  (void) g_signal_connect (G_OBJECT (viewer), "button_press_event",
                      G_CALLBACK (mouse_press), NULL);

  chiudi_dlg_data *st = g_new0(chiudi_dlg_data, 1);
  st->window = father;
  st->name = PairDistributionWindow;

  GtkWidget *button = myimage_button("  _Export", "save.png");
  gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_ACCEPT);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(SavePDF), dlg);
  button = myimage_button("_Settings", "preferences.png");
  gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, 1);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(PDFSettings), dlg);
  button = RESET_button();
  gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_APPLY);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(on_reset_zoom), viewer);
  gtk_widget_set_sensitive(button, FALSE);
  (void) g_signal_connect (G_OBJECT (viewer), "button_release_event",
                      G_CALLBACK (mouse_release), button);
  button = CLOSE_button();
  gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_CANCEL);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(chiudi_dlg), (gpointer) st);
  g_signal_connect_swapped(G_OBJECT(dlg), "delete-event", G_CALLBACK(gtk_button_clicked), button);
  g_object_set_data(G_OBJECT(father), PairDistributionWindow, dlg);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)),
      GTK_BUTTONBOX_SPREAD);

  gtk_widget_show_all(dlg);
  return dlg;
}

void OpenPairDistribution(GtkWidget *window)
{
   GtkWidget *dlg;
   if(g_object_get_data(G_OBJECT(window), PairDistributionWindow))
   {
      dlg = get_widget(window, PairDistributionWindow);
      gdk_window_raise(gtk_widget_get_window(dlg));
   }
   else
   {
      dlg = create_pdf_dlg(window);
      g_object_set_data(G_OBJECT(window), PairDistributionWindow, dlg);
   }
}


void PairDistribution(GtkWidget *window, int whichModel, std::string fn)
{
using namespace GraphicSpace;
    Window = window;
    Wm = whichModel;
    Filename = fn;
    Attesa("Computing PDF. Please wait ...");
    jav_pdf_(&whichModel, &pdfG);
}

extern "C" void do_pdf_(float x[], float y[], int *num)
{
using namespace GraphicSpace;
    FineAttesa("Computing PDF. Please wait ...done!");
    if(graf != NULL)
       delete graf;
    graf = new Grafico();
    std::string labelx = "R(";
    labelx.append(special_car(Aring));
    labelx.append(")");
    std::string labely = "G(";
    labely.append(special_car(Aring));
    labely.append("<sup>-2</sup>)");
    graf->SetLabels(labelx.c_str(), labely.c_str());
    graf->AddPlot(x, y, *num, 0, 0, false);
    OpenPairDistribution(Window);
    GtkWidget *dlg = get_widget(Window, PairDistributionWindow);
    GtkAllocation allocation;
    GtkWidget *wi = get_widget(dlg, "drawarea");
    gtk_widget_get_allocation(wi, &allocation);

    int w = (int)allocation.width;
    int h = (int)allocation.height;

    gpointer context = get_object(wi, "context");

    gpointer pixmap = get_object(wi, "offscreen");

    graf->SetArea(w, h);
    graf->SetContext(context, pixmap);
    graf->Aggiorna(wi);
}
