#ifdef HAVE_CONFIG_H
#  include <config.h>
#else
#  include <config_any.h>
#endif
#include "utils.h"
#include "sir_application.h"
#include "gtkdefs.h"
#ifdef HAVE_OPENGL_GL_H
#include <OpenGL/gl.h>
#elif defined (HAVE_GL_GL_H)
#include <GL/gl.h>
#elif defined(_WIN32)
#include <GL/gl.h>
#endif

extern "C" void mr_setenv_(char *string);

enum {  PROGRAM, HD, IB, PVR, GRP, WD, STRU, OUTF, PROPF, FIXF,
                USERF, MISC1, AA, PRJ1, PRJ2, PRJ3, PRJ4, PRJ5,
                CFGLINES
             }; 
//enum {  STICKS = 1, RODS, SIMPLE, STICKS_N_BALLS, BUBBLES,
//        SHADING, VAN_DER_WAALS, ELLIPSOIDS};


#define MAX_ERR ((1<<AA) -1)
#define N_AUX NUM_COLORS+N_OBJS+N_FIXED

namespace main_app
{
/* Default Palette */
static speCol obj_defs[] = { 
                            {0.0f,  0.0f, 0.0f,  1.0f},   //Black
                            {1.0f,  1.0f, 1.0f,  1.0f},   //White
                            {0.8f,  1.0f, 1.0f,  1.0f},   //GHOSTS
                            {0.6f,  0.6f, 0.6f,  1.0f},   //BONDS
                            {1.0f,  0.0f, 0.0f,  1.0f},   //LABELS
                            {0.83f, 0.83f,0.83f, 1.0f},   //CELL
//                            {0.0f,  0.0f, 0.9f,  1.0f},   //CELL
                            {0.63f, 0.0f, 0.63f, 1.0f},   //RESTRAINTS
                            {0.83f, 0.83f,0.83f, 1.0f},   //AXES
                            {0.00f, 1.00f,1.00f, 1.0f},   //HBONDS
                            {1.00f, 1.00f,0.00f, 1.0f},   //SHORTC
                            {0.00f, 1.00f,0.00f, 1.0f},   //INFOS
//                            {0.83f, 0.83f,0.83f, 1.0f},   //INFOS
                            {0.0f,  0.8f, 0.8f,  1.0f},   //MONO
                            {0.0f,  0.0f, 0.0f,  1.0f},   //BACKGROUND
//                            {0.63f, 0.63f,0.9f,  1.0f},   //BG1
                            {0.45f, 0.62f,0.81f, 1.0f},   //BG1
                            {0.04f, 0.015f,0.24f,1.0f},   //BG2
                            {1.0f,  1.0f, 1.0f,  1.0f}    //White
                           };

static speCol fix_defs[] = { 
                             {0.00f, 0.00f, 0.00f, 1.00f},
                             {1.00f, 1.00f, 1.00f, 1.00f},
                             {0.75f, 0.75f, 0.75f, 1.00f},
                             {0.50f, 0.50f, 0.50f, 1.00f},
                             {0.00f, 0.00f, 0.62f, 1.00f},
                             {0.00f, 0.62f, 0.00f, 1.00f},
                             {0.00f, 0.62f, 0.62f, 1.00f},
                             {0.62f, 0.00f, 0.00f, 1.00f},
                             {0.62f, 0.00f, 0.62f, 1.00f},
                             {0.62f, 0.62f, 0.00f, 1.00f},
                             {0.00f, 0.00f, 1.00f, 1.00f},
                             {0.00f, 1.00f, 0.00f, 1.00f},
                             {0.00f, 1.00f, 1.00f, 1.00f},
                             {1.00f, 0.00f, 0.00f, 1.00f},
                             {1.00f, 0.00f, 1.00f, 1.00f},
                             {1.00f, 1.00f, 0.00f, 1.00f}};

  Application::Application(const char *pack, const char *vers, const char *shnam,
      const char *dire, const char *ex_dire, int col, const char *about_d,
      const char *about_p)
  {
        strPackageName = pack;
        Version = vers;
        ShortName = shnam;
        PrgFiles = dire;
        ExampDir = ex_dire;
        AppError = 0;
        standalone = true;
        DefBg = col;
        AboutDlg = NULL;
        AboutDesc = about_d;
        AboutPixmap = about_p;
        strUserExamplesDir = g_get_home_dir();
        strUserExamplesDir.append(G_DIR_SEPARATOR_S);
        strUserExamplesDir.append("share");
        strUserExamplesDir.append(G_DIR_SEPARATOR_S);
        strUserExamplesDir.append(PACKAGE);
        strUserExamplesDir.append(G_DIR_SEPARATOR_S);
        strUserExamplesDir.append("examples");
//        strUserExamplesDir.append(VERSION);
        strUserExamplesDir.append(G_DIR_SEPARATOR_S);
        strUserExamplesDir.append(ExampDir);
        for(int i=0; i<N_OBJS; i++)
            memcpy(app_defs[i], obj_defs[i], 4*sizeof(float));
        for(int i=0; i<NUM_COLORS; i++)
        {
            float *col = get_gen_color(i);
            memcpy(app_defs[i+N_OBJS], col, 4*sizeof(float));
            g_free(col);
        }
        for(int i=0; i<N_FIXED; i++)
            memcpy(app_defs[i+NUM_COLORS+N_OBJS], fix_defs[i], 4*sizeof(float));
        for(int i=0; i<N_FIXED; i++)
            memcpy(fix_color[i], fix_defs[i], 4*sizeof(float));
        for(int i=0; i< 16; i++)
        {
            colorPalette[i].red   = bcolors[i].red*255;
            colorPalette[i].green = bcolors[i].green*255;
            colorPalette[i].blue  = bcolors[i].blue*255;
            colorPalette[i].pixel = 0;
        }
  }

  std::string Application::DateInfo(const char *fil)
  {
   const char *Months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
                           "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
   if(fil == NULL)
   {
      const char *ddir = g_path_get_dirname(PkgDataDir().c_str());
      //char *filinfo = g_build_filename(ddir, "files", ".instdate", NULL);
      char *filinfo = g_build_filename(ddir, ".instdate", NULL);
      return dateinfo(filinfo);
   }
   struct tm *clock;
   struct stat attrib;
   stat(fil, &attrib);
#if !__STDC_WANT_SECURE_LIB__
   clock = gmtime(&(attrib.st_mtime));  
#else
   tm pclock;
   clock = &pclock;
   (void) gmtime_s(clock,  &(attrib.st_mtime));  
#endif
   std::stringstream out;
   out << "Created on " << clock->tm_year+1900 << " "
       << Months[clock->tm_mon] << " " << clock->tm_mday;
   return out.str();

  }

  int Application::LastUpdate(GtkWidget *mw, bool ifSilent, int *versionNumber,
                 bool ifGraph)
  {
   const char *ddir = g_path_get_dirname(PkgDataDir().c_str());
   //char *filinfo = g_build_filename(ddir, "files", ".instdate", NULL);
   char *filinfo = g_build_filename(ddir, ".instdate", NULL);
   GtkWidget *w;
   std::string Mesg = "Contacting IC server, please wait...";
   w   = lookup_widget(mw, "statusbar");
   if(ifGraph)
     gtk_statusbar_push(GTK_STATUSBAR(w), 1, Mesg.c_str());
      //w = OpenContactWindow(mw);
   else
      printf("%s", Mesg.c_str());
   int result = IsUpdateAvailable(filinfo, ifSilent, versionNumber);
   if(result < 0)
      Mesg.append(" failed.");
   else
      Mesg.append(" done.");
   if(ifGraph)
     gtk_statusbar_push(GTK_STATUSBAR(w), 1, Mesg.c_str());
      //CloseContactWindow(w);
   else
      printf("%s", Mesg.c_str());
   return result;
  }

#if 0
  void Application::OpenAbout(GtkWidget *main_window)
  {
    gtkext::AboutItem it;
    GtkWidget *label;

    memset(&it, 0, sizeof(gtkext::AboutItem));
    GdkCursor *hand_cursor = create_my_cursor(HAND_CURSOR);
    char *path = g_strdup(GetProgName());
    char *abs = g_build_filename(path, g_get_prgname(), NULL);

    if (AboutDlg == NULL)
    {
      const char *str;
      if(strstr(PackageVersion(), "beta"))
         str = g_strdup_printf("%2.2d beta", MY_MINOR_VERSION(PackageVersion()));
      else
         str = g_strdup_printf("%2.2d", MY_MINOR_VERSION(PackageVersion()));
      it.text1 = g_strdup_printf(
          "<span foreground=\"blue\" size=\"xx-large\">%s </span> Version %d.%s"
          "\n%s\n%s\n%s\n\nSupport: sirmail@ic.cnr.it",
          PackageName(),  
          MY_MAJOR_VERSION(PackageVersion()),
          str, //MY_MINOR_VERSION(PackageVersion()),
          DateInfo(abs).c_str(),
          ArchInfo().c_str(),
          AboutDesc);
      it.htext1 = g_strdup_printf("http://www.ic.cnr.it/\n\n");
      it.text2  = g_strdup_printf("Graphical User Interface based on GTK libraries (V. %d.%d)",
                   GTK_MAJOR_VERSION, GTK_MINOR_VERSION);
      it.htext2 = g_strdup_printf("http://www.gtk.org");
#ifdef USE_CAIRO
      it.text3  = g_strdup_printf("Cairo libraries (V. %d.%d)",
              CAIRO_VERSION_MAJOR, CAIRO_VERSION_MINOR);
      it.htext3 = g_strdup_printf("http://www.cairographics.org");
#endif

      if(strstr(PackageName(), "Jav"))
         it.text4  = g_strdup_printf("and OpenGL libraries (V. %s)", glGetString(GL_VERSION));
      else
         it.text4  = g_strdup_printf("and OpenGL libraries ");

      it.htext4 = g_strdup_printf("http://www.opengl.org");
      if(strstr(PackageName(), "Jav"))
         it.text5  = g_strdup_printf("created by M.R. Mallamo.\n");

      it.title            = g_strdup_printf("About %s", PackageName());
      it.image            = g_strdup_printf("%s", AboutPixmap);
      it.browser          = g_strdup_printf("%s", InternetBrowser().c_str());
      AboutDlg = create_about_dialog(main_window, &it);
     }

    gtk_widget_show_all (AboutDlg);
    label = get_widget(AboutDlg, "htext1");
    gdk_window_set_cursor (gtk_widget_get_window(label), hand_cursor);
    label = get_widget(AboutDlg, "htext2");
    gdk_window_set_cursor (gtk_widget_get_window(label), hand_cursor);
#ifdef USE_CAIRO
    label = get_widget(AboutDlg, "htext3");
    gdk_window_set_cursor (gtk_widget_get_window(label), hand_cursor);
#endif
    label = get_widget(AboutDlg, "htext4");
    gdk_window_set_cursor (gtk_widget_get_window(label), hand_cursor);

    gdk_window_raise (gtk_widget_get_window(AboutDlg));
  }
#endif

  void Application::MsgWinErr(const char *cmsg, int type)
  {
#ifdef GUI_ENV
        size_t sz = strlen(cmsg) + 50;
        char *messaggio = (char *) malloc(sz*sizeof(char));
        formatta_msg(messaggio, sz, cmsg, 40);

        std::string msg = messaggio;
        type = -type;
        if(type == SEVERE_ERR_WINDOW)
           msg.append("\nProgram will be terminated.");
        if(gtk_init_check(NULL, NULL))
        {
           GtkWidget *dialog = gtk_message_dialog_new (NULL,
                                  GTK_DIALOG_DESTROY_WITH_PARENT,
                                  GTK_MESSAGE_ERROR,
                                  GTK_BUTTONS_CLOSE,
                                  "%s", msg.c_str());
           gtk_dialog_run (GTK_DIALOG (dialog));
           gtk_widget_destroy (dialog);
        }
        else
           MsgWinErr_nogtk(cmsg, type);
#else
        MsgWinErr_nogtk(cmsg, type);
#endif
  }

  bool Application::Initialize(int argc, const char *const argv[])
  {
    bool result = false;
    GtkOn = false;
    std::string composta;
    gchar *tmpstr;
/* On MacOSX it is better to look for the examples in the same directory 
   containing the ilmilione2 bundle */
#ifdef MACOSXX
    composta.assign("../../..");
    composta.append(G_DIR_SEPARATOR_S);
    composta.append("examples");
    composta.append(G_DIR_SEPARATOR_S);
    composta.append(ExampDir);
#else
    composta.assign("examples");
    composta.append(G_DIR_SEPARATOR_S);
    composta.append(ExampDir);
#endif

    Argc = argc;
    Argv = g_new0(char *, argc);
    for(int i=0; i<argc; i++)
    {
        Argv[i] = g_new0(char, MaxLenFilename);
        CopiaStringa(Argv[i], MaxLenFilename, argv[i]);
    }
    tmpstr = get_prog_dir(argv[0]);
    strMainDir = tmpstr;
    g_free(tmpstr);
    Application::SetProgName(strMainDir.c_str());
    tmpstr = get_pkg_dir(strMainDir.c_str());
    strAuxDataDir = tmpstr;
    g_free(tmpstr);

    tmpstr = get_files_dir(strMainDir.c_str(), composta.c_str());
    strExamplesDir = tmpstr;
    g_free(tmpstr);
    tmpstr = get_files_dir(strMainDir.c_str(), PrgFiles);
    strPkgDataDir = tmpstr;
    g_free(tmpstr);

    char *tmp = g_path_get_basename(argv[0]);
    char *progname = g_build_filename(strMainDir.c_str(), tmp, NULL);
    g_free(tmp);
//#ifdef _WIN32
#if 0
    std::string nome;
    tmp = g_path_get_dirname(strPkgDataDir.c_str());
    nome = tmp;
    g_free (tmp);
    nome.append(G_DIR_SEPARATOR_S);
    nome.append("files");
    nome.append(G_DIR_SEPARATOR_S);
    nome.append("syminfo.lib");
    std::string symfile = nome;
    std::string ccp4sym = "SYMINFO=" + symfile;
    mr_setenv_(g_strdup(ccp4sym.c_str()));
#endif
#ifdef _WIN32
//    _putenv(ccp4sym.c_str());
    tmp = get_ctime();
    strMtime = tmp;
    g_free(tmp);
#else
    tmp = get_mtime(progname);
    strMtime = tmp;
    g_free(tmp);
#endif
    g_free(progname);
    for(int i=0; i<NUM_COLORS; i++)
    {
        float *col = get_gen_color(i);
        memcpy(app_defs[i+N_OBJS], col, 4*sizeof(float));
        g_free(col);
    }
    if(str_white(strPackageName))
    {
       ResettaPalette();
       return true;
     }
     result = CheckConfig();
     GetExamples();
     char *ddir = g_path_get_dirname(PkgDataDir().c_str());
     //char *filxen = g_build_filename(ddir, "files", "Sir.xen", NULL);
     char *filxen = g_build_filename(ddir, "AtomProperties.xen", NULL);
     FILE *fp = fOpen(filxen, "r");
     if(!fp)
     {
        g_free(filxen);
        filxen = g_build_filename(strMainDir.c_str(), "expodir", "AtomProperties.xen", NULL);
     }
     leggixen(filxen);
     g_free(ddir);
     g_free(filxen);
     return(result);
  }
                                                                                                                  
  bool Application::Initialize()
  {
    bool result = false;
    GtkOn = false;

    const gchar *prgnam = Application::GetProgName();
    if(prgnam == NULL)
    {
       prgnam = g_get_prgname();
       strMainDir = get_prog_dir(prgnam);
    }
    else
       strMainDir = prgnam;
    strPkgDataDir = get_files_dir(strMainDir.c_str(), PrgFiles);
    strAuxDataDir = get_pkg_dir(strMainDir.c_str());

    for(int i=0; i<NUM_COLORS; i++)
    {
        float *col = get_gen_color(i);
        memcpy(app_defs[i+N_OBJS], col, 4*sizeof(float));
        g_free(col);
    }
    if(str_white(strPackageName))
    {
       ResettaPalette();
       return true;
     }
     result = CheckConfig();
     char *filxen = g_build_filename(AuxDataDir().c_str(), "AtomProperties.xen", NULL);
     FILE *fp = fOpen(filxen, "r");
     if(!fp)
     {
        g_free(filxen);
        filxen = g_build_filename(strMainDir.c_str(), "expodir", "AtomProperties.xen", NULL);
     }
     leggixen(filxen);
     g_free(filxen);

     return(result);
  }

  void Application::AddLast(const std::string &fn)
  {
    std::vector< std::string > lst;
                                                                                                                  
    if( str_white(fn.c_str()) )
      return;
                                                                                                                  
    lst.push_back(fn);
                                                                                                                  
    for( unsigned i=0; i<vstrLast.size(); i++ )
    {
      if( !str_white(vstrLast[i].c_str()) && (vstrLast[i].compare(fn) != 0) )
        lst.push_back(vstrLast[i]);
    }
    vstrLast = lst;
  }

  bool Application::CheckConfig()
  {
//#ifdef _WIN32
//    const char *arch = SystemArch();
//    char *tmp = g_strdup_printf(".%s%s", PACKAGE, arch);
//    char *tmp1 = g_build_filename(g_get_home_dir(),  tmp, NULL);
//    strConfDir = tmp1;
//    g_free(tmp1);
//#else
    char *tmp1 = g_build_filename(g_get_home_dir(), "." PACKAGE, NULL);
    strConfDir = tmp1;
    g_free(tmp1);
//#endif

      int i = 0;
      if(mr_access(strConfDir.c_str(), 'm') == 0)
         i = dir_create(strConfDir.c_str());
      if(i == -1)
      {
          std::string msg =  "Error in open configuration directory: "+strConfDir+". \n";
          msg.append("You must set the environment variable\n            "
                   "SIRCFGDIR\n to a directory with write permission!\n"
                   "(see \"Installing the program\" in Help)");
          MsgWinErr(msg.c_str(), -SEVERE_ERR_WINDOW);
          return  false;
      }

      if(mr_access(strConfDir.c_str(), 'm') == 0)
      {
       MsgWinErr("Error in open configuration directory", -SEVERE_ERR_WINDOW);
       return false;
      }
//#ifndef _WIN32
      
//      char *Licence = g_build_filename(strConfDir.c_str(), ".lic", NULL);
//      int result = AcceptLicence(Licence, strPkgDataDir);
//      g_free(Licence);
//      if(!result)
//         return false;
//#endif
      char *tmpstr = g_build_filename(strConfDir.c_str(), strPackageName, NULL);
      strConfFile = tmpstr;
      g_free(tmpstr);
      strConfFile.append("-");
      strConfFile.append(Version);
      strConfFile.append(".cfg");
// controlla l'esistenza del file di configurazione
      return ReadConfig();
  }

  bool Application::ReadConfig()
  {
    int i, errore, np, err_cod, val[15];
    char rigar[MAX_LEN+1];
    FILE *fp;
    char *pkv[2];
  
    errore= 0;
    err_cod= 0;
    np = 0;
  
    for(i=0; i<255; i++)
    {
       filesir[i] = ' ';
       filebin[i] = ' ';
       fileout[i] = ' ';
    }
    filesir[255] = '\0';
    filebin[255] = '\0';
    fileout[255] = '\0';

    fp = fOpen(strConfFile.c_str(), "r");
    if (fp == NULL)
        return CreaConfig();
    else
    {
       if((fgets(rigar,MAX_LEN,fp) != NULL) && (strlen(rigar) > 1))
       {
           (void) get_str_tokens(filtranl(rigar).c_str(), ";",2, pkv);
           if((pkv[1] == NULL) || strcmp(g_strchomp(pkv[1]), Version))
              errore = b_set(errore, err_cod);
           for(int kk=0; kk<2; kk++)
               if(pkv[kk] != NULL)
                  g_free(pkv[kk]);
       }
       else
           errore = b_set(errore, err_cod);
  
       err_cod++;
       if((fgets(rigar,MAX_LEN,fp) != NULL) && (strlen(rigar) > 1))
       {
           char *tmp = g_strdup(filtranl(rigar).c_str());
           if(strcmp(tmp, strConfDir.c_str()))
              errore = b_set(errore, 0);
           else
              strConfDir =  tmp;
       }
       else
           errore = b_set(errore,err_cod);
  
       err_cod++;
       if((fgets(rigar,MAX_LEN,fp) != NULL) && (strlen(rigar) > 1))
           strInternetBrowser =  filtranl(rigar).c_str();
       else
           errore = b_set(errore,err_cod);
  
       err_cod++;
       if((fgets(rigar,MAX_LEN,fp) != NULL) && (strlen(rigar) > 1))
           strPovrayP =  filtranl(rigar).c_str();
       else
           errore = b_set(errore,err_cod);
  
       err_cod++;
       if((fgets(rigar,MAX_LEN,fp) != NULL) && (strlen(rigar) > 1))
           strGroupsFile =  filtranl(rigar).c_str();
       else
           errore = b_set(errore,err_cod);
  
       err_cod++;
       if((fgets(rigar,MAX_LEN,fp) != NULL) && (strlen(rigar) > 1))
           strWorkingDirectory.assign(filtranl(rigar));
       else
           errore = b_set(errore,err_cod);
  
       if((fgets(rigar,MAX_LEN,fp) != NULL) && (strlen(rigar) > 1))
       {
           char filext[5];
           filext[0] = '.';
           for(i=0; i<3; i++)
               filext[i+1] = (char) tolower(strPackageName[i]);
           filext[4] = '\0';
           CopiaStringa(filesir, MaxLenFilename, filtranl(rigar).c_str());
           CopiaStringa(filebin, MaxLenFilename, filtranl(rigar).c_str());
           AppendiStringa(filesir, MaxLenFilename, filext);
           AppendiStringa(filebin, MaxLenFilename, ".bin");
       }
  
       if((fgets(rigar,MAX_LEN,fp) != NULL) && (strlen(rigar) > 1))
           ScriviStringa(fileout, MaxLenFilename, "%s", filtranl(rigar).c_str());
  
       err_cod++;
       if((fgets(rigar,MAX_LEN,fp) != NULL) && (strlen(rigar) > 1))
           strFontP =  filtranl(rigar).c_str();
       else
           errore = b_set(errore,err_cod);
  
       err_cod++;
       if((fgets(rigar,MAX_LEN,fp) != NULL) && (strlen(rigar) > 1))
           strFontN =  filtranl(rigar).c_str();
       else
           errore = b_set(errore,err_cod);
  
       err_cod++;
       if((fgets(rigar,MAX_LEN,fp) != NULL) && (strlen(rigar) > 1))
           strFontU =  filtranl(rigar).c_str();
       else
           errore = b_set(errore,err_cod);
  
       err_cod++;
       if((fgets(rigar,MAX_LEN,fp) != NULL) && (strlen(rigar) > 1))
       {
           get_int_tokens(rigar, " ", 11, val);
           PlotType    = val[0];
           Kscrx       = val[1];
           Kscry       = val[2];
           Bestv       = val[3];
           Background  = val[4];
           IfLabels    = val[5];
           IfMaximized = val[6];
           auto_update = val[7];
           IfSoftRend  = val[8];
           XStart      = val[9];
           YStart      = val[10];
//           if ((PlotType < 1) || (PlotType >= NUM_STYLES))
//                  PlotType = SIMPLE;
           if(Kscry < 1)
              Kscry = 1;
       }
       else
           errore = b_set(errore,err_cod);
  
       err_cod++;
       if((fgets(rigar,MAX_LEN,fp) != NULL) && (strlen(rigar) > 1))
       {
           get_int_tokens(rigar, " ", 4, val);
           MSSupported = val[0];
           MSEnabled   = val[1];
           NSamples    = val[2];
           nPixelFormat= val[3];
       }

       vstrLast.clear();
       for(i=1; i<=MAXPROJ; i++)
       {
           if(fgets(rigar,MAX_LEN,fp) != NULL)
           {
              if(rigar[0] != '#')
              {
                 vstrLast.push_back(filtranl(rigar).c_str());
                 ScriviStringa(RecentProj[np],  MaxLenFilename, "%s", filtranl(rigar).c_str());
                 if(strlen(RecentProj[np]) >1)
                    np++;
              }
           }
       }
       err_cod++;
       nRecentProjects = np;
       ReadPalette(fp, &errore, err_cod);
       err_cod++;
       if(fgets(rigar,MAX_LEN,fp) != NULL)
       {
           get_int_tokens(rigar, " ", 1, val);
           if(val[0] != CFGLINES)
              errore = b_set(errore, 0);
       }
       else
           errore = b_set(errore, 0);

       (void) fclose(fp);
    }
    if(b_test(errore, 0))
    {
//011211       MsgWinErr_nogtk( "Configuration file is an old version!!!\n"
//011211                        "It will be restored by the program.\n",
//011211                         -ERR_WINDOW);
       return CreaConfig();
    }
    else if(errore)
    {
//011211       MsgWinErr_nogtk( "Configuration file is corrupted!!!\n"
//011211                        "It will be restored by the program.\n",
//011211                         -ERR_WINDOW);
       return RicreaConfig(errore);
    }
    return true;
  }

  bool Application::CreaConfig(void)
  {
    int i;
    //char *nome=NULL;
    std::string grfile;
    std::string msg;
  
    grfile.append(ShortName);
    grfile.append(GREXT);
  
    if(!mr_access(strConfDir.c_str(), 'm'))
    {
      i = dir_create(strConfDir.c_str());
      if(i)
      {
         msg = "Error in open configuration directory: " + strConfDir;
         MsgWinErr(msg.c_str(), -SEVERE_ERR_WINDOW);
         return false;
      }
    }
    if(g_file_test(strConfFile.c_str(), G_FILE_TEST_EXISTS))
       (void) Cancella(strConfFile.c_str());
    {
       FILE *fp = fOpen(strConfFile.c_str(), "w");
       if(!fp)
       {
            msg = "Error in open config file: " + strConfFile;
            MsgWinErr(msg.c_str(), -SEVERE_ERR_WINDOW);
            return false;
       }
       for(i=0; i<CFGLINES+PALETTELINES+N_AUX+1;i++)
           (void) fprintf(fp,"\n");
       (void) fclose(fp);
    }
    bool result = RicreaConfig(static_cast<int>(MAX_ERR));
#if 0
    nome = g_build_filename(g_path_get_dirname(strPkgDataDir.c_str()), "files", grfile.c_str(), NULL);
    if(strcmp(ShortName, "Sir") == 0)
       mr_CopyFile(nome, strGroupsFile.c_str());
//    GetExamples();
    g_free(nome);
#endif
    return result;
  }
  
  bool Application::UpdateCfg(int nline, const char *line)
  {
     int ier;
     FILE *fp, *fp1;
     char riga1[MAX_LEN];
     gchar *ScratchFile = g_build_filename(strConfDir.c_str(), "tmp.out", NULL);
     std::string strScratchFile = ScratchFile;
     g_free(ScratchFile);
  
     if(nline == MISC1)
     {
        if((Kscry < 1) || (Kscry < 1))
           exit(-1);
     }
     (void) Cancella(strScratchFile.c_str());
     ier = rename(strConfFile.c_str(), strScratchFile.c_str());
     std::string err_msg;
     if(ier != 0)
     {
//011211        err_msg = "Error in update file: " + strConfFile;
//011211        (void) MsgWinErr("Error", err_msg.c_str(),  -SEVERE_ERR_WINDOW);
//011211        return false;
        return true;
     }
     else
     {

        fp = fOpen(strScratchFile.c_str(), "r");
        fp1 = fOpen(strConfFile.c_str(), "w");
        err_msg = "Error in open file: ";
        if (fp == NULL)
           (void) MsgWinErr((err_msg + strScratchFile).c_str(), -ERR_WINDOW);
        if (fp1 == NULL)
           (void) MsgWinErr((err_msg + strConfFile).c_str(), -ERR_WINDOW);
        if(fp && fp1)
        {
             for(ier=0; ier<nline;ier ++)
                 if(fgets(riga1,MAX_LEN,fp) != NULL) 
                    (void) fputs(riga1,fp1);
                 else
                    (void) fputs(" \n", fp1);
             if(fgets(riga1,MAX_LEN,fp) != NULL) 
                     ier = 0;
             (void) fputs(line, fp1);
             (void) fputs("\n", fp1);
  
             for(ier=0; ier<CFGLINES+PALETTELINES+N_AUX-nline-1;ier ++)
                 if(fgets(riga1,MAX_LEN,fp) != NULL)
                    (void) fputs(riga1,fp1);
                 else
                    (void) fputs(" \n", fp1);
             (void) fclose(fp);
             (void) fclose(fp1);
        }
        else
            return false;
     }
     (void) Cancella(strScratchFile.c_str());
     return true;
  }

  void Application::ScriviFonts (void)
  {
     FILE *fp;
     std::string nome = ShortName;
     nome.append("fonts.bu");
     char *strfback = g_build_filename(strConfDir.c_str(), nome.c_str(), NULL);
  
     fp = fOpen(strfback, "w");
     g_free(strfback);
     if (fp != NULL)
     {
        if(!str_white(strFontN.c_str()))
           (void) fputs(strFontN.c_str(), fp);
        else
           (void) fputs("*", fp);
        (void) fputs("\n", fp);
        if(!str_white(strFontP.c_str()))
           (void) fputs(strFontP.c_str(), fp);
        else
           (void) fputs("*", fp);
        (void) fputs("\n", fp);
        if(!str_white(strFontU.c_str()))
           (void) fputs(strFontU.c_str(), fp);
        else
           (void) fputs("*", fp);
        (void) fputs("\n", fp);
        (void) fclose(fp);
     }
  }

  void Application::LeggiFonts (void)
  {
     FILE *fp;
     
     std::string nome = ShortName;
     nome.append("fonts.bu");
     char *strfback = g_build_filename(strConfDir.c_str(), nome.c_str(), NULL);
     char riga1[MAX_LEN+1];
  
     strFontN = "*";
     strFontP = "*";
     strFontU = "*";
     fp = fOpen(strfback, "r");
     g_free(strfback);
     if (fp != NULL)
     {
         if((fgets(riga1,MAX_LEN,fp) != NULL) &&
               (strlen(riga1) > 1))
                strFontN = filtranl(riga1).c_str();
         if((fgets(riga1,MAX_LEN,fp) != NULL) &&
               (strlen(riga1) > 1))
                strFontP = filtranl(riga1).c_str();
         if((fgets(riga1,MAX_LEN,fp) != NULL) &&
               (strlen(riga1) > 1))
                strFontU = filtranl(riga1).c_str();
         (void) fclose(fp);
     }
  }

  bool Application::RicreaConfig(int errore)
  {
    int i, ibit, val[15];
    char num[4];
    std::string grfile, riga;
  
    grfile.append(ShortName);
    grfile.append(GREXT);
  
    ibit = 0;
    if(b_test(errore, ibit))                  /* program version */
    {
       riga.append(strPackageName);
       riga.append(";");
       riga.append(Version);
       if(!UpdateCfg(static_cast<int>(PROGRAM), riga.c_str()))
         return false;
    }
    ibit++;
    if(b_test(errore, ibit))                 /*  Home Directory */
    {
       if(!UpdateCfg(static_cast<int>(HD), strConfDir.c_str()))
         return false;
    }
    ibit++;
    if(b_test(errore, ibit))                 /*  internet browser */
    {
        strInternetBrowser =  GetBrowser();
        if(!UpdateCfg(static_cast<int>(IB), strInternetBrowser.c_str()))
         return false;
    }
    ibit++;
    if(b_test(errore, ibit))                 /*  PovRay */
    {
  #ifdef _WIN32
       strPovrayP = "pvengine.exe";
  #else
       strPovrayP = "/usr/bin/povray";
  #endif
       if(!UpdateCfg(static_cast<int>(PVR), strPovrayP.c_str()))
         return false;
    }
    ibit++;
    if(b_test(errore, ibit))                 /*  GroupsFile */
    {
       strGroupsFile =  g_build_filename(strConfDir.c_str(), grfile.c_str(), NULL);
       if(!UpdateCfg(static_cast<int>(GRP), strGroupsFile.c_str()))
         return false;
    }
    ibit++;
    if(b_test(errore, ibit))
    {
       char *aWorkingDirectory = (char *)malloc(MaxLenFilename*sizeof(char));
       CopiaStringa(aWorkingDirectory, MaxLenFilename, strUserExamplesDir.c_str());
       if(!UpdateCfg(static_cast<int>(WD), aWorkingDirectory))
         return false;
       strWorkingDirectory.assign(aWorkingDirectory);
       free (aWorkingDirectory);
     }
     ibit++;
     if(b_test(errore, ibit))
     {
        LeggiFonts();
        if(!UpdateFonts())
           return false;
     }
     ibit++;
     if(b_test(errore, ibit))
     {
        LeggiFonts();
        if(!UpdateFonts())
           return false;
     }
     ibit++;
     if(b_test(errore, ibit))
     {
        LeggiFonts();
        if(!UpdateFonts())
           return false;
     }
     ibit++;
     if(b_test(errore, ibit))             /* last run values */
     {
        ScriviStringa(num, sizeof(num), "%1.1d", DefBg);
        riga =  "    4  1020  713    1    ";
        riga.append(num);
        riga.append("  0  0  1 0 -99 -99");
        get_int_tokens(riga.c_str(), " ", 11, val);
        PlotType    = val[0];
        Kscrx       = val[1];
        Kscry       = val[2];
        Bestv       = val[3];
        Background  = val[4];
        IfLabels    = val[5];
        IfMaximized = val[6];
        auto_update = val[7];
        IfSoftRend  = val[8];
        XStart      = val[9];
        YStart      = val[10];
        if(!UpdateCfg(static_cast<int>(MISC1), riga.c_str()))
         return false;
     }
     ibit++;
     if(b_test(errore, ibit))             /* extra values */
     {
        riga = "    0  0  0  0";
        get_int_tokens(riga.c_str(), " ", 4, val);
        MSSupported    = val[0];
        MSEnabled      = val[1];
        NSamples       = val[2];
        nPixelFormat   = val[3];
        if(!UpdateCfg(static_cast<int>(AA), riga.c_str()))
         return false;
     }
     for(i=0; i<5; i++)
     {
         riga = "#";
         ScriviStringa(num, sizeof(num), "%1.1d", i+1);
         riga.append(num);
         if(!UpdateCfg(static_cast<int>(CFGLINES-5+i), riga.c_str()))
         return false;
     } 
    ibit++;
    if(b_test(errore, ibit))                 /* color palette */
    {
       ResettaPalette();
       if(!WritePalette())
          return false;
    }
    if(b_test(errore, 0))
    {
         ScriviStringa(num, sizeof(num), "%d", CFGLINES);
         riga.assign(num);
         if(!UpdateCfg(static_cast<int>(CFGLINES+PALETTELINES+N_AUX), riga.c_str()))
         return false;
    }

    return true;
  }

  bool Application::UpdateFonts(void)
  {
    if(str_white(strPackageName))
       return true;
     bool res = UpdateCfg(static_cast<int>(PROPF), strFontP.c_str());
     res = res && UpdateCfg(static_cast<int>(FIXF), strFontN.c_str());
     res = res && UpdateCfg(static_cast<int>(USERF), strFontU.c_str());
     ScriviFonts();
     return res;
  }

  void Application::SetFont(char c, char *fontname)
  {
    switch(c)
    {
    case 'P':
        strFontP = fontname;
        UpdateFonts();
        break;
    case 'N':
        strFontN = fontname;
        UpdateFonts();
        break;
    case 'U':
        strFontU = fontname;
        UpdateFonts();
        break;
    }
  }

  void Application::ResettaPalette()
  {
    int i;
    for(i=0; i<16; i++)
    {
        pal_colors[i].red   = bcolors[i].red;
        pal_colors[i].green = bcolors[i].green;
        pal_colors[i].blue  = bcolors[i].blue;
    }
    for(i=0; i<NUM_COLORS+N_OBJS+N_FIXED; i++)
       memcpy(app_color[i], app_defs[i], 4*sizeof(float));
    for(i=0; i<N_FIXED; i++)
       memcpy(fix_color[i], fix_defs[i], 4*sizeof(float));
    Background = DefBg;
  }
    
  bool Application::WritePalette ()
  {
    FILE *fp, *fp1;
    int ier, i;
    char riga1[MAX_LEN];
    std::string strScratchFile = g_build_filename(strConfDir.c_str(), "tmp.out", NULL);

    (void) Cancella(strScratchFile.c_str());
    ier = rename(strConfFile.c_str(), strScratchFile.c_str());
    if(ier)
       return false;
    fp1 = fOpen(strScratchFile.c_str(), "r");
    fp = fOpen(strConfFile.c_str(), "w");
    if(fp && fp1)
    {
        for(i=0;i<CFGLINES;i++)
        {
            if(fgets(riga1, MAX_LEN, fp1) != NULL)
               (void) fputs(riga1, fp);
            else
               (void) fputs("\n", fp);
        }
        for(i=0; i<16; i++)
        {
            ScriviStringa(riga1, MAX_LEN, "%d %d %d\n", pal_colors[i].red,
                    pal_colors[i].green,pal_colors[i].blue);
            fputs(riga1, fp);
        }
        for(i=0; i<NUM_COLORS+N_OBJS+N_FIXED; i++)
        {
           float col[4];
           memcpy(col, app_color[i], 4*sizeof(float));
           ScriviStringa(riga1, MAX_LEN, "%f %f %f %f\n", col[0],
             col[1], col[2], col[3]);
           fputs(riga1, fp);
        }
        ScriviStringa(riga1, MAX_LEN, "%d\n", CFGLINES);
        (void) fputs(riga1, fp);

        (void) fclose(fp);
        (void) fclose(fp1);
     }
     else
        return false;
     (void) Cancella(strScratchFile.c_str());
     return true;
  }
  
  void Application::ReadPalette(FILE *fp, int *errore, int err_cod)
  {
    int ier, rgb[3], i;
    char riga1[MAX_LEN+1];
    
    ier = 0;

    for(i=0; i<16; i++)
    {
       if(fgets(riga1,MAX_LEN,fp) != NULL) 
       {
           get_int_tokens(riga1, " ", 3, rgb);
           pal_colors[i].red   = (guint16) rgb[0];
           pal_colors[i].green = (guint16) rgb[1];
           pal_colors[i].blue  = (guint16) rgb[2];
       }
       else
       {
           ier = 1;
           pal_colors[i].red   = bcolors[i].red;
           pal_colors[i].green = bcolors[i].green;
           pal_colors[i].blue  = bcolors[i].blue;
       }
    }
    for(i=0; i<NUM_COLORS+N_OBJS+N_FIXED; i++)
    {
       if(fgets(riga1,MAX_LEN,fp) != NULL)
       {
           float col[4];
           get_float_tokens(riga1, " ", 4, col);
           memcpy(app_color[i], col, 4*sizeof(float));
       }
       else
       {
           ier = 1;
           memcpy(app_color[i], app_defs[i], 4*sizeof(float));
       }
    }

    if(ier)
       *errore = b_set(*errore, err_cod);
  }

  bool Application::UpdateMisc1()
  {
     char line[MAX_LEN];
     ScriviStringa(line, MAX_LEN, "%d %d %d %d %d %d %d %d %d  %d %d", 
              PlotType, Kscrx, Kscry, Bestv, Background, IfLabels,
              IfMaximized, auto_update, IfSoftRend, XStart, YStart);
     return UpdateCfg(static_cast<int>(MISC1), line);
  }
  
  bool Application::UpdateAA()
  {
     char line[MAX_LEN];
     ScriviStringa(line, MAX_LEN, "%d %d %d %d", MSSupported,
              MSEnabled, NSamples, nPixelFormat);
     return UpdateCfg(static_cast<int>(AA), line);
  }
  
  void Application::UpdatePS(int val)
  {
     PlotType = val;
     UpdateMisc1();
  }
  
  void Application::UpdateAutoU(int val)
  {
     auto_update = val;
     UpdateMisc1();
  }

  void Application::UpdateWD(const char *str)
  {
      strWorkingDirectory.assign(str);
      UpdateCfg(static_cast<int>(WD), strWorkingDirectory.c_str());
  }
  
  void Application::UpdateIB(char *str)
  {
      strInternetBrowser = str;
      UpdateCfg(static_cast<int>(IB), strInternetBrowser.c_str());
  }
  
  void Application::UpdatePovray(const char *str)
  {
      strPovrayP = str;
      UpdateCfg(static_cast<int>(PVR), strPovrayP.c_str());
  }
  
  void Application::UpdateGrf(char *str)
  {
      strGroupsFile =  str;
      UpdateCfg(static_cast<int>(GRP), strGroupsFile.c_str());
  }
                                                                                                         
  void Application::UpdateDime(int w, int h)
  {
     Kscrx = w;
     Kscry = h;
     UpdateMisc1();
  }

  void Application::UpdatePosition(int x, int y)
  {
     if((x < 0 ) || (y < 0))
       return;
     XStart = x;
     YStart = y;
     UpdateMisc1();
  }

  void Application::UpdateBestv(int val)
  {
     Bestv = val;
     UpdateMisc1();
  }

  void Application::UpdateRend(int val)
  {
     IfSoftRend  = val;
     UpdateMisc1();
  }

  void Application::UpdateBG(int val)
  {
     Background = val;
     UpdateMisc1();
     if(val <= 1)
     {
        float light[] = {(float) val, (float) val, (float) val};
        putfCol(BGC, light);
     }
  }
  
  void Application::UpdateLabel( int val)
  {
     IfLabels = val;
     UpdateMisc1();
  }
  
  void Application::GetExamples()
  {
      std::string ex_dir = g_get_home_dir();
      std::vector<std::string> Dirs;
      gchar *curdir = getPWD();
  
      ex_dir.append(G_DIR_SEPARATOR_S);
      ex_dir.append("share"); 
      dir_create(ex_dir.c_str());
      ex_dir.append(G_DIR_SEPARATOR_S);
      ex_dir.append(PACKAGE); 
      dir_create(ex_dir.c_str());
      ex_dir.append(G_DIR_SEPARATOR_S);
      ex_dir.append("examples");
//      ex_dir.append(VERSION);
      dir_create(ex_dir.c_str());
      ex_dir.append(G_DIR_SEPARATOR_S);
      ex_dir.append(ExampDir);
      dir_create(ex_dir.c_str());
      if(ex_dir.compare(strExamplesDir) != 0)
         gtkext::mr_CopyDir(strExamplesDir.c_str(), ex_dir.c_str());
  
      ex_dir.append(G_DIR_SEPARATOR_S);
      dir_set_current(ex_dir.c_str());
      ChangeMode(ex_dir.c_str(), tXt);
      std::vector<std::string> SirFiles;
      gtkext::clistfiles("*.sir", SirFiles);
      for(unsigned int j=0; j < SirFiles.size(); j++)
      {
          char *filnam = g_build_filename(ex_dir.c_str(), SirFiles[j].c_str(), NULL);
          std::string fn = filnam;
          ChangeMode(fn.c_str(),  rwXt);
          g_free(filnam);
      }
      dir_set_current(curdir);
      g_free(curdir);
  }
  
  GdkColor *Application::getgdkCol(int quale)
  {
      return(ToGdkColor(app_color[quale]));
  }

  float *Application::getfCol(int quale)
  {
      float *col = g_new0(float, 4);
      memcpy(col, app_color[quale], 4*sizeof(float));
      return col;
  }

  float *Application::getdefCol(int quale)
  {
      float *col = g_new0(float, 4);
      if (quale < N_OBJS)
          memcpy(col, obj_defs[quale], 4*sizeof(float));
      else if (quale < (N_OBJS+NUM_COLORS))
      {
          float *cg = get_gen_color(quale-N_OBJS);
          memcpy(col, cg, 4*sizeof(float));
          g_free(cg);
      }
      else
          memcpy(col, fix_defs[quale-N_OBJS-NUM_COLORS], 4*sizeof(float));
      return col;
  }

  void Application::putfCol(int quale, float *col)
  {
      memcpy(app_color[quale], col, 4*sizeof(float));
  }

  void my_log_handler(const gchar *, GLogLevelFlags ,
                      const gchar *message, gpointer )
  {
#ifndef NDEBUG
      g_print("%s\n", message);
#endif
  }

  int Application::InitGraphics()
  {
      PangoFontDescription *fdesc1;
      GtkSettings *settings;

      if(!gtk_init_check(&Argc, &Argv))  
      {
          g_print("GUI cannot be inizialized\n");
	  return -1;
      }
      g_log_set_handler("Gtk", G_LOG_LEVEL_WARNING, my_log_handler, NULL);
      g_log_set_handler("GLib-GObject", G_LOG_LEVEL_WARNING, my_log_handler, NULL);

#ifdef USE_CMAP
      GdkColormap *cmap = gdk_colormap_get_system();
      gtk_widget_set_default_colormap(cmap);
#endif

      settings = gtk_settings_get_default();
#if GTK_MAJOR_VERSION < 3
      PangoFontDescription *fontnew  = DefaultFont();
#ifdef USE_MY_FONT_SIZE
      pango_font_description_set_size (fontnew, MY_FONT_SIZE*PANGO_SCALE);
#endif
 
      char *fontname = pango_font_description_to_string(fontnew);
      gtk_settings_set_string_property (settings, "gtk-font-name",
                                    fontname, "XProperty");
#ifdef _WIN32
      char *nome = g_build_filename(g_path_get_dirname(MainDir().c_str()), "etc", "gtk-2.0", "gtkrc", NULL);
      gtk_rc_parse(nome);
#endif
      gchar *rc_string = g_strdup_printf("style \"user-font\" { "
                     "font_name=\"%s\" \n} \n"
                     "class \"*\" style \"user-font\"", fontname);
      gtk_rc_parse_string (rc_string);
      g_free(rc_string);
#else
//---------------- CSS -----------------
  GtkCssProvider *provider = gtk_css_provider_new ();
  GdkDisplay *display = gdk_display_get_default ();
  GdkScreen *screen = gdk_display_get_default_screen (display);
  gtk_style_context_add_provider_for_screen (screen, GTK_STYLE_PROVIDER (provider),     GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);

  gsize bytes_written, bytes_read;

  //char *nome = g_build_filename(AuxDataDir().c_str(), "files", "addon.css", NULL);
  char *nome = g_build_filename(AuxDataDir().c_str(), "addon.css", NULL);
//  g_print("css file %s\n", nome);
  if(g_file_test(nome, G_FILE_TEST_EXISTS))
  {
     GError *error = 0;
     gchar *css_file = g_filename_to_utf8(nome, strlen(nome),
                               &bytes_read, &bytes_written, &error);
     gtk_css_provider_load_from_path (provider, css_file, NULL);
     g_free(css_file);
  }
  g_free(nome);
  g_object_unref (provider);
  GtkIconTheme *icon_theme = gtk_icon_theme_get_default ();
  char *directory = g_build_filename(AuxDataDir().c_str(), "pixmaps", NULL);
  gtk_icon_theme_append_search_path (icon_theme, directory);
  //g_print("%s\n", directory);
  g_free(directory);

#endif
#ifndef NOIMAGEBUTTON
      g_type_class_unref (g_type_class_ref (GTK_TYPE_BUTTON));
      g_object_set (settings, "gtk-button-images", TRUE, NULL);
#ifdef GTK_TYPE_IMAGE_MENU_ITEM
      g_type_class_unref (g_type_class_ref (GTK_TYPE_IMAGE_MENU_ITEM));
      g_object_set (settings, "gtk-menu-images", TRUE, NULL);
#endif
#endif
      gboolean result = gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &wmenu, &hmenu);
//      gboolean result = gtk_icon_size_lookup_for_settings(settings,
//           GTK_ICON_SIZE_MENU, &wmenu, &hmenu);
      if(!result)
      {
           wmenu = 16;
           hmenu = 16;
      }
      if( (strlen(strFontN.c_str()) < 2)  ||
              (!gtkext::is_monospace(strFontN.c_str())) )
             fdesc1 = DefaultFontMono();
      else
             fdesc1 = pango_font_description_from_string (strFontN.c_str());

      char *tmp = pango_font_description_to_string(fdesc1);
      SetFont('N', tmp);
      g_free(tmp);
      pango_font_description_free(fdesc1);

      if(strlen(strFontP.c_str()) < 2)
      {
           fdesc1 = DefaultFont();
#ifdef _WIN32
           pango_font_description_set_size (fdesc1, 10*PANGO_SCALE);
           pango_font_description_set_weight(fdesc1,PANGO_WEIGHT_BOLD);
#endif
      }
      else
           fdesc1 = pango_font_description_from_string (strFontP.c_str());
      tmp = pango_font_description_to_string(fdesc1);
      SetFont('P', tmp);
      g_free(tmp);
      pango_font_description_free(fdesc1);

      fdesc1 = DefaultFont();
#ifdef USE_MY_FONT_SIZE
      pango_font_description_set_size (fdesc1, MY_FONT_SIZE*PANGO_SCALE);
#endif
      tmp = pango_font_description_to_string(fdesc1);
      SetFont('U', tmp);
      g_free(tmp);
      pango_font_description_free(fdesc1);

      char *pixmapdir = get_files_dir(strMainDir.c_str(), "pixmaps");
      add_pixmap_directory (pixmapdir);
      g_free(pixmapdir);

#if GTK_MAJOR_VERSION < 3
#ifdef _WIN32
      gtk_rc_parse_string ("style \"sir-window\" { "
                           "   bg[NORMAL] = \"#e6e6e6\"\n"
                           "   bg[INSENSITIVE] = \"#b6b6b6\"\n"
                           "   bg[PRELIGHT] = \"#f6f6f6\"\n"
                           "   fg[PRELIGHT] = \"#000000\"\n"
                           "   bg[SELECTED] = \"#f6f6f6\"\n"
                           "   bg[ACTIVE] = \"#f6f6f6\"\n"
//                         "   fg[ACTIVE] = \"#ffffff\"\n"
                           "}\n"
                           "class \"GtkWidget\" style \"sir-window\"");
#endif

      gtk_rc_parse_string ("style \"sir-window-liste\" { "
                           "   base[SELECTED] = \"#004080\"\n"
                           "   text[SELECTED] = \"#ffffff\"\n"
                           "   base[PRELIGHT] = \"#004080\"\n"
                           "   text[PRELIGHT] = \"#ffffff\"\n"
                           "   base[ACTIVE] = \"#004080\"\n"
                           "   text[ACTIVE] = \"#ffffff\"\n"
                           "}\n"
                           "class \"GtkTreeView\" style \"sir-window-liste\"");

      rc_string = g_strdup_printf("style \"user-font\" { "
                  "font_name=\"%s\" \n} \n"
                  "class \"*\" style \"user-font\"", strFontU.c_str());
      gtk_rc_parse_string (rc_string);
      g_free(rc_string);
#endif
      GtkOn = true;
      return 0;
  }
  void Application::Set_AA(int param, int val)
  {
      switch(param)
      {
      case param_SUPPORT:
           MSSupported = val;
           break;
      case param_ENABLE:
           MSEnabled = val;
           break;
      case param_BUFFERS:
           NSamples = val;
           break;
      case param_FORMAT:
           nPixelFormat = val;
           break;
      }
      UpdateAA();
  }

  char *Application::strProgName = NULL;
}
